<?
/*
*  Verify if an email address is legitimate
*  returns true for legit email address
*/

function verifyemail($email) {
     if (eregi("^[0-9a-z_]([-_.]?[0-9a-z])*@[0-9a-z][-.0-9a-z]*\\.[a-z]{2,5}[.]?$", $email, $check)) {

	  $host = substr(strstr($check[0], '@'), 1) . ".";
	  if (getmxrr($host, $mxhosts))
	       return true;

	  // THIS WILL CATCH DNSs THAT ARE NOT MX.
	  if (checkdnsrr($host, "ANY"))
	       return true;
     }
     return false;
}

/*
*  Check a password make sure its acceptable - may want to put this regex in the db to
*  it can be changed by the admin
*/

function checkpass($pass) {
     // Make sure pass is composed of letters or numbers and is between 6 and 20 chars long
     if (ereg("^([a-zA-Z0-9]){6,20}$", $pass))
	  return true;
     else
	  return false;
}

/*
*  Performs a cURL request with the blesta_url using the key and data given.
*/
function call($blesta_url, $data) {
     $postdata = array();
     $postdata['base64'] = base64_encode(serialize($data));

     // Hold the response.
     $response = "";

     $ch = curl_init();
     curl_setopt($ch, CURLOPT_URL, $blesta_url);
     curl_setopt($ch, CURLOPT_HEADER, 0);
     curl_setopt($ch, CURLOPT_POST, 1);
     curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
     curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
     curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,1);
     curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,1);

     $response = curl_exec($ch);
     curl_close($ch);

     return unserialize(base64_decode($response));
}

/*
*  Process an API request call to retrieve package info
*/
function getpackages($blesta_url, $key, $modulename=false, $instantact=true, $status="active", $formatcurrency="false") {
     $data = array();
     $data['key'] = $key;
     $data['getpackages'] = "true";
     $data['formatcurrency'] = $formatcurrency; // DEFAULT is false (i.e 123.45 format)

     if ($modulename !== false)
	  $data['modulename'] = $modulename;
     if ($instantact)
	  $data['instantact'] = "true";
     else
	  $data['instantact'] = "false";

     $data['status'] = $status;

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return $response['packages']; // return the package info

     // Die with error response
     die($response['message']);
}

/*
*  Process an API request call to retrieve a package's info
*/
function getpackage($blesta_url, $key, $pid, $coupon=false, $status="active", $formatcurrency="false", $userid=false) {
     $data = array();
     $data['key'] = $key;
     $data['getpackage'] = "true";
     $data['packageid'] = $pid;
     $data['status'] = $status;
     $data['formatcurrency'] = $formatcurrency; // DEFAULT is false (i.e 123.45 format)

     if ($coupon)
	  $data['couponcode'] = $coupon;
     if ($userid)
     	$data['userid'] = $userid;

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return $response['package']; // return the package info

     // Die with error response
     die($response['message']);
}


/*
*  Process an API request call to retrieve HTML field info from a specific module
*/
function getServiceHTMLFields($blesta_url, $key, $module, $user=true) {
     $data = array();
     $data['key'] = $key;
     $data['getservicefields'] = "true";
     $data['module'] = $module;

     if ($user)
	  $data['userfields'] = "true";
     else
	  $data['userfields'] = "false";

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return $response[$module]; // return the HTML fields info

     // Die with error response
     die($response['message']);
}


/*
*  Get coupon data
*/
function getcoupon($blesta_url, $key, $couponid=false, $code=false) {
     $data = array();
     $data['key'] = $key;
     if ($couponid) {
	  $data['getcoupon'] = "true";
	  $data['couponid'] = $couponid;
     }
     elseif ($code) {
	  $data['getcoupon'] = "true";
	  $data['couponcode'] = $code;
     }
     else
	  $data['getcoupons'] = "true";

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return (is_array($response['coupon']) ? $response['coupon'] : $response['coupons']); // return the coupon or coupons info

     // Die with error response
     die($response['message']);
}

/*
*  Get all states
*/
function getstates($blesta_url, $key) {
     $data = array();
     $data['key'] = $key;
     $data['getstates'] = "true";

     return call($blesta_url, $data);
}

/*
*  Get all countries
*/
function getcountries($blesta_url, $key) {
     $data = array();
     $data['key'] = $key;
     $data['getcountries'] = "true";

     return call($blesta_url, $data);
}

/*
*  Get all countries and states
*/
function getcountriesandstates($blesta_url, $key) {
     $data = array();
     $data['key'] = $key;
     $data['getstates'] = "true";
     $data['getcountries'] = "true";

     return call($blesta_url, $data);
}

/*
*  Makes an RPC call to module for method with data as a parameter
*/
function RPC($blesta_url, $key, $module, $method, $param) {
     $data = array();
     $data['key'] = $key;
     $data['moduleRPC'] = "true";
     $data['module'] = $module;
     $data['method'] = $method;
     $data['param'] = $param;

     return call($blesta_url, $data);
}

/*
*  Notify all admins for an order that was processed but for which the service
*  was unexpectedly not created.
*/
function send_failure_notice($blesta_url, $key, $orderinfo) {
     $data = array();
     $data['key'] = $key;
     $data['notifyadmins'] = "true";
     $data['status'] = "failure";
     $data['orderinfo'] = $orderinfo;

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return true;

     // Die with error response
     die($response['message']);
}

/*
*  Notify all admins for an order that was processed successfully.
*/
function send_success_notice($blesta_url, $key, $orderinfo) {
     $data = array();
     $data['key'] = $key;
     $data['notifyadmins'] = "true";
     $data['status'] = "success";
     $data['orderinfo'] = $orderinfo;

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return true;

     // Die with error response
     die($response['message']);
}

/*
*  Send a welcome e-mail to the user for the given service id. If no sid is given
*  a welcome e-mail containing *ALL* services will be sent.
*/
function send_welcome($blesta_url, $key, $userid, $serviceid=false) {
     $data = array();
     $data['key'] = $key;
     $data['sendwelcome'] = "true";
     $data['userid'] = $userid; // REQUIRED
     if ($serviceid)
	  $data['serviceid'] = $serviceid; // Optional

     // Make a call to the API
     $response = call($blesta_url, $data);

     if ($response['response'] == "success")
	  return true;

     // Die with error response
     die($response['message']);
}

/*
*  Get a client's information.
*/
function get_client($blesta_url, $key, $vars) {
     $data = array();
     $data['key'] = $key;
     $data['getclient'] = "true";

     $data['email'] = $vars['user']; // optional
     $data['userid'] = $vars['user']; // optional
     $data['password'] = $vars['pass']; // optional

     // Make a call to the API
     return call($blesta_url, $data);
}

/*
*  Create the account using the information given.
*/
function create_account($blesta_url, $key, $vars, $autodebit="true") {
     $data = array();
     $data['key'] = $key;
     $data['createacct'] = "true";

     // User Info
     $data['userinfo']['fname'] = $vars['fname'];   // REQUIRED
     $data['userinfo']['lname'] = $vars['lname'];   // REQUIRED
     $data['userinfo']['cname'] = $vars['cname'];   // Optional
     $data['userinfo']['address'] = $vars['address']; // REQUIRED
     $data['userinfo']['state'] = $vars['state']; // REQUIRED
     $data['userinfo']['country'] = $vars['country']; // REQUIRED
     $data['userinfo']['city'] = $vars['city']; // REQUIRED
     $data['userinfo']['zip'] = $vars['zip']; // Optional
     $data['userinfo']['email'] = $vars['email']; // REQUIRED
     $data['userinfo']['pass'] = $vars['pass']; // REQUIRED
     $data['userinfo']['refby'] = $vars['refby']; // Optional
     $data['userinfo']['autod'] = $vars['autod']; // Optional
     $data['userinfo']['suspendable'] = $vars['suspendable']; // Optional
     $data['userinfo']['autod'] = $autodebit; // Optional, true by default
     $data['userinfo']['ccno'] = $vars['ccno']; // Optional
     $data['userinfo']['ccexp'] = $vars['ccexp']; // Optional
     $data['userinfo']['refby'] = $vars['coupon']; // Optional
     $data['userinfo']['phone'] = $vars['phone']; // Optional
     $data['userinfo']['fax'] = $vars['fax']; // Optional

     // Make a call to the API
     return call($blesta_url, $data);
}

/*
*  Create the account using the information given.
*/
function create_contact($blesta_url, $key, $vars) {
     $data = array();
     $data['key'] = $key;
     $data['createcontact'] = "true";

     // User Info
     $data['contactinfo']['uid'] = $vars['uid'];   // REQUIRED
     $data['contactinfo']['fname'] = $vars['fname'];   // REQUIRED
     $data['contactinfo']['lname'] = $vars['lname'];   // REQUIRED
     $data['contactinfo']['email'] = $vars['email'];   // REQUIRED
     $data['contactinfo']['cname'] = $vars['cname'];   // Optional
     $data['contactinfo']['address'] = $vars['address']; // REQUIRED
     $data['contactinfo']['state'] = $vars['state']; // REQUIRED
     $data['contactinfo']['country'] = $vars['country']; // REQUIRED
     $data['contactinfo']['city'] = $vars['city']; // REQUIRED
     $data['contactinfo']['zip'] = $vars['zip']; // Optional
     $data['contactinfo']['phone'] = $vars['phone']; // Optional
     $data['contactinfo']['fax'] = $vars['fax']; // Optional
     $data['contactinfo']['ccno'] = $vars['ccno']; // Optional
     $data['contactinfo']['ccexp'] = $vars['ccexp']; // Optional
     $data['contactinfo']['type'] = $vars['type']; // Optional, default billing
     $data['overwrite'] = $vars['overwrite']; // Optional, default false
     $data['setasdefault'] = $vars['setasdefault']; // Optional, default false

     // Make a call to the API
     return call($blesta_url, $data);
}

/*
*  Create the service using the information given.
*/
function create_service($blesta_url, $key, $vars) {
     $data = array();
     $data['key'] = $key;
     $data['createacct'] = "true";
     $data['createservice'] = "true";
     $data['userid'] = $vars['uid'];

     // Service Info
     $data['serviceinfo']['pid'] = $vars['pid']; // REQUIRED
     $data['serviceinfo']['term'] = $vars['term']; // REQUIRED
     $data['serviceinfo']['user1'] = $vars['user1']; // Requirement defined in module
     $data['serviceinfo']['user2'] = $vars['user2']; // Requirement defined in module
     $data['serviceinfo']['pass'] = $vars['pass']; // Requirement defined in module
     $data['serviceinfo']['opt1'] = $vars['opt1']; // Requirement defined in module
     $data['serviceinfo']['opt2'] = $vars['opt2']; // Requirement defined in module
     $data['serviceinfo']['notes'] = $vars['notes']; // Optional
     $data['serviceinfo']['pending'] = $vars['pending']; // true will create as pending, false will attempt to add, leave blank to use Blesta default setting.
     $data['serviceinfo']['paid'] = $vars['paid']; // true will mark as paid (if pending).

     // If the service is added as pending, then giving the rawresponse returned from the fraudcheck will allow the anti-fraud module
     // to display this information under the pending service within Blesta.
     $data['fraudinfo'] = $vars['fraudinfo'];

     // Send the coupon if it exists so that if an invoice is created it will
     // have the correct discount
     if ($vars['coupon'])
	  $data['couponcode'] = $vars['coupon'];

     // Make a call to the API
     return call($blesta_url, $data);
}

/*
*  Get Available Payment Gateways
*/
function get_gateways($blesta_url, $key, $vars) {
     $data = array();
     $data['key'] = $key;
     $data['getgateways'] = "true";
     $data['type'] = $vars['type']; // "all", "merchant", "nonmerchant", default is "all"
     $data['orderinfo']['uid'] = $vars['uid']; // Required for nonmerchant gateways requests.
     $data['orderinfo']['amount'] = $vars['amount']; // Required for nonmerchant gateways requests.
     $data['orderinfo']['iid'] = $vars['iid']; // Optional for nonmerchant gateways requests.
     $data['orderinfo']['description'] = $vars['description']; // Optional for nonmerchant gateways requests.
     $data['orderinfo']['pending'] = $vars['pending']; // Optional for nonmerchant gateways requests.
     $data['orderinfo']['coupon'] = $vars['coupon']; // Optional for nonmerchant gateways requests.
     $data['orderinfo']['recurr'] = $vars['recurr']; // Optional recurring information (term and amount)


     // Make a call to the API
     return call($blesta_url, $data);
}

/*
*  Create the account and service using the information given.
*/
function create_account_and_service($blesta_url, $key, $vars, $autodebit="true") {
     $data = array();
     $data['key'] = $key;
     $data['createacct'] = "true";
     $data['createservice'] = "true";

     // User Info
     $data['userinfo']['fname'] = $vars['account']['fname'];   // REQUIRED
     $data['userinfo']['lname'] = $vars['account']['lname'];   // REQUIRED
     $data['userinfo']['cname'] = $vars['account']['cname'];   // Optional
     $data['userinfo']['address'] = $vars['account']['address']; // REQUIRED
     $data['userinfo']['state'] = $vars['account']['state']; // REQUIRED
     $data['userinfo']['country'] = $vars['account']['country']; // REQUIRED
     $data['userinfo']['city'] = $vars['account']['city']; // REQUIRED
     $data['userinfo']['zip'] = $vars['account']['zip']; // Optional
     $data['userinfo']['email'] = $vars['account']['email']; // REQUIRED
     $data['userinfo']['pass'] = $vars['account']['pass']; // REQUIRED
     $data['userinfo']['refby'] = $vars['account']['refby']; // Optional
     $data['userinfo']['autod'] = $vars['account']['autod']; // Optional
     $data['userinfo']['suspendable'] = $vars['account']['suspendable']; // Optional
     $data['userinfo']['autod'] = $autodebit; // Optional, true by default
     $data['userinfo']['ccno'] = $vars['account']['ccno']; // Optional
     $data['userinfo']['ccexp'] = $vars['account']['ccexp']; // Optional
     $data['userinfo']['refby'] = $vars['coupon']; // Optional
     $data['userinfo']['phone'] = $vars['account']['phone']; // Optional
     $data['userinfo']['fax'] = $vars['account']['fax']; // Optional

     // Service Info
     $data['serviceinfo']['pid'] = $vars['pid']; // REQUIRED
     $data['serviceinfo']['term'] = $vars['term']; // REQUIRED
     $data['serviceinfo']['user1'] = $vars['user1']; // Requirement defined in module
     $data['serviceinfo']['user2'] = $vars['user2']; // Requirement defined in module
     $data['serviceinfo']['pass'] = $vars['pass']; // Requirement defined in module
     $data['serviceinfo']['opt1'] = $vars['opt1']; // Requirement defined in module
     $data['serviceinfo']['opt2'] = $vars['opt2']; // Requirement defined in module
     $data['serviceinfo']['notes'] = $vars['notes']; // Optional
     $data['serviceinfo']['pending'] = $vars['pending']; // true will create as pending, false will attempt to add, leave blank to use Blesta default setting.

     // If the service is added as pending, then giving the rawresponse returned from the fraudcheck will allow the anti-fraud module
     // to display this information under the pending service within Blesta.
     $data['fraudinfo'] = $vars['fraudinfo'];

     // Send the coupon if it exists so that if an invoice is created it will
     // have the correct discount
     if ($vars['coupon'])
	  $data['couponcode'] = $vars['coupon'];

     // Make a call to the API
     return call($blesta_url, $data);
}

/*
*  Processes a payment.
*/
function process_payment($blesta_url, $key, $vars, $amount, $emailreceipt=false) {
     $data = array();
     $data['key'] = $key;
     $data['processpayment'] = "true";

     if ($emailreceipt)
	  $data['emailreceipt'] = "true";

     $data['testmode'] = $vars['testmode']; // Optional

     // User Info
     $data['userinfo']['fname'] = $vars['fname'];   // REQUIRED
     $data['userinfo']['lname'] = $vars['lname'];   // REQUIRED
     $data['userinfo']['email'] = $vars['email'];   // REQUIRED
     $data['userinfo']['address'] = $vars['address']; // REQUIRED
     $data['userinfo']['state'] = $vars['state']; // REQUIRED
     $data['userinfo']['country'] = $vars['country']; // REQUIRED
     $data['userinfo']['city'] = $vars['city']; // REQUIRED
     $data['userinfo']['zip'] = $vars['zip']; // REQUIRED

     $data['userinfo']['ccno'] = $vars['ccno']; // REQUIRED
     $data['userinfo']['ccexp'] = $vars['ccexp']; // REQUIRED
     $data['amount'] = $amount; // AMOUNT

     $data['couponcode'] = $vars['coupon']; // Optional

     // Make a call to the API
     return call($blesta_url, $data);
}


/*
*  Update the transaction with the proper user id. Also update the maillog with the correct user id as well.
*/
function update_transaction($blesta_url, $key, $userid, $transid, $mailid, $contactid=false) {
     $data = array();
     $data['key'] = $key;
     $data['updatetransaction'] = "true";

     // User Info
     $data['userid'] = $userid;   // REQUIRED
     $data['transid'] = $transid;   // REQUIRED
     $data['mailid'] = $mailid; // REQUIRED
     if ($contactid !== false)
     	$data['contactid'] = $contactid; // Optional

     // Make a call to the API
     return call($blesta_url, $data);
}

function update_pending($blesta_url, $key, $pendingid, $fraudinfo) {
     $data = array();
     $data['key'] = $key;
     $data['updatependingservice'] = "true";
     $data['pendingid'] = $pendingid;
     $data['paid'] = "true"; // DEFAULT is false
     $data['fraudinfo'] = $fraudinfo;

     // Make a call to the API
     return call($blesta_url, $data);
}

function format_currency($blesta_url, $key, $amount, $pretty="true") {
     $data = array();
     $data['key'] = $key;
     $data['formatcurrency'] = "true";
     $data['amount'] = $amount;
     $data['pretty'] = $pretty; // DEFAULT is true

     // Make a call to the API
     $response = call($blesta_url, $data);

     return $response['amount'];

}

function fraudcheck($blesta_url, $key, $input) {
     $data = array();
     $data['key'] = $key;
     $data['fraudcheck'] = "true";
     $data['input'] = $input;

     // Make a call to the API
     return call($blesta_url, $data);
}

?>