<?php

/**
 * This class fetches exchange rate information from a remote source and passes
 * that data back to the caller
 *
 */
class exchangeRates {
	private $default_currency;
	private $currencies;
	
	/**
	 * Constructs a new exchange rate object
	 *
	 * @param string $default_currency The Default Currency set in Blesta
	 * @param string $currencies An array containing all currencies available in Blesta
	 */
	public function __construct($default_currency, $currencies) {
		 // The current default currency, all currency rates should be based off this currency
		$this->default_currency = $default_currency;
		// All available currencies in Blesta, no need to fetch rates not in this list
		$this->currencies = $currencies;
	}
	
	/**
	 * Fetches all exchange rates
	 *
	 * @return array An array containing all exchange rates in the format [code] => value
	 * (e.g. ['USD'] => 1.000000)
	 */
	public function fetch() {
		$url = "http://feeds.currencysource.com/";
		
		// Initialize RSS parser
		$rss = new fetchRSS();
		
		// curl request to fetch the currency feed
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url . $this->default_currency . ".xml");
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
	
		$xml = curl_exec($ch);
		$data = $rss->parse($xml);
		
		$curr = array(); // hold all currencies and their exchange rates
		for ($i=0; $i<count($data['items']); $i++) {
			preg_match("/= ([A-Z]{3}) \((.*)\)/i", $data['items'][$i]['title'], $matches);
			$curr[$matches[1]] = number_format(str_replace(",", "", $matches[2]), 6, ".", "");
		}
		
		return $curr;
	}
}

?>