<?php
/*******************************************************************************
 @name MaxMind
 @author Cody Phillips
 @date October 10, 2007
 @version 1.0
 @notes For use with Blesta version 1.2.0+
*******************************************************************************/

class maxmind implements AntiFraudInterface {
     private static $antifraud;

     public function __construct() {
	  // Adds a new maxmind antifraud that can be added, edited, or deleted
	  maxmind::$antifraud = new AntiFraud("maxmind");

           // Adds the given fields to the database for this module if it has not already
           // been created. WARNING: if the antifraud has already been created with the name
           // given in the antifraud constructor you must delete the old antifraud before the
           // new one will be created. This may have an adverse effect on services.
           maxmind::$antifraud->addDBField("varchar(255)", "server", "MaxMind Server", "www.maxmind.com");
           maxmind::$antifraud->addDBField("varchar(255)", "key", "License Key");
           maxmind::$antifraud->addDBField("varchar(5)", "usessl", "Use SSL", "true");
           maxmind::$antifraud->addDBField("float(2,2)", "maxscore", "Max Score", "10.00");
           /*
           maxmind::$antifraud->addDBField("varchar(5)", "phonechallenge", "Enable Phone Verification", "false");
           maxmind::$antifraud->addDBField("varchar(255)", "phonelanguage", "Phone Language", "English");
           maxmind::$antifraud->addDBField("int(3)", "phoneminscore", "Minimum Score", "100");
     	  */
           maxmind::$antifraud->addDBField("varchar(5)", "rejectemails", "Reject Free Email", "false");
           maxmind::$antifraud->addDBField("varchar(5)", "rejectmismatch", "Reject Country Mismatch", "false");
           maxmind::$antifraud->addDBField("varchar(5)", "rejectcountries", "Reject High Risk Countries", "false");
           maxmind::$antifraud->addDBField("varchar(5)", "rejectproxies", "Reject Anonymous Proxies", "false");

           // Options for creating a new antifraud row
       	  // NOTE: The HTML Fields will appear in the order defined here on the antifraud setup/edit pages
           maxmind::$antifraud->addAntiFraudHTMLField("text", "MaxMind Server", "server", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "Server must be a valid domain.");
           maxmind::$antifraud->addAntiFraudHTMLField("text", "License Key", "key", "^.{2,255}$", "You must enter a License Key.");
           maxmind::$antifraud->addAntiFraudHTMLField("select", "Use SSL", "usessl");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("usessl", "Yes", "true");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("usessl", "No", "false");
           maxmind::$antifraud->addAntiFraudHTMLField("text", "Max Passing Score", "maxscore", "^[0-9]{1,2}\.[0-9]{1,2}$", "Max Score must be an decimal number (ex. 2.75).");
           /*
           maxmind::$antifraud->addAntiFraudHTMLField("select", "Enable Phone Verification", "phonechallenge");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonechallenge", "Yes", "true");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonechallenge", "No", "false");
           maxmind::$antifraud->addAntiFraudHTMLField("select", "Phone Language", "phonelanguage");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "English", "English");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "Chinese", "Chinese");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "French", "French");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "German", "German");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "Italian", "Italian");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "Japanese", "Japanese");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "Polish", "Polish");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "Russian", "Russian");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("phonelanguage", "Spanish", "Spanish");
           maxmind::$antifraud->addAntiFraudHTMLField("text", "Minimum Score to Enable Phone", "phoneminscore", "^[0-9]+", "Minimum Phone Score must be an integer.");
           */

           maxmind::$antifraud->addAntiFraudHTMLField("select", "Reject Free Email Accounts", "rejectemails");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectemails", "Yes", "true");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectemails", "No", "false");
           maxmind::$antifraud->addAntiFraudHTMLField("select", "Reject Country Mismatches", "rejectmismatch");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectmismatch", "Yes", "true");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectmismatch", "No", "false");
           maxmind::$antifraud->addAntiFraudHTMLField("select", "Reject High Risk Countries", "rejectcountries");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectcountries", "Yes", "true");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectcountries", "No", "false");
           maxmind::$antifraud->addAntiFraudHTMLField("select", "Reject Anonymous Proxies", "rejectproxies");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectproxies", "Yes", "true");
                maxmind::$antifraud->addAntiFraudHTMLSelectOption("rejectproxies", "No", "false");


	  // Creates the antifraud module, adds the antifraud to the database if the antifraud table does not
	  // already exist. WARNING: if a antifraud has already been created and you attempt to
	  // add additional fields they WILL NOT BE ADDED, and any refrence to those fields
	  // WILL RESULT IN AN ERROR.
	  maxmind::$antifraud->createAntiFraud();
     }

     // Returns the antifraud object, this function MUST be present
     public function getAntiFraud() {
	  return maxmind::$antifraud;
     }

     // Formats the raw response into a format Blesta can handle
     public function formatRawResponse($rawresponse) {
           $temp = explode(";", $rawresponse);
           for ($i=0, $j=0; $i<count($temp); $i++) {
                $temp_val = explode("=", $temp[$i]);

                if (strlen($temp_val[1]) == 0 || $temp_val[1] == "NA")
                	     continue;

                switch ($temp_val[0]) {
                      case "distance":
                      	$response[$j]['field'] = "Distance";
			$response[$j++]['value'] = $temp_val[1] . " kilometers";
                      	break;
                      case "countryMatch":
                      	$response[$j]['field'] = "Country Match";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "countryCode":
                      	$response[$j]['field'] = "Country Code";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "freeMail":
                      	$response[$j]['field'] = "Free Email";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "anonymousProxy":
                      	$response[$j]['field'] = "Anonymous Proxy";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "score":
                      	$response[$j]['field'] = "Score";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "binMatch":
                      	$response[$j]['field'] = "Bank ID Match";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "binCountry":
                      	$response[$j]['field'] = "Bank Country Match";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "err":
                      	switch ($temp_val[1]) {
				case "IP_NOT_FOUND":
                      			$response[$j]['field'] = "Error";
					$response[$j++]['value'] = "IP Not Found";
                                 	break;
				case "COUNTRY_NOT_FOUND":
                      			$response[$j]['field'] = "Error";
					$response[$j++]['value'] = "Country Not Found";
                                 	break;
				case "CITY_NOT_FOUND":
                      			$response[$j]['field'] = "Error";
                                 	$response[$j++]['value'] = "City Not Found";
                                         break;
                                 case "POSTAL_CODE_NOT_FOUND":
                      			$response[$j]['field'] = "Error";
                                 	$response[$j++]['value'] = "Zip/Postal Code Not Found";
                                         break;
                         }
                      	break;
                      case "proxyScore":
                      	$response[$j]['field'] = "Proxy Score";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "ip_region":
                      	$response[$j]['field'] = "IP Region";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "ip_city":
                      	$response[$j]['field'] = "IP City";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "ip_latitude":
                      	$response[$j]['field'] = "IP Latitude";
			$response[$j++]['value'] = $temp_val[1] . " degrees";
                      	break;
                      case "ip_longitude":
                      	$response[$j]['field'] = "IP Longitude";
			$response[$j++]['value'] = $temp_val[1] . " degrees";
                      	break;
                      case "binName":
                      	$response[$j]['field'] = "Bank Name";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "ip_isp":
                      	$response[$j]['field'] = "ISP";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "ip_org":
                      	$response[$j]['field'] = "IP Organization";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "binNameMatch":
                      	$response[$j]['field'] = "Bank Name Match";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "binPhoneMatch":
                      	$response[$j]['field'] = "Bank Phone Match";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "binPhone":
                      	$response[$j]['field'] = "Bank Phone";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "custPhoneInBillingLoc":
                      	$response[$j]['field'] = "Phone Matches Zip";
                         if ($temp_val[1] == "NotFound")
				$response[$j++]['value'] = "Unknown";
                      	break;
                      case "highRiskCountry":
                      	$response[$j]['field'] = "High Risk Country";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "queriesRemaining":
                      	$response[$j]['field'] = "Queries Left";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "cityPostalMatch":
                      	$response[$j]['field'] = "Zip/PostalMatch";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "shipCityPostalMatch":
                      	$response[$j]['field'] = "Shipping City Match";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "maxmindID":
                      	$response[$j]['field'] = "MaxMind ID";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "carderEmail":
                      	$response[$j]['field'] = "High Risk Email";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "riskScore":
                      	$response[$j]['field'] = "Risk Score";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                      case "explanation":
                      	$response[$j]['field'] = "MaxMind Explanation";
			$response[$j++]['value'] = $temp_val[1];
                      	break;
                }
           }
           return $response;
     }


     // Returns true if the raw response data passes the checks, false otherwise
     public function evaluate($input) {
           $info = maxmind::$antifraud->getRow(); // Retrieve all maxmind info.

           $response = array();

           $temp = explode(";", $input);
           for ($i=0; $i<count($temp); $i++) {
           	$temp_val = explode("=", $temp[$i]);
                 $response[$temp_val[0]] = $temp_val[1];
           }

           // Analyize the response and based upon the settings for this antifraud module as defined by the
           // database fields
           if ($info['maxscore'] < $response['score'])
           	return false;
           if ($info['rejectemails'] == "true" && $response['freeMail'] == "Yes")
           	return false;
           if ($info['rejectmismatch'] == "true" && $response['countryMatch'] == "No")
           	return false;
           if ($info['rejectcountries'] == "true" && $response['highRiskCountry'] == "Yes")
           	return false;
           if ($info['rejectproxies'] == "true" && $response['anonymousProxy'] == "Yes")
           	return false;

           return true;
     }

     // Returns an array of information containing ['pass'] = true/false, ['message'], ['data'] containing the raw response data
     public function validate($input) {
           $info = maxmind::$antifraud->getRow(); // Retrieve all maxmind info.

           // MaxMind Details (Required)
           $uri = "app/ccv2r"; // Path to minFraud method

           // License Key (Required)
           $data['license_key'] = $info['key'];

           // Client Info (Required)
           $data['i'] = $input['ip']; // the client ip address
           $data['city'] = $input['city']; // the billing city
           $data['region'] = $input['state']; // the billing state
           $data['postal'] = $input['zip']; // the billing zip code
           $data['country'] = $input['country']; // the billing country

           // Recommended fields
           $email_domain = explode("@", strtolower($input['email']));
           $data['domain'] = $email_domain[1]; // Email domain
           $data['bin'] = $input['ccno6'];  // bank identification number (first 6 of Credit Card)
           //$data['forwardedIP'] = "24.24.24.25"; // X-Forwarded-For or Client-IP HTTP Header
           $data['emailMD5'] = md5(strtolower($input['email'])); // md5 email address
           //$data['usernameMD5'] = md5(strtolower("MyUser"));        // md5 username checks high risk users
           //$data['passwordMD5'] = md5(strtolower("MyPass"));        // md5 password checks high risk passwords

           // Optional fields
           //$data['binName'] = "MBNA America Bank"; // bank name
           //$data['binPhone'] = "800-421-2110"; // bank customer service phone number on back of credit card
           //$data['custPhone'] = "212-242"; // Area-code and local prefix of customer phone number
           //$data['requested_type'] = "premium"; // Which level (free, city, premium) of CCFD to use
           //$data['txnID'] = "1234"; // Transaction ID
           //$data['sessionID'] = $input['phpsessid']; // Session ID


           // Start cURL request
           $ch = curl_init();
           curl_setopt($ch, CURLOPT_URL, ($info['usessl'] == "true" ? "https://" : "http://") . $info['server'] . "/" . $uri);
           curl_setopt($ch, CURLOPT_HEADER, 0);
           curl_setopt($ch, CURLOPT_POST, 1);
           curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
           curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
           curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
           curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
           curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
           curl_setopt($ch, CURLOPT_TIMEOUT, 15);
           
           // Log data sent to maxmind
           maxmind::$antifraud->logData($data, "input", true);
           
           $rawresponse = curl_exec($ch);
           
           // Log response from maxmind
           maxmind::$antifraud->logData($rawresponse, "output", true);

           curl_close($ch);

           $response = array();

           $temp = explode(";", $rawresponse);
           for ($i=0; $i<count($temp); $i++) {
           	$temp_val = explode("=", $temp[$i]);
                 $response[$temp_val[0]] = $temp_val[1];
           }

           // Analyize the response and based upon the settings for this antifraud module as defined by the
           // database fields, determine whether to return true or false.
           $pass = true;
           if ($info['maxscore'] < $response['score']) {
           	$pass = false;
                 $reason = "Exceeded Max Passing Score of " . $info['maxscore'] . ";";
           }

           if ($info['rejectemails'] == "true" && $response['freeMail'] == "Yes") {
           	$pass = false;
                 $reason .= "Free Email Address;";
           }

           if ($info['rejectmismatch'] == "true" && $response['countryMatch'] == "No") {
           	$pass = false;
                 $reason .= "Country Mismatch;";
           }

           if ($info['rejectcountries'] == "true" && $response['highRiskCountry'] == "Yes") {
           	$pass = false;
                 $reason .= "High Risk Country;";
           }

           if ($info['rejectproxies'] == "true" && $response['anonymousProxy'] == "Yes") {
           	$pass = false;
                 $reason .= "Anonymous Proxy;";
           }

           $validation = array();
           $validation['pass'] = $pass; // true/false
           $validation['reason'] = $reason; // reason for failure (if any)
           $validation['rawresponse'] = $rawresponse; // raw response from maxmind

           return $validation;
     }


     /* ------------------------------------------------------------------------------------------------
       The following are support functions for this antifraud module
     --------------------------------------------------------------------------------------------------*/

}
?>