<?php
/*******************************************************************************
 @name AuthorizeNet
 @type Merchant Gateway
 @author Cody Phillips/Paul Phillips
 @date May 7, 2009
 @version 1.3
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

/*
*  Does not support shipping information
*
*  Notable data within array returned:
*  - x_response_code 1=approved, 2=declined, 3=error, null=no response
*  - x_response_reason_text = Response from gateway (text)
*  - x_trans_id = Transaction ID
*  - x_amount = amount of charge - good to use this so that its 100% accurate with gateway
*/

class authnet extends Gateway {

	private $authnet_url = "https://secure.authorize.net/gateway/transact.dll";
	private $curl = "/usr/bin/curl";
	private $username;
	private $key;
	private $testmode = "True";
	private $currency;
	private $vars;
	private $data;
	private static $settingFields = array("username" => "Login ID", "key" => "Transaction Key", "testmode" => "Test Mode");
	
	// Note: Authorize.net supports the following currency codes.  They may also support many others not listed here.
	// for a complete list see Authorize.net's Advanced Integration Method Implrementation Guide Appendix I.
	private static $supportedCurrencyCodes = array("AUD", "CAD", "EUR", "GBP", "HKD", "JPY",
		"MXN", "RUB", "SEK", "USD");

	function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
		$this->testmode = ($gatewayinfo['testmode']['value'] == "True" ? "TRUE" : "FALSE");
		//$this->currency = (payment::$supportedCurrencyCodes[$gatewayinfo['currency']] == 1 ? $gatewayinfo['currency'] : "");
		$this->currency = (in_array($gatewayinfo['currency'], self::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "");
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		/*
		*  Make sure no coma's in vars as they can cause things returned from authnet to be wrong
		*  since we use coma's as the delimiter. Dont want the results to jump fields.
		*/
		$this->vars[$name] = $value;
	}

	public function processCard() {

		/*
		*  Start building the data var, start with Authorize.net Login Info
		*/
	
		$data = "x_login=" . $this->username . "&";
		$data .= "x_password=" . $this->key . "&";
	
		/*
		*  Contact Information
		*/
	
		if ($this->vars['fname']) $data .= "x_first_name=" . $this->vars['fname'] . "&";
		if ($this->vars['lname']) $data .= "x_last_name=" . $this->vars['lname'] . "&";
		if ($this->vars['phone']) $data .= "x_phone=" . $this->vars['phone'] . "&";
		if ($this->vars['fax']) $data .= "x_fax=" . $this->vars['fax'] . "&";
		if ($this->vars['address']) $data .= "x_address=" . $this->vars['address'] . "&";
		if ($this->vars['city']) $data .= "x_city=" . $this->vars['city'] . "&";
		if ($this->vars['state']) $data .= "x_state=" . $this->vars['state'] . "&";
		if ($this->vars['zip']) $data .= "x_zip=" . $this->vars['zip'] . "&";
		if ($this->vars['country']) $data .= "x_country=" . $this->vars['country'] . "&";
		if ($this->vars['company']) $data .= "x_company=" . $this->vars['company'] . "&";
		if ($this->vars['email']) $data .= "x_email=" . $this->vars['email'] . "&";

		/*
		*  Order Information
		*/
	
		if ($this->vars['ccno']) $data .= "x_card_num=" . $this->vars['ccno'] . "&";
		if ($this->vars['ccexp']) $data .= "x_exp_date=" . $this->vars['ccexp'] . "&";
		if ($this->vars['amount']) $data .= "x_amount=" . $this->vars['amount'] . "&";
		if ($this->vars['cvv2']) $data .= "x_card_code=" . $this->vars['cvv2'] . "&";
		if ($this->vars['authcode']) $data .= "x_auth_code=" . $this->vars['authcode'] . "&";
		if ($this->vars['cid']) $data .= "x_cust_id=" . $this->vars['cid'] . "&";
		if ($this->vars['iid']) $data .= "x_invoice_num=" . $this->vars['iid'] . "&";
		if ($this->vars['description']) $data .= "x_description=" . $this->vars['description'] . "&";
		if ($this->vars['xid']) $data .= "x_trans_id=" . $this->vars['xid'] . "&";


		/*
		*  Authorizenet Configuration Defaults
		*/
		if (strlen($this->currency) > 0)
			$data .= "x_currency_code=" . $this->currency . "&";
	
		if ($this->vars['merchantemail']) $data .= "x_merchant_email=" . $this->vars['merchantemail'] . "&";

		$data .= "x_adc_delim_data=true&";
		$data .= "x_adc_delim_character=,&";
		$data .= "x_adc_url=false&";
		$data .= "x_version=3.0&";
		$data .= "x_method=CC&";
		$data .= "x_delim_date=TRUE&x_delim_char=|&";

		// AUTH_CAPTURE, AUTH_ONLY, CAPTURE_ONLY, CREDIT, VOID, PRIOR_AUTH_CAPTURE
		$data .= "x_type=AUTH_CAPTURE&";

		/*
		*  Change this to False to NOT send an email from authorize.net to customer
		*  We will send the customer an e-mail if we wish to
		*/
	
		$data .= "x_email_customer=False&";
		$data .= "x_test_request=" . $this->testmode;

		// Replaces any spaces with + for URL formatting
		//$data = ereg_replace(" ","+",$data);
		$data = preg_replace("/ /","+",$data);

		/*
		*  Attempt to use PHP embedded CURL functions first. If they do not exist,
		*  then we will try using a curl binary on the server.
		*/
		
		// Log what we send to Authorize.net
		$this->logData("authnet", str_replace($this->vars['ccno'], "xxxx", $data), "input", true);
			
		if (function_exists("curl_init")) {
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $this->authnet_url);
			curl_setopt($ch, CURLOPT_HEADER, 0);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
   
			$response = curl_exec($ch);
			$authnet_array = explode("|", $response);

			// Log what we received from Authorize.net
			$this->logData("authnet", $response, "output", ($authnet_array[0] == 1 ? true : false));

			curl_close($ch);
		}
		else {
			// Try to run it with curl from the command line
			exec("$curl -d \"$data\" $authnet_url", $authnet_array);
			$authnet_array=explode("|",$authnet_array[0]);
		}

		// Takes the results from authorizenet and builds an array with the correct key value pairs
		$authnet_results=array(
		"x_response_code"=> $authnet_array[0],
		"x_response_subcode"=> $authnet_array[1],
		"x_response_reason_code"=> $authnet_array[2],
		"x_response_reason_text"=> $authnet_array[3],
		"x_auth_code"=> $authnet_array[4],
		"x_avs_code"=> $authnet_array[5],
		"x_trans_id"=> $authnet_array[6],
		"x_invoice_num"=> $authnet_array[7],
		"x_description"=> $authnet_array[8],
		"x_amount"=> $authnet_array[9],
		"x_method"=> $authnet_array[10],
		"x_type"=> $authnet_array[11],
		"x_cust_id"=> $authnet_array[12],
		"x_first_name"=> $authnet_array[13],
		"x_last_name"=> $authnet_array[14],
		"x_company"=> $authnet_array[15],
		"x_address"=> $authnet_array[16],
		"x_city"=> $authnet_array[17],
		"x_state"=> $authnet_array[18],
		"x_zip"=> $authnet_array[19],
		"x_country"=> $authnet_array[20],
		"x_phone"=> $authnet_array[21],
		"x_fax"=> $authnet_array[22],
		"x_email"=> $authnet_array[23],
		"x_ship_to_first_name"=> $authnet_array[24],
		"x_ship_to_last_name"=> $authnet_array[25],
		"x_ship_to_company"=> $authnet_array[26],
		"x_ship_to_address"=> $authnet_array[27],
		"x_ship_to_city"=> $authnet_array[28],
		"x_ship_to_state"=> $authnet_array[29],
		"x_ship_to_zip"=> $authnet_array[30],
		"x_ship_to_country"=> $authnet_array[31],
		"x_tax"=> $authnet_array[32],
		"x_duty"=> $authnet_array[33],
		"x_freight"=> $authnet_array[34],
		"x_tax_exempt"=> $authnet_array[35],
		"x_po_num"=> $authnet_array[36],
		"x_md5_hash"=> $authnet_array[37],
		"x_cvv2_resp_code"=> $authnet_array[38]);

		return $authnet_results;
	}
}


/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
MasterCard 5424000000000015
*/


?>