<?php
/*******************************************************************************
 @name Chargeback Guardian
 @type Merchant Gateway
 @author Cody Phillips
 @date May 27, 2009
 @version 1.0
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class chargebackguardian extends Gateway {

	private $cbguardian_url = "https://secure.chargebackguardiangateway.com/api/transact.php";
	private $username;
	private $key;
	private $vars;
	private static $settingFields = array("username" => "Username", "key" => "Password");
	
	private static $supportedCurrencyCodes = array("USD");

	function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		/*
		*  Make sure no coma's in vars as they can cause things returned from cbguardian to be wrong
		*  since we use coma's as the delimiter. Dont want the results to jump fields.
		*/
		$this->vars[$name] = $value;
	}

	public function processCard() {

		$fields = array(
			"type" => "sale",
			"username" => $this->username,
			"password" => $this->key,
			"ccnumber" => $this->vars['ccno'],
			"ccexp" => $this->vars['ccexp'],
			"amount" => $this->vars['amount'],
			"firstname" => $this->vars['fname'],
			"lastname" => $this->vars['lname'],
			"company" => $this->vars['company'],
			"address1" => $this->vars['address'],
			"address2" => $this->vars['address2'],
			"city" => $this->vars['city'],
			"state" => $this->vars['state'],
			"zip" => $this->vars['zip'],
			"country" => $this->vars['country'],
			"phone" => $this->vars['phone'],
			"fax" => $this->vars['fax']
		);
		
		// Log what we send to Authorize.net
		$this->logData("chargebackguardian", str_replace($this->vars['ccno'], "xxxx", $fields), "input", true);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->cbguardian_url);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

		$response = curl_exec($ch);
		curl_close($ch);
		
		// return the parsed response
		return $this->parseResponse($response, $fields);
	}
	
	private function parseResponse($response, $fields) {
		parse_str($response, $out);
		
		// Takes the results from protx and builds an array with the correct key value pairs
		$results=array(
		"x_response_code"=> ($out['response'] == "2" ? 2 : ($out['response'] == "1" ? 1 : 3)),
		"x_response_reason_text"=> $out['responsetext'],
		"x_auth_code"=> $out['authcode'],
		"x_trans_id"=> $out['transactionid']);
		
		// Log what we receive from Chargeback Guadian
		$this->logData("chargebackguardian", $response, "output", ($results['x_response_code'] == 1 ? true : false));
		
		return $results;
	}
}


/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
Visa 4111111111111111
MasterCard 5431111111111111
DiscoverCard 6011601160116611
American Express 341111111111111
Credit Card Expiration: 10/10
Amount > 1.00

- To cause a declined message, pass an amount less than 1.00.
- To trigger a fatal error message, pass an invalid card number.
- To simulate an AVS Match, pass 888 in the address1 field, 77777 for zip.
- To simulate a CVV Match, pass 999 in the cvv field.
*/


?>