<?php
/*******************************************************************************
 @name EPSsecurenet
 @type Merchant Gateway
 @author Mark Munoz Mac Matters  http://www.macmatters.net
 @date May 7, 2009
 @version 1.2
 @notes For use with Blesta version 2.2.0+
        This gateway updated by Cody Phillips.
*******************************************************************************/
class eps extends Gateway {

	private $eps_url = "https://www.epssecurenet.com/egate/api_securegate.php";
	private $username;
	private $transType = "SALE";
	private $vars;
	private static $settingFields = array("username" => "EPS Unique ID Number");

	private static $supportedCurrencyCodes = array("USD");

	/*
	*  Initializes a EPS gateway
	*/
	public function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];

	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* processCard()
	*
	* Populate the POST query, then send it, and store
	* the response, and finally parse the response.
	*/
	public function processCard() {
		/* fill in the fields */
		$fields = array (
			'TranType' => $this->transType,
			'IDNum' => $this->username,
			'TranAmount' => $this->vars['amount'],
			'InvoiceNbr' => ($this->vars['iid'] > 0 ? $this->vars['iid'] : "Account Payment"),
			'CardHolderName' => $this->vars['fname'] . " " . $this->vars['lname'],
			'CardNumber' => $this->vars['ccno'],
			'ExpiryDate' => $this->vars['ccexp'],
			'AddressLine1' => $this->vars['address'],
			'AddressLine2' => $this->vars['address2'],
			'Bill_City' => $this->vars['city'],
			'Bill_State' => $this->vars['state'],
			'ZipCode' => $this->vars['zip'],
			'Bill_Country' => $this->vars['country'],
			'Bill_Phone' => $this->vars['phone'],
			'Bill_Email' => $this->vars['email'],
			'responseMeth' => "STRING",
			'str_delim_char' => ",",
			'RecureBill' => $this->vars['recure'],
			'CVV2Code' => $this->vars['cvv2']
		);


		// perform the transaction
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $this->eps_url); // Set the URL
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));
	  
		//Log the transaction data sent to EPS
		$this->logData("eps", str_replace($this->vars['ccno'], "xxxx", $fields), "input", true);

		$response = curl_exec($ch);
		curl_close($ch);

		// return the parsed response
		return $this->parseResponse($response);

	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		$response = explode(",", $response);
		$response[0] = trim($response[0]);
		//parse response
		$eps_results=array(
			//this line takes the response from EPS and formats it for Blesta
			"x_response_code" => ($response[0] == "Approved" ? 1 : ($response[0] == "Declined" ? 2 : 3)),
			"x_response_reason_text" => $response[5],
			"x_trans_id" => $response[8]);
		
		//This logs the output from EPS.
		$this->logData("eps", $response, "output", ($response[0] == "Approved" ? true : false));
		
		return $eps_results;
	}    
}
?>