<?php
/*******************************************************************************
 @name Innovative Gateway
 @type Merchant Gateway
 @author Cody Phillips
 @date April 23, 2009
 @version 1.0
 @notes For use with Blesta version 1.4.0+
 
 For testing use Username: gatewaytest Password: GateTest2002
 Set testmode to "true" to always return successful transactions
 
 Before attempting real transactions you must ensure that your server's IP
 address has been whitelisted by innovative gateway.
*******************************************************************************/

class innovative extends Gateway {
	private $innovative_url = "https://transactions.innovativegateway.com/servlet/com.gateway.aai.Aai";

	private $username;
	private $password;
	private $testmode;
	private $transType = "sale";
	private $vars;
	private static $settingFields = array("username" => "Username", "key" => "Password", "testmode" => "Test Mode");
	
	// Note: innovative supports the following currency codes.  They may also support many others not listed here.
	// for a complete list see the innovative web site.
	private static $supportedCurrencyCodes = array("USD");


	/*
	*  Initializes a innovative gateway
	*/
	public function __construct($gatewayinfo) {
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->testmode = true;
			
		$this->username = $gatewayinfo['username']['value'];
		$this->password = $gatewayinfo['key']['value'];
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* processCard()
	*
	* Populate the POST query, then send it, and store
	* the response, and finally parse the response.
	*/
	public function processCard() {
		$fields = array();
		$fields["target_app"] = "WebCharge_v5.06";
		$fields["response_mode"] = "simple";
		$fields["response_fmt"] = "delimited";
		$fields["upg_auth"] = "zxcvlkjh";
		$fields["delimited_fmt_field_delimiter"] = "=";
		$fields["delimited_fmt_include_fields"] = "true";
		$fields["delimited_fmt_value_delimiter"] = "|";
		
		$fields["username"] = $this->username;
		$fields["pw"] = $this->password;

		// Allowable Transaction Types:
		// Options:  preauth, postauth, sale, credit, void	
		$fields["trantype"] = $this->transType;
		
		$fields["reference"] = "";  // Blank for new sales...
		// required for VOID, POSTAUTH, and CREDITS.
		// Will be original Approval value.
		
		$fields["trans_id"] = "";   // Blank for new sales...
		// required for VOID, POSTAUTH, and CREDITS.
		// Will be original ANATRANSID value.
		
		$fields["authamount"] = ""; // Only valid for POSTAUTH and 
		// is equal to the original 
		// preauth amount.
		
		if ($this->testmode)
			$fields["test_override_errors"] = "true";
			
		$fields["cardtype"] = $this->getCardType($this->vars['ccno']);
		
		// Credit Card information
		$fields["ccnumber"] = $this->vars['ccno']; 
		$fields["month"] = substr($this->vars['ccexp'], 0, 2);
		$fields["year"] =  substr($this->vars['ccexp'], 2, 2);
		$fields["ccidentifier1"] = ""; // CVV2
		$fields["fulltotal"] = $this->vars['amount'];
		$fields["baddress"] = $this->vars['address'];
		$fields["baddress1"] = $this->vars['address2'];
		$fields["bcity"] = $this->vars['city'];
		$fields["bstate"] = $this->vars['state'];
		$fields["bcountry"] = $this->vars['country'];
		$fields['bzip'] = $this->vars['zip'];
		$fields['ccname'] = $this->vars['fname'] . " " . $this->vars['lname'];
		$fields['bphone'] = $this->vars['phone'];
		$fields['email'] = "name@domain";
		
		$fields['ReceiptEmail'] = "no";


		// perform the transaction
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $this->innovative_url); // Set the URL
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));

		// Log what we receive from OptimalPayments
		$this->logData("innovative", str_replace($this->vars['ccno'], "xxxx", http_build_query($fields)), "input", true);
		
		$response = curl_exec($ch);
		
		curl_close($ch);
		
		// return the parsed response
		return $this->parseResponse($response, $fields);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response, $fields) {

		$out = explode("|", $response);

		for ($i=0; $i<count($out); $i++) {
			$temp = explode("=", $out[$i]);

			$result[$temp[0]] = $temp[1];
		}

		// Takes the results from innovative and builds an array with the correct key value pairs
		$innovative_results=array(
		"x_response_code"=> ($result['approval'] != "" ? 1 : ($result['error'] != "" ? 2 : 3)),
		"x_response_reason_text"=> strip_tags($result['error']),
		"x_auth_code"=> $result['approval'],
		"x_trans_id"=> $result['anatransid']);

		// Log what we receive from OptimalPayments
		$this->logData("innovative", str_replace($this->vars['ccno'], "xxxx", $response), "output", ($result['approval'] != "" ? true : false));
		
		return $innovative_results;
	}

	/***
	* getCardType()
	*
	* This method returns the card type associated with the number, if it can
	* be deteremined, otherwise false is returned.
	*/
	private function getCardType($cardnum) {
		$pattern[0] = "/^([4]{1})([0-9]{12,15})$/"; // Visa
		$pattern[1] = "/^([51|52|53|54|55]{2})([0-9]{14})$/"; // Mastercard
		$pattern[2] = "/^([30|36|38]{2})([0-9]{12})$/"; // Diner's Club
		$pattern[3] = "/^([34|37]{2})([0-9]{13})$/"; // American Express
		$pattern[4] = "/^(3[0-9]{15})$|^(2131|1800[0-9]{11})$/"; // JCB
		$pattern[5] = "/^6(011|5[0-9]{2})[0-9]{12}$/"; // Discover
		
		$returnval = array("visa", "mc", "diners", "amex", "jcb", "discover");
		
		for ($i=0; $i<count($pattern); $i++) {
			if (preg_match($pattern[$i], $cardnum))
				return $returnval[$i];
		}
		
		return false;
	}

}

?>