<?php
/*******************************************************************************
 @name LinkPoint
 @type Merchant Gateway
 @author Cody Phillips
 @date May 7, 2009
 @version 1.1
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class linkpoint extends Gateway {
	private $linkpoint_url;
	private $curl = "/usr/bin/curl";
	private $username;
	private $key;
	private $testmode = "TEST";
	private static $port = "1129";
	private $vars;
	private static $settingFields = array("username" => "Account Number", "key" => "Path to PEM File", "testmode" => "Test Mode");
	
	private static $supportedCurrencyCodes = array("USD");


	/*
	*  Initializes the gateway
	*/
	public function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
		$this->testmode = ($gatewayinfo['testmode']['value'] == "True" ? true : false);
		
		$this->linkpoint_url = "https://" . ($this->testmode ? "staging.linkpt.net" : "secure.linkpt.net") . ":" . self::$port . "/LSGSXML";
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/*
	* Process the card
	*/
	public function processCard() {
		$fields = array (                   
			'keyfile' => $this->key,  // PEM file location
			'configfile' => $this->username,  // Account number 
			'ordertype' => "SALE",
			'result' => "LIVE", // For testing in a live environment set to GOOD, DECLINE, or DUPLICATE. Set to LIVE to make live.
			'transactionorigin' => "ECI",
			'terminaltype' => "UNSPECIFIED",     
			'chargetotal' => $this->vars['amount'],    
			'name' => $this->vars['fname'] . " " . $this->vars['lname'],
			'cardnumber' => $this->vars['ccno'],
			'cardexpmonth' => substr($this->vars['ccexp'], 0, 2),
			'cardexpyear' => substr($this->vars['ccexp'], 2, 2),
			'cvmindicator' => "not_provided",
			'cvmvalue' => "",
			'address1' => $this->vars['address'],
			'address2' => $this->vars['address2'],
			'city' => $this->vars['city'],
			'state' => $this->vars['state'],
			'addrnum' => substr($this->vars['address'], 0, strpos($this->vars['address'], " ")), 
			'zip' => $this->vars['zip'],
			'country' => $this->vars['country'],
			'phone' => $this->vars['phone']
		);

		$xml = $this->buildXML($fields);

		$ch = curl_init ();
		curl_setopt($ch, CURLOPT_URL, $this->linkpoint_url);
		curl_setopt($ch, CURLOPT_POST, 1); 
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		curl_setopt($ch, CURLOPT_SSLCERT, $this->key);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);          

		// Log what we send to LinkPoint
		$this->logData("linkpoint", str_replace($this->vars['ccno'], "xxxx", $xml), "input", true); // (only available in 1.4+)

		$response = curl_exec($ch);
		curl_close($ch);

		// Invalid, or no response
		if (strlen($response) < 2)
			$response = "<r_approved>FAILURE</r_approved><r_error>Could not connect.</r_error>"; 
		return $this->processResponse($this->decodeXML($response));
	}

	/***
	* processResponse()
	*
	* This method will process the response parameter values.
	*/
	private function processResponse($response) {

		// Takes the results from bluepay and builds an array with the correct key value pairs
		$linkpoint_results=array(
		"x_response_code"=> ($response['r_approved'] == 'FAILURE' ? 3 : ($response['r_approved'] == "APPROVED" ? 1 : 2)),
		"x_response_reason_text"=> $response['r_message'],
		"x_trans_id"=> $response['r_ref']);

		// Log what we receive from Bluepay
		$this->logData("linkpoint", $response, "output", ($response['r_approved'] == "APPROVED" ? true : false));           
		
		return $linkpoint_results;
	}
    
	private function decodeXML($xmlstg) {
		preg_match_all ("/<(.*?)>(.*?)\</", $xmlstg, $out, PREG_SET_ORDER);
		
		for ($n=0; isset($out[$n]); $n++)
			$retarr[$out[$n][1]] = strip_tags($out[$n][0]);

		return $retarr;
	}     
     
	private function buildXML($pdata) {
		### ORDEROPTIONS NODE ###
		$xml = "<order><orderoptions>";

		if (isset($pdata["ordertype"]))
			$xml .= "<ordertype>" . $pdata["ordertype"] . "</ordertype>";

		if (isset($pdata["result"]))
			$xml .= "<result>" . $pdata["result"] . "</result>";

		$xml .= "</orderoptions>";


		### CREDITCARD NODE ###
		$xml .= "<creditcard>";

		if (isset($pdata["cardnumber"]))
			$xml .= "<cardnumber>" . $pdata["cardnumber"] . "</cardnumber>";

		if (isset($pdata["cardexpmonth"]))
			$xml .= "<cardexpmonth>" . $pdata["cardexpmonth"] . "</cardexpmonth>";

		if (isset($pdata["cardexpyear"]))
			$xml .= "<cardexpyear>" . $pdata["cardexpyear"] . "</cardexpyear>";

		if (isset($pdata["cvmvalue"]))
			$xml .= "<cvmvalue>" . $pdata["cvmvalue"] . "</cvmvalue>";

		if (isset($pdata["cvmindicator"]))
			$xml .= "<cvmindicator>" . $pdata["cvmindicator"] . "</cvmindicator>";

		if (isset($pdata["track"]))
			$xml .= "<track>" . $pdata["track"] . "</track>";

		$xml .= "</creditcard>";


		### BILLING NODE ###
		$xml .= "<billing>";

		if (isset($pdata["name"]))
			$xml .= "<name>" . $pdata["name"] . "</name>";

		if (isset($pdata["company"]))
			$xml .= "<company>" . $pdata["company"] . "</company>";

		if (isset($pdata["address1"]))
			$xml .= "<address1>" . $pdata["address1"] . "</address1>";
		elseif (isset($pdata["address"]))
			$xml .= "<address1>" . $pdata["address"] . "</address1>";

		if (isset($pdata["address2"]))
			$xml .= "<address2>" . $pdata["address2"] . "</address2>";

		if (isset($pdata["city"]))
			$xml .= "<city>" . $pdata["city"] . "</city>";

		if (isset($pdata["state"]))
			$xml .= "<state>" . $pdata["state"] . "</state>";

		if (isset($pdata["zip"]))
			$xml .= "<zip>" . $pdata["zip"] . "</zip>";

		if (isset($pdata["country"]))
			$xml .= "<country>" . $pdata["country"] . "</country>";

		if (isset($pdata["userid"]))
			$xml .= "<userid>" . $pdata["userid"] . "</userid>";

		if (isset($pdata["email"]))
			$xml .= "<email>" . $pdata["email"] . "</email>";

		if (isset($pdata["phone"]))
			$xml .= "<phone>" . $pdata["phone"] . "</phone>";

		if (isset($pdata["fax"]))
			$xml .= "<fax>" . $pdata["fax"] . "</fax>";

		if (isset($pdata["addrnum"]))
			$xml .= "<addrnum>" . $pdata["addrnum"] . "</addrnum>";

		$xml .= "</billing>";


		## SHIPPING NODE ##
		$xml .= "<shipping>";

		if (isset($pdata["sname"]))
			$xml .= "<name>" . $pdata["sname"] . "</name>";

		if (isset($pdata["saddress1"]))
			$xml .= "<address1>" . $pdata["saddress1"] . "</address1>";

		if (isset($pdata["saddress2"]))
			$xml .= "<address2>" . $pdata["saddress2"] . "</address2>";

		if (isset($pdata["scity"]))
			$xml .= "<city>" . $pdata["scity"] . "</city>";

		if (isset($pdata["sstate"]))
			$xml .= "<state>" . $pdata["sstate"] . "</state>";
		elseif (isset($pdata["state"]))
			$xml .= "<state>" . $pdata["sstate"] . "</state>";

		if (isset($pdata["szip"]))
			$xml .= "<zip>" . $pdata["szip"] . "</zip>";
		elseif (isset($pdata["sip"]))
			$xml .= "<zip>" . $pdata["zip"] . "</zip>";

		if (isset($pdata["scountry"]))
			$xml .= "<country>" . $pdata["scountry"] . "</country>";

		if (isset($pdata["scarrier"]))
			$xml .= "<carrier>" . $pdata["scarrier"] . "</carrier>";

		if (isset($pdata["sitems"]))
			$xml .= "<items>" . $pdata["sitems"] . "</items>";

		if (isset($pdata["sweight"]))
			$xml .= "<weight>" . $pdata["sweight"] . "</weight>";

		if (isset($pdata["stotal"]))
			$xml .= "<total>" . $pdata["stotal"] . "</total>";

		$xml .= "</shipping>";


		### TRANSACTIONDETAILS NODE ###
		$xml .= "<transactiondetails>";

		if (isset($pdata["oid"]))
			$xml .= "<oid>" . $pdata["oid"] . "</oid>";

		if (isset($pdata["ponumber"]))
			$xml .= "<ponumber>" . $pdata["ponumber"] . "</ponumber>";

		if (isset($pdata["recurring"]))
			$xml .= "<recurring>" . $pdata["recurring"] . "</recurring>";

		if (isset($pdata["taxexempt"]))
			$xml .= "<taxexempt>" . $pdata["taxexempt"] . "</taxexempt>";

		if (isset($pdata["terminaltype"]))
			$xml .= "<terminaltype>" . $pdata["terminaltype"] . "</terminaltype>";

		if (isset($pdata["ip"]))
			$xml .= "<ip>" . $pdata["ip"] . "</ip>";

		if (isset($pdata["reference_number"]))
			$xml .= "<reference_number>" . $pdata["reference_number"] . "</reference_number>";

		if (isset($pdata["transactionorigin"]))
			$xml .= "<transactionorigin>" . $pdata["transactionorigin"] . "</transactionorigin>";

		if (isset($pdata["tdate"]))
			$xml .= "<tdate>" . $pdata["tdate"] . "</tdate>";

		$xml .= "</transactiondetails>";


		### MERCHANTINFO NODE ###
		$xml .= "<merchantinfo>";

		if (isset($pdata["configfile"]))
			$xml .= "<configfile>" . $pdata["configfile"] . "</configfile>";

		if (isset($pdata["keyfile"]))
			$xml .= "<keyfile>" . $pdata["keyfile"] . "</keyfile>";

		if (isset($pdata["host"]))
			$xml .= "<host>" . $pdata["host"] . "</host>";

		if (isset($pdata["port"]))
			$xml .= "<port>" . $pdata["port"] . "</port>";

		if (isset($pdata["appname"]))
			$xml .= "<appname>" . $pdata["appname"] . "</appname>";

		$xml .= "</merchantinfo>";

		### PAYMENT NODE ###
		$xml .= "<payment>";

		if (isset($pdata["chargetotal"]))
			$xml .= "<chargetotal>" . $pdata["chargetotal"] . "</chargetotal>";

		if (isset($pdata["tax"]))
			$xml .= "<tax>" . $pdata["tax"] . "</tax>";

		if (isset($pdata["vattax"]))
			$xml .= "<vattax>" . $pdata["vattax"] . "</vattax>";

		if (isset($pdata["shipping"]))
			$xml .= "<shipping>" . $pdata["shipping"] . "</shipping>";

		if (isset($pdata["subtotal"]))
			$xml .= "<subtotal>" . $pdata["subtotal"] . "</subtotal>";

		$xml .= "</payment>";


		### CHECK NODE ### 


		if (isset($pdata["voidcheck"]))
			$xml .= "<telecheck><void>1</void></telecheck>";
		elseif (isset($pdata["routing"])) {
			$xml .= "<telecheck>";
			$xml .= "<routing>" . $pdata["routing"] . "</routing>";

			if (isset($pdata["account"]))
				$xml .= "<account>" . $pdata["account"] . "</account>";

			if (isset($pdata["bankname"]))
				$xml .= "<bankname>" . $pdata["bankname"] . "</bankname>";

			if (isset($pdata["bankstate"]))
				$xml .= "<bankstate>" . $pdata["bankstate"] . "</bankstate>";

			if (isset($pdata["ssn"]))
				$xml .= "<ssn>" . $pdata["ssn"] . "</ssn>";

			if (isset($pdata["dl"]))
				$xml .= "<dl>" . $pdata["dl"] . "</dl>";

			if (isset($pdata["dlstate"]))
				$xml .= "<dlstate>" . $pdata["dlstate"] . "</dlstate>";

			if (isset($pdata["checknumber"]))
				$xml .= "<checknumber>" . $pdata["checknumber"] . "</checknumber>";

			if (isset($pdata["accounttype"]))
				$xml .= "<accounttype>" . $pdata["accounttype"] . "</accounttype>";

			$xml .= "</telecheck>";
		}


		### PERIODIC NODE ###

		if (isset($pdata["startdate"])) {
			$xml .= "<periodic>";

			$xml .= "<startdate>" . $pdata["startdate"] . "</startdate>";

			if (isset($pdata["installments"]))
				$xml .= "<installments>" . $pdata["installments"] . "</installments>";

			if (isset($pdata["threshold"]))
				$xml .= "<threshold>" . $pdata["threshold"] . "</threshold>";

			if (isset($pdata["periodicity"]))
				$xml .= "<periodicity>" . $pdata["periodicity"] . "</periodicity>";

			if (isset($pdata["pbcomments"]))
				$xml .= "<comments>" . $pdata["pbcomments"] . "</comments>";

			if (isset($pdata["action"]))
				$xml .= "<action>" . $pdata["action"] . "</action>";

			$xml .= "</periodic>";
		}


		### NOTES NODE ###

		if (isset($pdata["comments"]) || isset($pdata["referred"])) {
			$xml .= "<notes>";

			if (isset($pdata["comments"]))
				$xml .= "<comments>" . $pdata["comments"] . "</comments>";

			if (isset($pdata["referred"]))
				$xml .= "<referred>" . $pdata["referred"] . "</referred>";

			$xml .= "</notes>";
		}


		while (list($key, $val) = each($pdata)) {
			if (is_array($val)) {
				$otag = 0;
				$ostag = 0;
				$items_array = $val;
				$xml .= "<items>";

				while(list($key1, $val1) = each ($items_array)) {
					$xml .= "<item>";

					while (list($key2, $val2) = each ($val1)) {
						if (!is_array($val2))
							$xml .= "<$key2>$val2</$key2>";
						else {
							if (!$ostag) {
								$xml .= "<options>";
								$ostag = 1;
							}

							$xml .= "<option>";
							$otag = 1;
							
							while (list($key3, $val3) = each ($val2))
								$xml .= "<$key3>$val3</$key3>";
						}

						if ($otag) {
							$xml .= "</option>";
							$otag = 0;
						}
					}

					if ($ostag) {
						$xml .= "</options>";
						$ostag = 0;
					}
					$xml .= "</item>";
				}
				$xml .= "</items>";
			}
		}

		$xml .= "</order>";

		return $xml;
	}
}


/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
VISA                4007 0000 0002 7
                    4111 1111 1111 1111
MASTERCARD          5454 5454 5454 5454
American Express    3434 3434 3434 343
Discover            6011 6011 6011 6611
JCB                 3088 0000 0000 0009
Diner's Club        3000 0000 0000 04
*/
?>