<?php
/*******************************************************************************
 @name Optimal Payments FirePay
 @type Merchant Gateway
 @author Cody Phillips
 @date May 7, 2009
 @version 1.2
 @notes For use with Blesta version 2.2.0+
        Optimal Payments supports only the currency code that was used to create
        the account with them.
*******************************************************************************/

class optimalpayments extends Gateway {
	private $firepay_url = "https://realtime.firepay.com/servlet/DPServlet";
	private $firepay_url_test = "https://realtime.test.firepay.com:443/servlet/DPServlet";
	private $account;
	private $password;
	private $merchantID;
	private $opteration = "P";
	private $vars;
	private static $clientVersion = "1.1";
	private static $settingFields = array("username" => "Account Number", "opt1" => "Merchant ID", 
	   "key" => "Merchant Password", "testmode" => "Test Mode");
	
	private static $supportedCurrencyCodes = array("AUD", "CAD", "EUR", "GBP", "HKD", "JPY", "USD");

	/*
	*  Initializes a optimal payments gateway
	*/
	public function __construct($gatewayinfo) {
		$this->account = $gatewayinfo['username']['value'];
		$this->password = $gatewayinfo['key']['value'];
		$this->merchantID = $gatewayinfo['opt1']['value'];

		if ($gatewayinfo['testmode']['value'] == "True")
			$this->firepay_url = $this->firepay_url_test;
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* processCard()
	*
	* Populate the POST query, then send it, and parse the response.                                                            
	*/
	public function processCard() {
		/* fill in the fields */
		$fields = array (
		'account' => $this->account,
		'merchantId' => $this->merchantID,
		'merchantPwd' => $this->password,
		'merchantTxn' => mt_rand(100000, 999999),
		'clientVersion' => self::$clientVersion,
		'operation' => $this->opteration,
		'amount' => number_format(100*$this->vars['amount'], 0, ".", ""), // Amount in cents         
		'custName1' => $this->vars['fname'],
		'custName2' => $this->vars['lname'],
		'streetAddr' => $this->vars['address'],
		'streetAddr2' => $this->vars['address2'],
		'phone' => $this->vars['phone'],
		'email' => $this->vars['email'],
		'city' => $this->vars['city'],
		'province' => $this->vars['state'],
		'zip' => $this->vars['zip'],
		'country' => $this->vars['country'],
		'cardNumber' => $this->vars['ccno'],
		'cardExp' => substr($this->vars['ccexp'], 0, 2) . "/" . substr($this->vars['ccexp'], 2, 2),         
		'cardType' => $this->getCardType($this->vars['ccno']),
		'cvdIndicator' => "0",
		#'issueNumber' => "", // Can use only for Solo of Switch cards -- not supported by Blesta
		);
	
		// perform the transaction
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $this->firepay_url);          
		curl_setopt($ch, CURLOPT_HEADER, 0); 
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));         
		
		// Log what we send to OptimalPayments
		$this->logData("optimalpayments", str_replace($this->vars['ccno'], "xxxx", http_build_query($fields)), "input", true);          
		
		$response = curl_exec($ch);         
		curl_close($ch);
		
		
		// return the parsed response
		return $this->parseResponse($response);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {         
		// Parse response string
		$temp = explode('&', urldecode($response));
		for ($i=0; $i<count($temp); $i++) {
			$temp_sub = explode('=', $temp[$i]);
			$result[$temp_sub[0]] = $temp_sub[1];
		}
		
		// Takes the results from optimalpayments and build an array with the correct key value pairs
		$firepay_results=array(
		"x_response_code"=> ($result['status'] == "E" ? 3 : ($result['status'] == "SP" ? 1 : 2)),
		"x_response_reason_text"=> $result['errString'],
		"x_auth_code"=> $result['authCode'],
		"x_trans_id"=> $result['txnNumber']);
		
		// Log what we receive from OptimalPayments
		$this->logData("optimalpayments", $response, "output", ($result['status'] == "SP" ? true : false));
		
		return $firepay_results;
	}

	/***
	* getCardType()
	*
	* This method returns the card type associated with the number, if it can
	* be deteremined, otherwise false is returned.
	*/
	private function getCardType($cardnum) {
		$pattern[0] = "/^([4]{1})([0-9]{12,15})$/"; // Visa
		$pattern[1] = "/^([51|52|53|54|55]{2})([0-9]{14})$/"; // Mastercard
		$pattern[2] = "/^(5020|5038|5641|6759)([0-9]{12})$/"; // Maestro
		$pattern[3] = "/^([30|36|38]{2})([0-9]{12})$/"; // Diner's Club
		$pattern[4] = "/^([34|37]{2})([0-9]{13})$/"; // American Express
		$pattern[5] = "/^(3[0-9]{15})$|^(2131|1800[0-9]{11})$/"; // JCB
		$pattern[6] = "/^6334[5-9]([0-9]{11}|[0-9]{13,14})|^6767([0-9]{12}|[0-9]{14,15})$/"; // Solo
		
		$returnval = array("VI", "MC", "MD", "DC", "AM", "JC", "SO");
		
		for ($i=0; $i<count($pattern); $i++) {
			if (preg_match($pattern[$i], $cardnum))
				return $returnval[$i];
		}
	
		return false;
	}
}



/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
Card Type	Card Number    		Issue	CV2	Address	PostCode
Visa Credit	4929 0000 0000 	6		123	88	412
MasterCard 	5404 0000 0000  0001		123	88	412
Visa Debit 	4462 0000 0000  0003		123	88	412
Solo		6334 9000 0000  0005		123	88	412
UK Maestro	5641 8200 0000  0005 	01	123	88	412
American Express 3742 000000 00004		123	88	412
Visa Electron	4917 3000 0000 0008		123	88	412
JCB		3569 9900 0000 0009		123	88	412
Diners Club	3600 0000 0000 08		123	88	412
*/
?>