<?php
/*******************************************************************************
 @name Payflow Pro
 @type Merchant Gateway
 @author Cody Phillips
 @date Sep 21, 2009  
 @version 1.5
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class payflow extends Gateway {
	private $payflow_url = "https://payflowpro.paypal.com/"; // Live mode;
	private $payflow_url_test = "https://pilot-payflowpro.paypal.com/"; // Test mode;
	private $curl = "/usr/bin/curl";
	private $partner = "PayPal";
	private $username;
	private $user;
	private $key;
	private $testmode = "True";
	private $currency;
	private $vars;
	private $data;
	private static $settingFields = array("username" => "Login ID", "key" => "Password", "opt1" => "User", "opt2" => "Partner", "testmode" => "Test Mode");

	// Note: Payflow Pro supports the following currency codes.  They may also support many others not listed here.
	// for a complete list see Payflow Pro's Payflow Pro Developer's Guide.
	private static $supportedCurrencyCodes = array("AUD", "CAD", "EUR", "GBP", "JPY", "USD");

	function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->payflow_url = $this->payflow_url_test;

		$this->currency = (in_array($gatewayinfo['currency'], self::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "");

		// By default the merchant account has no additional users, so we set the login id as the user.
		// Otherwise we specify the user for which transactions will be processed under.
		if (strlen($gatewayinfo['opt1']['value']) == 0)
			$this->user = $this->username;
		else
			$this->user = $gatewayinfo['opt1']['value'];
		
		// Set the partner value if given, otherwise defaults to "PayPal"
		if (strlen($gatewayinfo['opt2']['value']) != 0)
			$this->partner = $gatewayinfo['opt2']['value'];
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}

	private function getHeaders() {
		$headers = array();
		$headers[0] = "Content-Type: text/namevalue"; //or maybe text/xml
		$headers[1] = "X-VPS-Timeout: 60";
		$headers[2] = "X-VPS-VIT-OS-Name: Linux";  // Name of your OS
		$headers[3] = "X-VPS-VIT-OS-Version: RHEL 4";  // OS Version
		$headers[4] = "X-VPS-VIT-Client-Type: PHP/cURL";  // What you are using
		$headers[5] = "X-VPS-VIT-Client-Version: 0.01";  // For your info
		$headers[6] = "X-VPS-VIT-Client-Architecture: x86";  // For your info
		$headers[7] = "X-VPS-VIT-Client-Certification-Id: " . '5kd93dksie' . ""; // Paypal is removing this requirement... just ignore it
		$headers[8] = "X-VPS-VIT-Integration-Product: Blesta";  // For your info, would populate with application name
		$headers[9] = "X-VPS-VIT-Integration-Version: 1.0"; // Application version

		return $headers;
	}

	public function processCard() {
		/*
		*  Start building the data var, start with Payflow Login Info
		*/
		$data = "CERT_OVERRIDE=ON" . "&";
		$data .= "VENDOR=" . $this->username . "&";
		$data .= "PARTNER=" . $this->partner . "&";
		$data .= "PWD=" . $this->key . "&";
		$data .= "USER=" . $this->user . "&";
		$data .= "TRXTYPE=S" . "&"; // Always sale transactions
		$data .= "TENDER=C" . "&"; // Always credit card transactions
		$data .= "VERBOSITY=MEDIUM" . "&";
		$data .= "REQUEST_ID=" . md5($this->vars['ccno'] . $this->vars['amount'] . date('YmdGis') . "2");
	
		/*
		*  Contact Information
		*/
	
		if ($this->vars['fname']) $data .= "&FIRSTNAME=" . $this->vars['fname'];
		if ($this->vars['lname']) $data .= "&LASTNAME=" . $this->vars['lname'];
		if ($this->vars['address']) $data .= "&STREET=" . $this->vars['address'];
		if ($this->vars['city']) $data .= "&CITY=" . $this->vars['city'];
		if ($this->vars['state']) $data .= "&STATE=" . $this->vars['state'];
		if ($this->vars['zip']) $data .= "&ZIP=" . preg_replace("/\s/", "", $this->vars['zip']);
		if ($this->vars['country']) $data .= "&COUNTRY=" . $this->vars['country'];
	
		/*
		*  Order Information
		*/
	
		if ($this->vars['ccno']) $data .= "&ACCT=" . $this->vars['ccno'];
		if ($this->vars['ccexp']) $data .= "&EXPDATE=" . $this->vars['ccexp'];
		if ($this->vars['amount']) $data .= "&AMT=" . $this->vars['amount'];
		if ($this->vars['cvv2']) $data .= "&CVV2=" . $this->vars['cvv2'];
		if ($this->vars['description']) $data .= "&COMMENT1=" . $this->vars['description'];
	
		if (strlen($this->currency) > 0)
		$data .= "&CURRENCY=" . $this->currency;
	
	
		// Replaces any spaces with + for URL formatting
		//$data = ereg_replace(" ","+",$data);
		$data = preg_replace("/ /","+",$data);
		$headers = $this->getHeaders();
		$headers[] = "X-VPS-Request-ID: " . md5($this->vars['ccno'] . $this->vars['amount'] . date('YmdGis') . "2");
	
		/*
		*  Attempt to use PHP embedded CURL functions first. If they do not exist,
		*  then we will try using a curl binary on the server.
		*/
	
		if(function_exists("curl_init")) {
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $this->payflow_url);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_HEADER, 1);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
   
			// Log what we receive from OptimalPayments
			$this->logData("payflow", str_replace($this->vars['ccno'], "xxxx", $data), "input", true);

			$response = curl_exec($ch);
   
			curl_close($ch);
		}
		else {
			// Try to run it with curl from the command line
			exec("$curl -d \"$data\" $payflow_url", $response);
		}
	
		return $this->parseResponse($response);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		$response = strstr($response, "RESULT");
		parse_str($response, $out);

		// Takes the results from payflow and builds an array with the correct key value pairs
		$payflow_results=array(
		"x_response_code"=> ($out['RESULT'] == 12 ? 2 : ($out['RESULT'] == 0 ? 1 : 3)),
		"x_response_reason_text"=> $out['RESPMSG'],
		"x_auth_code"=> $out['AUTHCODE'],
		"x_trans_id"=> $out['PNREF']);

		// Log what we receive from OptimalPayments
		$this->logData("payflow", $response, "output", ($out['RESULT'] == 0 ? true : false));

		return $payflow_results;
	}
}

/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
American Express    378282246310005
American Express    371449635398431
Amex Corporate      378734493671000
Australian BankCard 5610591081018250
Diners Club         30569309025904
Diners Club         38520000023237
Discover            6011111111111117
Discover            6011000990139424
JCB                 3530111333300000
JCB                 3566002020360505
MasterCard          5555555555554444
MasterCard          5105105105105100
Visa                4111111111111111
Visa                4012888888881881
Visa                4222222222222
*/
?>