<?php
/*******************************************************************************
 @name PayJunction
 @type Merchant Gateway
 @author Cody Phillips
 @date May 7, 2009
 @version 1.1
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class payjunction extends Gateway {
	private $payjunction_url;
	private $test_url = "https://demo.payjunction.com/quick_link"; // TEST
	private $live_url = "https://payjunction.com/quick_link"; // LIVE
	private $username;
	private $password;
	private $bypass_avs = true; // true/false - true will bypass AVS checking and process the transaction even if AVS fails.
	private $vars;
	private static $settingFields = array("username" => "Username", "key" => "Password", "testmode" => "Test Mode");

	private static $supportedCurrencyCodes = array("USD");

	/*
	*  Initializes the gateway
	*/
	public function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->password = $gatewayinfo['key']['value'];
		$this->payjunction_url = ($gatewayinfo['testmode']['value'] == "True" ? $this->test_url : $this->live_url);
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}


	/***
	* processCard()
	*
	* Populate the POST query, then send it, and store
	* the response, and finally parse the response.
	*/
	public function processCard() {
		/* fill in the fields */
		$fields = array (
		'dc_logon' => $this->username,
		'dc_password' => $this->password,
		'dc_first_name' => $this->vars['fname'],
		'dc_last_name' => $this->vars['lname'],
		'dc_number' => $this->vars['ccno'],
		'dc_expiration_month' => substr($this->vars['ccexp'], 0, 2),
		'dc_expiration_year' => substr($this->vars['ccexp'], 2, 2),
		'dc_transaction_amount' => $this->vars['amount'],
		'dc_transaction_type' => "AUTHORIZATION_CAPTURE",
		'dc_version' => "1.2",
		'dc_address' => $this->vars['address'],
		'dc_city' => $this->vars['city'],
		'dc_state' => $this->vars['state'],
		'dc_zipcode' => $this->vars['zip'],
		'dc_country' => $this->vars['country'],
		'dc_security' => "XY|I|false|" . ($this->bypass_avs ? "true" : "false") . "|false" // FORMAT: AVS|CVS|PREAUTH|AVSFORCE|CVVFORCE
		);
		
		
		// perform the transaction
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $this->payjunction_url); // Set the URL
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));
		
		//Log the transaction data sent to PayJunction
		$this->logData("payjunction", str_replace($this->vars['ccno'], "xxxx", $fields), "input", true);
		
		$response = curl_exec($ch);
		curl_close($ch);
		
		// return the parsed response
		return $this->parseResponse($response);

	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		$response = explode(chr(0x1C), $response); // Parse based on FS character
		$data = array();
		for ($i=0; $i<count($response); $i++) {
			$temp = explode("=", trim($response[$i]));
			$data[$temp[0]] = $temp[1];
			unset($temp);
		}
		
		// Format the response for Blesta           
		$payjunction_results=array(
		"x_response_code" => ($data['dc_response_code'] == "00" || $data['dc_response_code'] == "85" ? 1 : (isset($data['dc_response_code']) ? 2 : 3)),
		"x_response_reason_text" => $data['dc_response_message'],
		"x_trans_id" => $data['dc_transaction_id']);
		
		// Log the formatted response from PayJunction
		$this->logData("payjunction", $data, "output", ($data['dc_response_code'] == "00" || $data['dc_response_code'] == "85" ? true : false));
		
		return $payjunction_results;
	}    
}


/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
4433221111223344
4444333322221111
*/
?>