<?php
/*******************************************************************************
 @name Protx VSP Direct
 @type Merchant Gateway
 @author Cody Phillips
 @date May 7, 2009
 @version 1.2
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class protx extends Gateway {
	private $protx_url_test = "https://ukvpstest.protx.com/vspgateway/service/vspdirect-register.vsp"; // Test
	private $protx_url = "https://ukvps.protx.com/vspgateway/service/vspdirect-register.vsp"; // Live
	//private $protx_url = "https://ukvpstest.protx.com/VSPSimulator/VSPDirectGateway.asp"; // DEV

	private $username;
	private $transType = "PAYMENT";
	private $vpsVersion = "2.22";
	private $currency;
	private $vars;
	private static $settingFields = array("username" => "Vendor Name", "testmode" => "Test Mode");
	
	// Note: Protx supports the following currency codes.  They may also support many others not listed here.
	// for a complete list see the Protx web site.
	private static $supportedCurrencyCodes = array("AUD", "CAD", "CHF", "DKK", "EUR", "GBP",
		"HKD", "IDR", "JPY", "LUF", "NOK", "NZD", "SEK", "SGD", "TRL", "USD");

	/*
	*  Initializes a protx gateway
	*/
	public function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->protx_url = $this->protx_url_test;
		
		$this->currency = (in_array($gatewayinfo['currency'], self::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "GBP");
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* processCard()
	*
	* Populate the POST query, then send it, and store
	* the response, and finally parse the response.
	*/
	public function processCard() {
		/* fill in the fields */
		$fields = array (
		'VPSProtocol' => $this->vpsVersion,
		'TxType' => $this->transType,
		'Vendor' => $this->username,
		'VendorTxCode' => rand(1000000, 9999999), // Some random value
		'Amount' => $this->vars['amount'],
		'Currency' => $this->currency,
		'Description' => ($this->vars['iid'] > 0 ? $this->vars['iid'] : "Account Payment"),
		'CardHolder' => $this->vars['fname'] . " " . $this->vars['lname'],
		'CardNumber' => $this->vars['ccno'],
		'ExpiryDate' => $this->vars['ccexp'],
		'CardType' => $this->getCardType($this->vars['ccno']),
		'BillingAddress' => $this->vars['address'],
		'BillingPostCode' => $this->vars['zip'],
		'ContactNumber' => $this->vars['phone'],
		'ApplyAVSCV2' => "2", // Force NO AVS/CV2 checks even if enabled on account.
		);
		
		
		// perform the transaction
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $this->protx_url); // Set the URL
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));
		
		// Log what we receive from OptimalPayments
		$this->logData("protx", str_replace($this->vars['ccno'], "xxxx", http_build_query($fields)), "input", true);
		
		$response = curl_exec($ch);
		curl_close($ch);
		
		// return the parsed response
		return $this->parseResponse($response, $fields);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response, $fields) {

		$out = explode("\r\n", $response);
		
		for ($i=0; $i<count($out); $i++) {
			$temp = explode("=", $out[$i]);
			
			$result[$temp[0]] = $temp[1];
		}
		
		// Takes the results from protx and builds an array with the correct key value pairs
		$protx_results=array(
		"x_response_code"=> ($result['Status'] == "NOTAUTHED" ? 2 : ($result['Status'] == "OK" ? 1 : 3)),
		"x_response_reason_text"=> $result['StatusDetail'],
		"x_auth_code"=> $result['SecurityKey'],
		"x_trans_id"=> $fields['VendorTxCode']); // $result['VPSTxId'] could optionally be stored here.
		
		// Log what we receive from OptimalPayments
		$this->logData("protx", $response, "output", ($result['status'] == "OK" ? true : false));
		
		return $protx_results;
	}

	/***
	* getCardType()
	*
	* This method returns the card type associated with the number, if it can
	* be deteremined, otherwise false is returned.
	*/
	private function getCardType($cardnum) {
		$pattern[0] = "/^([4]{1})([0-9]{12,15})$/"; // Visa
		$pattern[1] = "/^([51|52|53|54|55]{2})([0-9]{14})$/"; // Mastercard
		$pattern[2] = "/^(5020|5038|5641|6759)([0-9]{12})$/"; // Maestro
		$pattern[3] = "/^([30|36|38]{2})([0-9]{12})$/"; // Diner's Club
		$pattern[4] = "/^([34|37]{2})([0-9]{13})$/"; // American Express
		$pattern[5] = "/^(3[0-9]{15})$|^(2131|1800[0-9]{11})$/"; // JCB
		$pattern[6] = "/^6334[5-9]([0-9]{11}|[0-9]{13,14})|^6767([0-9]{12}|[0-9]{14,15})$/"; // Solo
		
		$returnval = array("VISA", "MC", "MAESTRO", "DC", "AMEX", "JCB", "SOLO");
		
		for ($i=0; $i<count($pattern); $i++) {
			if (preg_match($pattern[$i], $cardnum))
				return $returnval[$i];
		}
		
		return false;
	}
}


/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
Card Type	Card Number    		Issue	CV2	Address	PostCode
Visa Credit	4929 0000 0000 	6		123	88	412
MasterCard 	5404 0000 0000  0001		123	88	412
Visa Debit 	4462 0000 0000  0003		123	88	412
Solo		6334 9000 0000  0005		123	88	412
UK Maestro	5641 8200 0000  0005 	01	123	88	412
American Express 3742 000000 00004		123	88	412
Visa Electron	4917 3000 0000 0008		123	88	412
JCB		3569 9900 0000 0009		123	88	412
Diners Club	3600 0000 0000 08		123	88	412
*/
?>