<?php
/*******************************************************************************
 @name Quantum Gateway
 @type Merchant Gateway
 @author Cody Phillips
 @date May 7, 2009
 @version 1.3
 @notes For use with Blesta version 2.2.0+
        For testmode options see your Quantum Gateway control panel
*******************************************************************************/

class quantum extends Gateway {
	private $quantum_url = "https://secure.quantumgateway.com/cgi/tqgwdbe.php";
	private $curl = "/usr/bin/curl";
	private $username;
	private $key;
	private $vars;
	private static $settingFields = array("username" => "Login ID", "key" => "Restrict Key", "opt1" => "Use Maxmind (true/false)");
	
	// Note: quantumgateway.com supports the following currency codes.  They may also support many others not listed here.
	// for a complete list see quantumgateways's API documentation.
	private static $supportedCurrencyCodes = array("USD");


	/*
	*  Initializes the quantum gateway
	*/
	public function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
		$this->useMaxmind = (strtolower($gatewayinfo['opt1']['value']) == "true" ? true : false);
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* process()
	*
	* POST query, then send it, and parse the response.
	*/
	public function processCard() {
		// ccnum,ccmo and ccyr are required for CC payment.
		// aba, checkacct and trans_method with value of EFT are required for EFT payment
		$fields = array (
		'gwlogin' => $this->username,
		'trans_method' => "CC", // CC - Credit Card or EFT - Electronic bank transfer
		'ccnum' => $this->vars['ccno'],
		'ccmo' => substr($this->vars['ccexp'], 0, 2),
		'ccyr' => substr($this->vars['ccexp'], 2, 2),
		'aba' => "",
		'checkacct' => "",
		'amount' => $this->vars['amount'],
		'BNAME' => $this->vars['fname'] . " " . $this->vars['lname'],
		'BADDR1' => $this->vars['address'],
		'BZIP1' => $this->vars['zip'],
		'BCUST_EMAIL' => $this->vars['email'],
		'phone' => $this->vars['phone'],
		'fax' => $this->vars['fax'],
		'invoice_num' => $this->vars['iid'],
		'cust_id' => $this->vars['cid'],
		'override_email_customer' => "N", // Don't never send e-mails to customers, Blesta does this.
		'override_trans_email' => "N",
		'RestrictKey' => $this->key,
		'CVV2' => "",
		'CVVtype' => "0",
		'Dsep' => "|",
		'MAXMIND' => ($this->useMaxmind ? "1" : "2"), // 1 - Use Maxmind, 2 - Do not
		);
		
		// perform the transaction
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $this->quantum_url); // Set the URL
		curl_setopt($ch, CURLOPT_USERAGENT, "Blesta"); // Cosmetic
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // If not set, curl prints output to the browser
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));
		
		// Log what we receive from OptimalPayments
		$this->logData("quantum", str_replace($this->vars['ccno'], "xxxx", http_build_query($fields)), "input", true);          
		
		$response = curl_exec($ch);
		curl_close($ch);
		
		// return the parsed response
		return $this->parseResponse($response);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		// result, authcode, transID, AVRResponse, CVV Response, Max Score, decline_reason(if any), Error Code(if any)
		$out = explode("|", $response);
		for ($i=0; $i<count($out); $i++)
			$out[$i] = str_replace("\"", "", trim($out[$i]));
		
		$status = ($out[0] == "APPROVED" ? 1 : ($out[0] == "DECLINED" ? 2 : 3));
		if ($status == 3)
			$reason = (strlen($out[6]) == 0 ? $response : $out[6]);
		elseif ($status == 2)
			$reason = (strlen($out[6]) == 0 ? "Declined" : $out[6]);
		
		// Takes the results from Quantum and builds an array with the correct key value pairs
		$quantum_results= array(
		"x_response_code"=> $status,
		"x_response_reason_text"=> $reason,
		"x_auth_code"=> $out[1],
		"x_trans_id"=> $out[2]);
		
		// Log what we receive from OptimalPayments
		$this->logData("quantum", $response, "output", ($out[0] == "APPROVED" ? true : false));
		
		return $quantum_results;
	}
}



/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
VISA                4111 1111 1111 1111
MASTERCARD          5454 5454 5454 5454


*/
?>