<?php
/*******************************************************************************
 @name SagePayments
 @type Merchant Gateway
 @author Tyson Phillips
 @date October 8, 2010
 @version 1.0
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class sagepayments extends Gateway {
	private $url = "https://www.sagepayments.net/cgi-bin/eftBankcard.dll?transaction";
	private $id;
	private $key;
	private $vars;
	private static $settingFields = array("username" => "ID", "key" => "Key");

	private static $supportedCurrencyCodes = array("USD");

	/*
	*  Initializes the gateway
	*/
	public function __construct($gatewayinfo) {
		$this->id = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}
	
	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/***
	* processCard()
	*
	* Populate the POST query, then send it, and store
	* the response, and finally parse the response.
	*/
	public function processCard() {
		/* fill in the fields */
		$data = array(
			"M_id" => $this->id,
			"M_key" =>  $this->key,
			"T_amt" => $this->vars['amount'],
			"C_name" => $this->vars['fname'] . " " . $this->vars['lname'],
			"C_email" => $this->vars['email'],
			"C_address" => $this->vars['address'],
			"C_city" => $this->vars['city'],
			"C_state" => $this->vars['state'],
			"C_zip" => $this->vars['zip'],
			"C_cardnumber" => $this->vars['ccno'],
			"C_exp" => $this->vars['ccexp'],
			"T_code" => "01"
		);
		
		// perform the transaction
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $this->url); // Set the URL
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
		
		//Log the transaction data sent to SagePayments
		$this->logData("sagepayments", str_replace($this->vars['ccno'], "xxxx", $data), "input", true);
		
		$response = curl_exec($ch);
		curl_close($ch);
		
		// return the parsed response
		return $this->parseResponse($response);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		$response_code = substr($response, 1, 1);
		$response_text = substr($response, 8, 32);
		$order_index = strpos($response, chr(28)) + 1;
		$order_number = substr($response, $order_index, strpos(substr($response, $order_index), chr(28)));
		
		// Parse based on location/length
		$sagepayments_results = array(
			"x_response_code" => ($response_code == "A" ? 1 : ($response_code !== false ? 2 : 3)),
			"x_response_reason_text" => $response_text,
			"x_trans_id" => $order_number
		);
		
		// Log response from SagePayments
		$this->logData("sagepayments", $response, "output", ($response_code == "A" ? true : false));
		
		return $sagepayments_results;
	}    
}


/*
--------------------------------------------
Test Card Numbers
--------------------------------------------
Visa 4111111111111111
MasterCard 5499740000000057
Discover 6011000993026909
Amex 371449635392376
*/
?>