<?php
/*******************************************************************************
 @name Virtual Merchant
 @type Merchant Gateway
 @author Cody Phillips
 @date Apr 27, 2011
 @version 1.2
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class virtualmerchant extends Gateway {
	private static $virtualmerchant_url = "https://www.myvirtualmerchant.com/VirtualMerchant/process.do";
	private $vars;
	private $merchant_id;
	private $user_id;
	private $key;
	private $testmode;

	private static $settingFields = array("username" => "Account ID", "opt1" => "User ID", "key" => "PIN", "testmode" => "Test Mode");

	private static $supportedCurrencyCodes = array("USD");

	/*
	*  Initializes the virtualmerchant gateway
	*/
	public function __construct($gatewayinfo) {
		$this->merchant_id = $gatewayinfo['username']['value'];
		$this->user_id = $gatewayinfo['opt1']['value'];
		$this->key = $gatewayinfo['key']['value'];
		$this->testmode = ($gatewayinfo['testmode']['value'] == "True" ? true : false);
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}
	
	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* process()
	*
	* POST query, then send it, and parse the response.
	*/
	public function processCard() {
		// ccnum,ccmo and ccyr are required for CC payment.
		// aba, checkacct and trans_method with value of EFT are required for EFT payment
		$fields = array (
		'ssl_amount' => $this->vars['amount'],
		'ssl_merchant_id' => $this->merchant_id,
		'ssl_user_id' => $this->user_id,
		'ssl_pin' => $this->key,
		'ssl_show_form' => "false",
		'ssl_transaction_type' => "ccsale",
		'ssl_invoice_number' => $this->vars['iid'],
		'ssl_email' => $this->vars['email'],
		'ssl_first_name' => $this->vars['fname'],
		'ssl_last_name' => $this->vars['lname'],
		'ssl_avs_address' => $this->vars['address'],
		'ssl_avs_zip' => $this->vars['zip'],
		'ssl_card_number' => $this->vars['ccno'],
		'ssl_exp_date' => $this->vars['ccexp'],
		'ssl_result_format' => "ASCII"
		);
		
		if ($this->testmode)
			$fields['ssl_test_mode'] = "true";

		// perform the transaction
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, self::$virtualmerchant_url); // Set the URL
		curl_setopt($ch, CURLOPT_USERAGENT, "Blesta"); // Cosmetic
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // If not set, curl prints output to the browser
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));

		// Log what we receive from OptimalPayments
		$this->logData("virtualmerchant", str_replace($this->vars['ccno'], "xxxx", http_build_query($fields)), "input", true);          
		
		$response = curl_exec($ch);
		curl_close($ch);

		// return the parsed response
		return $this->parseResponse($response);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		$out = array();
		$temp = array();
		$temp2 = array();
		
		$temp = explode("\n", $response);
		
		for ($i=0; $i<count($temp); $i++) {
			$temp2 = explode("=", trim($temp[$i]));
			// Set the index to the left hand value and the value of this index to the right hand value.
			$out[$temp2[0]] = $temp2[1];
		}
		
		// Takes the results from virtualmerchant and builds an array with the correct key value pairs
		if ($out['errorCode']) {
			$virtualmerchant_results = array(
			"x_response_code"=> 3,
			"x_response_reason_text"=> $out['errorCode'] . " - " . $out['errorMessage'],
			);
		}
		else {
			$virtualmerchant_results = array(
			"x_response_code"=> ($out['ssl_result'] == "0" ? 1 : ($out['ssl_result'] == "1" ? 2 : 3)),
			"x_response_reason_text"=> $out['ssl_result_message'],
			"x_auth_code"=> $out['ssl_approval_code'],
			"x_trans_id"=> $out['ssl_approval_code']);
		}
		
		// Log what we receive from OptimalPayments
		$this->logData("virtualmerchant", $response, "output", ($out['ssl_result'] == "0" ? true : false));
		
		return $virtualmerchant_results;
	}
}



/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
VISA                4111 1111 1111 1111
MASTERCARD          5454 5454 5454 5454


*/
?>