<?php
/*******************************************************************************
 @name 2Checkout
 @type Non-Merchant Gateway
 @author Cody Phillips
 @date Mar 23, 2010
 @version 1.2
 @notes For use with Blesta version 2.2.0+
	Configure 2Checkout Look and Feel settings as follows:
	Select "Yes" for Direct Return
	Set both Pending and Approved URLs to https://www.yoursite.com/your_blesta_install/callback.php?gw=_2checkout
	Direct Return must be set to "Yes"
	
	WARNING: 2Checkout does not support multi-currency processing, therefore all
	payments sent to 2Checkout will be processed in the currency you have set
	in your 2Checkout account regardless of what currency the user attempts to
	pay in. 2Checkout developers are currently looking at implementing a
	system similar to Paypal's to handle multi-currencies.

*******************************************************************************/

class _2checkout extends Gateway {
	private $_2checkout_url = "https://www.2checkout.com/checkout/purchase";
	private $testmode = "True";
	private $gatewayinfo;
	private $orderinfo;
	private $button = array("url" => "", "alt" => "Pay with 2Checkout");
	private $hiddenFields = array("sid" => "", "total" => "");
	
	private static $supportedCurrencyCodes = array("AUD", "CAD", "CHF", "DKK", "EUR",
		"GBP", "HKD", "JPY", "NOK", "NZD", "USD", "SEK");
	private static $settingFields = array("username" => "Vendor Account Number", "opt1" => "Secret Word", "testmode" => "Test Mode");
	private static $gw = "_2checkout"; // Must be defined in /inc/config.php under "PAYTYPES"

	public function __construct($gatewayinfo, $orderinfo) {
		$this->gatewayinfo = $gatewayinfo;
		$this->orderinfo = $orderinfo;
		$this->button['url'] = "http" . ($_SERVER['HTTPS'] ? "s" : "") . "://" . $_SERVER['SERVER_NAME'] . WEBDIR . "templates/" . THEME . "/images/2checkout_button.gif";

		// If testmode is set the url to the test url
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->hiddenFields['demo'] = "Y";

		// If pending Order IDs are passed, add these as a hidden field to be passed
		// back by 2checkout
		if (is_array($orderinfo['pending']))
			$this->hiddenFields['pending'] = base64_encode(serialize($orderinfo['pending']));

		// If a coupon is passed add it. Coupons will be incremented in use by the callback script.
		if (strlen($orderinfo['coupon']) > 0)
			$this->hiddenFields['couponcode'] = $orderinfo['coupon'];

		$this->hiddenFields['sid'] = $gatewayinfo['username']['value'];
		$this->hiddenFields['total'] = $orderinfo['amount'];
		$this->hiddenFields['fixed'] = "Y";
		$this->hiddenFields['c_tangible'] = "N";
		$this->hiddenFields['uid'] = $orderinfo['uid'];

		if (is_numeric($orderinfo['iid'])) {
			$this->hiddenFields['iid'] = $orderinfo['iid'];
			$this->hiddenFields['cart_order_id'] = $orderinfo['iid'];
		}
		else
			$this->hiddenFields['cart_order_id'] = $orderinfo['uid'] . "-" . time();
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
		}

		$fields['postto'] = $this->_2checkout_url;
		$fields['button'] = $this->button;

		return $fields;
	}

	public function getSettingFields() {
		return _2checkout::$settingFields;
	}

	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway
		$this->logData("_2checkout", $postvars, "output", false);
		
		// In demo mode order number is ALWAYS 1, otherwise it is the correct order number
		if (strtoupper(md5($this->gatewayinfo['opt1']['value'] . $this->gatewayinfo['username']['value'] . ($postvars['demo'] == "Y" ? "1" : $postvars['order_number']) . $postvars['total'])) != $postvars['key'])
			return false; // This is probably a fraud attempt

		// Confirm that the payment is completed
		if ($postvars['credit_card_processed'] != "Y")
			return false;

		// Validate that the funds were actually sent to the right account
		if ($postvars['sid'] != $this->hiddenFields['sid'])
			return false;

		$this->updateLogStatus("output", true);
		
		// No problems? Process the transaction
		$trans['amount'] = $postvars['total'];
		//$trans['currency'] = $postvars['currency']; // not send back by 2Checkout?
		$trans['uid'] = $postvars['uid'];
		$trans['iid'] = $postvars['iid'];
		$trans['type'] = "2checkout"; // Must be defined in /inc/config.php under "PAYTYPES"
		$trans['transid'] = $postvars['order_number'];
		$trans['message'] = "";
		$trans['status'] = "approved";
		$trans['coupon'] = $postvars['couponcode'];

		// Send Pending Serivce IDs to mark as being paid
		if ($postvars['pending'])
			$trans['pending'] = unserialize(base64_decode($postvars['pending']));

		return $trans;
	}
}
?>