<?php
/*******************************************************************************
 @name cashU
 @type Non-Merchant Gateway
 @author Cody Phillips
 @date May 6, 2009
 @version 1.1
 @notes For use with Blesta version 2.2.0+
        Set thanx_url URL to https://www.yoursite.com/your_blesta_install/callback.php?gw=cashu

        "Secret Word" can be any string, it is only used to prevent fraud
        attempts on your account

        Test cashU account is test@cashucard.com and the account password is: 12345678
        Use the above test account for making test payments to your cashU account.

*******************************************************************************/

class cashu extends Gateway {
	private $cashu_url = "https://www.cashu.com/cgi-bin/pcashu.cgi";
	private $testmode = "True";
	private $gatewayinfo;
	private $orderinfo;
	private $button = array("url" => "https://www.cashu.com/images/newModel/buyIcons/icon_e6.gif", "alt" => "Make payments with cashU - it's fast, free and secure!");
	private $hiddenFields = array("merchant_id" => "", "amount" => "", "currency" => "USD", "language" => "en", "display_text" => "", "token" => "", "txt1" => "");
	private static $settingFields = array("username" => "Merchant ID", "key" => "Encryption Keyword", "opt1" => "Secret Word", "testmode" => "Test Mode");
	private static $gw = "cashu"; // Must be defined in /inc/config.php under "PAYTYPES"
	private static $supportedCurrencyCodes = array("USD", "AED", "EUR", "JOD", "EGP", "SAR");

	function __construct($gatewayinfo, $orderinfo) {
		$this->gatewayinfo = $gatewayinfo;
		$this->orderinfo = $orderinfo;

		// Coupons are stored in txt2, pending data is stored in txt3, invoice id is stored in txt1

		// If testmode is set the url to the test url
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->hiddenFields['test_mode'] = "1";

		// If pending Order IDs are passed, add these as a hidden field to be passed
		// back by 2checkout
		if (is_array($orderinfo['pending']))
			$this->hiddenFields['txt3'] = base64_encode(serialize($orderinfo['pending']));

		// If a coupon is passed add it. Coupons will be incremented in use by the callback script.
		if (strlen($orderinfo['coupon']) > 0)
			$this->hiddenFields['txt2'] = $orderinfo['coupon'];

		$this->hiddenFields['merchant_id'] = $gatewayinfo['username']['value'];
		$this->hiddenFields['amount'] = $orderinfo['amount'];
		//$this->hiddenFields['currency'] = ($this->supportedCurrencyCodes[$gatewayinfo['currency']] == 1 ? $gatewayinfo['currency'] : "USD");
		$this->hiddenFields['currency'] = (in_array($gatewayinfo['currency'], cashu::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "USD");
		$this->hiddenFields['session_id'] = $orderinfo['uid'];
		$this->hiddenFields['token'] = md5(strtolower($this->gatewayinfo['username']['value'] . ":" . $orderinfo['amount'] . ":" . $this->hiddenFields['currency'] . ":" . $gatewayinfo['key']['value']));

		// This string is validated when the callback is processed
		$this->hiddenFields['txt4'] = md5($this->hiddenFields['token'] . $gatewayinfo['opt1']['value']);

		if (is_numeric($orderinfo['iid'])) {
			$this->hiddenFields['txt1'] = $orderinfo['iid'];
			$this->hiddenFields['display_text'] = $orderinfo['description'];
		}
		else {
			$this->hiddenFields['txt1'] = "Account balance for UID: " . $orderinfo['uid'];
			$this->hiddenFields['display_text'] = $this->hiddenFields['txt1'];
		}
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
		}

		$fields['postto'] = $this->cashu_url;
		$fields['button'] = $this->button;

		return $fields;
	}

	public function getSettingFields() {
		return cashu::$settingFields;
	}

	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway
		$this->logData("cashu", $postvars, "output", false);
		
		// Check for fraudulent posts
		if ($postvars['txt4'] != md5($postvars['token'] . $this->gatewayinfo['opt1']['value']))
			return false; // Payment attempt was likely fraud

		// Validate the token
		if ($postvars['token'] != md5(strtolower($this->gatewayinfo['username']['value'] . ":" . $postvars['amount'] . ":" . $postvars['currency'] . ":" . $this->gatewayinfo['key']['value'])))
			return false;

		// Only allow test mode transactios to go through if this gateway is set in testmode
		if (isset($postvars['test_mode']) && $this->gatewayinfo['testmode']['value'] != "True")
			return false;

		$this->updateLogStatus("output", true);

		// No problems? Process the transaction
		$trans['amount'] = $postvars['amount'];
		$trans['currency'] = $postvars['currency'];
		$trans['uid'] = $postvars['session_id'];
		$trans['iid'] = (is_numeric($postvars['txt1']) ? $postvars['txt1'] : "");
		$trans['type'] = "cashu"; // Must be defined in /inc/config.php under "PAYTYPES"
		$trans['transid'] = $postvars['trn_id'];
		$trans['message'] = "";
		$trans['status'] = "approved";
		$trans['coupon'] = $postvars['txt2'];

		// Send Pending Serivce IDs to mark as being paid
		if ($postvars['txt3'])
			$trans['pending'] = unserialize(base64_decode($postvars['txt3']));

		return $trans;
	}
}
?>