<?php
/*******************************************************************************
 @name Chronopay
 @type Non-Merchant Gateway
 @author Cody Phillips
 @date May 6, 2009
 @version 1.1
 @notes For use with Blesta version 2.2.0+
        "Generic Product ID" is the product ID set in the Chronopay interface
        used for generic purchases.

        "Secret Word" can be any string, it is only used to prevent fraud
        attempts on your account.

        For testing accepted card:
        4111111111111111 Expiration date - current date + 1 month CVV - any

        Declined card:
        4000000000000002 Expiration date - current date + 1 month CVV - any
        
	WARNING: Chronopay does not support multi-currency using a single Product
	ID. This means that all payments processed through this gateway will be
	interpreted by Chronopay in the currency that is configured for this
	Product ID in the Chronopay interface.

*******************************************************************************/

class chronopay extends Gateway {
	private $chronopay_url = "https://secure.chronopay.com/index_shop.cgi";
	private $gatewayinfo;
	private $orderinfo;
	private $button = array("url" => "", "alt" => "Pay with Chronopay");
	private $hiddenFields = array("product_id" => "", "product_name" => "", "cs1" => "", "cs2" => "", "cs3" => "", "total" => "", "cb_url" => "");
	private static $settingFields = array("username" => "Generic Product ID", "key" => "Secret Word");
	private static $gw = "chronopay"; // Must be defined in /inc/config.php under "PAYTYPES"
	private static $supportedCurrencyCodes = array("AUD", "CAD", "EUR", "GBP", "JPY",
		"USD", "NZD", "CHF", "HKD", "SGD", "SEK", "DKK", "PLN", "NOK", "HUF", "CZK");

	function __construct($gatewayinfo, $orderinfo) {
		$this->gatewayinfo = $gatewayinfo;
		$this->orderinfo = $orderinfo;
		$this->button['url'] = "http" . ($_SERVER['HTTPS'] ? "s" : "") . "://" . $_SERVER['SERVER_NAME'] . WEBDIR . "templates/" . THEME . "/images/chronopay_button.gif";

		// If pending Order IDs are passed, add these as a hidden field to be passed
		// back by chronopay
		$pending = "";
		if (is_array($orderinfo['pending']))
			$pending = base64_encode(serialize($orderinfo['pending']));

		$this->hiddenFields['product_id'] = $gatewayinfo['username']['value'];
		$this->hiddenFields['product_name'] = $orderinfo['description'];
		$this->hiddenFields['product_price'] = $orderinfo['amount'];
		$this->hiddenFields['cs1'] = $orderinfo['uid'] . "-" . time() . "-" . $orderinfo['iid'];
		$this->hiddenFields['cs2'] = $orderinfo['coupon'] . "-" . $pending;
		$this->hiddenFields['cs3'] = md5($this->hiddenFields['cs1'] . $gatewayinfo['key']['value'] . $orderinfo['amount'] . $gatewayinfo['username']['value']);
		$this->hiddenFields['cb_url'] = GATEWAY_CALLBACK_URL . "?gw=chronopay";
		$this->hiddenFields['cb_type'] = "P";
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
		}

		$fields['postto'] = $this->chronopay_url;
		$fields['button'] = $this->button;

		return $fields;
	}

	public function getSettingFields() {
		return chronopay::$settingFields;
	}

	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway
		$this->logData("chronopay", $postvars, "output", false);
		
		// Confirm that this payment was successful
		if ($postvars['transaction_type'] != "initial" && $postvars['transaction_type'] != "onetime")
			return false; // Declined

		$uid_time_iid = explode("-", $postvars['cs1']);
		$coupon_pending = explode("-", $postvars['cs2']);

		// Detect fraud attempts
		$verify = md5($postvars['cs1'] . $this->gatewayinfo['key']['value'] . $postvars['total'] . $this->gatewayinfo['username']['value']);

		if ($verify != $postvars['cs3'])
			return false; // This is probably a fraud attempt

		// Validate that the funds were actually sent to the right account
		if ($postvars['product_id'] != $this->gatewayinfo['username']['value'])
			return false;

		$this->updateLogStatus("output", true);

		// No problems? Process the transaction
		$trans['amount'] = $postvars['total'];
		$trans['currency'] = $postvars['currency'];
		$trans['uid'] = $uid_time_iid[0];
		$trans['iid'] = $uid_time_iid[2];
		$trans['type'] = "chronopay"; // Must be defined in /inc/config.php under "PAYTYPES"
		$trans['transid'] = $postvars['transaction_id'];
		$trans['message'] = "";
		$trans['status'] = "approved";
		$trans['coupon'] = $coupon_pending[0];

		// Send Pending Serivce IDs to mark as being paid
		if (strlen($coupon_pending[1]) > 3)
			$trans['pending'] = unserialize(base64_decode($coupon_pending[1]));

		return $trans;
	}
}
?>