<?php
/*******************************************************************************
 @name Moneybookers
 @type Non-Merchant Gateway
 @author Cody Phillips 
 @date May 6, 2009
 @version 1.1
 @notes For use with Blesta version 2.2.0+

*******************************************************************************/
 
class moneybookers extends Gateway {

	private $moneybookers_url = "https://www.moneybookers.com/app/payment.pl";
	private $gatewayinfo;
	private $orderinfo;
	private $recurr;
	private $button = array("url" => "", "alt" => "Pay with Moneybookers");
	private $hiddenFields = array("language" => "EN", "currency" => "", "pay_to_email" => "",
		"detail1_description" => "", "detail1_text" => "", "status_url" => "");

	private static $settingFields = array("username" => "E-mail Address", "key" => "Secret Word", "opt1" => "Recur Payment (true/false)");
	private static $supportedCurrencyCodes = array("AUD", "BGN", "CAD", "CHF",
		"CZK", "DKK", "EEK", "EUR", "GBP", "HKD", "HRK", 
		"HUF", "ILS", "INR", "ISK", "JPY", "KRW", "LVL", 
		"LTL", "MYR", "NOK", "NZD", "PLN", "RON", "SEK", 
		"SGD", "SKK", "THB", "TRY", "TWD", "USD", "ZAR");

	function __construct($gatewayinfo, $orderinfo) {
		$this->gatewayinfo = $gatewayinfo;
		$this->orderinfo = $orderinfo;
		$this->recurr = $orderinfo['recurr'];
		$this->button['url'] = "http" . ($_SERVER['HTTPS'] ? "s" : "") . "://www.moneybookers.com/images/logos/checkout_logos/checkout_120x40px.gif";

		//$this->hiddenFields['currency'] = (moneybookers::$supportedCurrencyCodes[$gatewayinfo['currency']] == 1 ? $gatewayinfo['currency'] : "");
		$this->hiddenFields['currency'] = (in_array($gatewayinfo['currency'], moneybookers::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "");
		$this->hiddenFields['status_url'] =  GATEWAY_CALLBACK_URL . "?gw=moneybookers&uid=" . $orderinfo['uid'];
		$this->hiddenFields['pay_to_email'] = $gatewayinfo['username']['value'];
		$this->hiddenFields['detail1_description'] = " ";
		$this->hiddenFields['detail1_text'] = $orderinfo['description'];

		// If pending service info is required send it along.
		if (is_array($orderinfo['pending']))
			$this->hiddenFields['pending'] = base64_encode(serialize($orderinfo['pending']));

		// If a coupon is passed add it. Coupons will be incremented in use by the callback script.
		// Add timestamp to prevent coupons from counting for recurring payments.
		if (strlen($orderinfo['coupon']) > 0)
			$this->hiddenFields['coupon'] = $orderinfo['coupon'] . "|" . time();

		if (is_array($this->recurr) && $this->gatewayinfo['opt1']['value'] == "true") {
			$this->hiddenFields['rec_amount'] = $this->recurr['amount'];
			$this->hiddenFields['rec_start_date'] = ""; // start immediately
			// Moneybookers is a bit silly in requiring an END DATE for recurring payments, but since we're required we'll make it recur 12 times
			$this->hiddenFields['rec_end_date'] = date("d/m/Y", strtotime(CURDATE . " +" . (12*$this->recurr['term']) . " months"));
			$this->hiddenFields['rec_period'] = $this->recurr['term'];
			$this->hiddenFields['rec_cycle'] = "month";
			$this->hiddenFields['rec_status_url'] = $this->hiddenFields['status_url'];

			// If the recurring amount does not match the current amount then we must set up
			// both a recurring payment AND a one time payment amount
			if ($this->recurr['amount'] != $orderinfo['amount'])
				$this->hiddenFields['amount'] = $orderinfo['amount'];
		}
		else
			$this->hiddenFields['amount'] = $orderinfo['amount'];
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
	 	}

		$fields['postto'] = $this->moneybookers_url;
		$fields['button'] = $this->button;

		return $fields;
	}
	
	public function getSettingFields() {
		return moneybookers::$settingFields;
	}
 
	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway
		$this->logData("moneybookers", $postvars, "output", false);
		
		// Fraud Detection
		// Ensure the pay_to_email is this correct e-mail address
		if (strtolower($post['pay_to_email']) != strtolower($this->gatewayinfo['username']['value']))
			return false; // fraud
	
		$secretword = ($this->gatewayinfo['key']['value'] != "" ? strtoupper(md5($this->gatewayinfo['key']['value'])) : ""); // secret word as defined in the Moneybookers control panel
		// Ensure md5sig is a match
		if ($postvars['md5sig'] != strtoupper(md5($postvars['merchant_id'] . $postvars['transaction_id'] . $secretword . $postvars['mb_amount'] . $postvars['mb_currency'] . $postvars['status'])))
			return false; // fraud

		$this->updateLogStatus("output", true);
		
		$trans = array();
		$trans['amount'] = $postvars['amount'];
		$trans['currency'] = $postvars['currency'];
		$trans['uid'] = $getvars['uid'];
		$trans['iid'] = $postvars['iid'];
		$trans['type'] = "moneybookers";
		$trans['transid'] = $postvars['transaction_id'];
		$trans['message'] = $postvars['rec_payment_id']; // Store the recurring payment id, if it was passed

		if (isset($postvars['coupon'])) {
			$coupon = explode("|", $postvars['coupon']);

			// Only send the coupon if this trans is within 24 hours
			$trans['coupon'] = ($coupon[1] >= (time()-60*60*24) && $coupon[1] <= time() ? $coupon[0] : "");
		}

		if (isset($postvars['pending']))
			$trans['pending'] = unserialize(base64_decode($postvars['pending']));

		switch ($postvars['status']) {
			//Processed
			case "2":
				$trans['status'] = "approved";
				break;
			// Pending
			case "0":
				$trans['status'] = "pending";
				break;
			// Canceled
			case "-1":
				$trans['status'] = "declined";
				break;
			// Failed
			default:
			case "-2":
				$trans['status'] = "error";
				break;
			// Chargeback
			case "-3":
				$trans['status'] = "declined";
				break;
		}

		return $trans;
	}	
}
?>