<?php
/*******************************************************************************
 @name PayPal
 @type Non-Merchant Gateway
 @author Cody Phillips
 @date May 6, 2009
 @version 1.3
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class paypal extends Gateway {
	private $paypal_url = "https://www.paypal.com/cgi-bin/webscr"; // Live mode;
	private $paypal_url_test = "https://www.sandbox.paypal.com/cgi-bin/webscr"; // Test mode;
	private $testmode = "True";
	private $currency;
	private $vars;
	private $button = array("url" => "https://www.paypal.com/en_US/i/btn/x-click-but02.gif", "alt" => "Make payments with PayPal - it's fast, free and secure!");
	private $hiddenFields = array("cmd" => "_xclick", "cpp_header_image" => "", "notify_url" => "",
				  "business" => "", "item_number" => "", "amount" => "",
				  "item_name" => "", "no_note" => "1", "currency_code" => "", "invoice" => "", "custom" => ""
				  );
	private static $settingFields = array("username" => "Paypal E-mail", "opt1" => "Logo URL", "testmode" => "Test Mode");

	// Note: PayPal supports the following currency codes.
	private static $supportedCurrencyCodes = array("AUD", "CAD", "EUR", "GBP", "JPY",
		"USD", "NZD", "CHF", "HKD", "SGD", "SEK", "DKK", "PLN", "NOK", "HUF", "CZK");

	function __construct($gatewayinfo, $orderinfo) {
		$this->hiddenFields['business'] = $gatewayinfo['username']['value'];
		$this->hiddenFields['cpp_header_image'] = $gatewayinfo['opt1']['value'];
		$this->hiddenFields['item_number'] = $orderinfo['iid'];
		$this->hiddenFields['invoice'] = $orderinfo['iid'];
		$this->hiddenFields['amount'] = $orderinfo['amount'];
		//$this->hiddenFields['currency_code'] = (paypal::$supportedCurrencyCodes[$gatewayinfo['currency']] == 1 ? $gatewayinfo['currency'] : "");
		$this->hiddenFields['currency_code'] = (in_array($gatewayinfo['currency'], paypal::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "");
		$this->hiddenFields['notify_url'] = GATEWAY_CALLBACK_URL . "?gw=paypal&uid=" . $orderinfo['uid'];
		$this->hiddenFields['item_name'] = $orderinfo['description'];

		// If testmode is set the url to the test url
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->paypal_url = $this->paypal_url_test;

		// If pending Order IDs are passed, add these as a hidden field to be passed
		// back by PayPal
		if (is_array($orderinfo['pending']))
			$this->hiddenFields['custom'] = base64_encode(serialize($orderinfo['pending']));

		$this->hiddenFields['custom'] .= "|";

		// If a coupon is passed add it. Coupons will be incremented in use by the callback script.
		// Add timestamp to prevent coupons from counting for recurring payments.
		if (strlen($orderinfo['coupon']) > 0)
			$this->hiddenFields['custom'] .= $orderinfo['coupon'] . "|" . time();
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
		}

		$fields['postto'] = $this->paypal_url;
		$fields['button'] = $this->button;

		return $fields;
	}

	public function getSettingFields() {
		return  paypal::$settingFields;
	}

	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway (default to failure -- we'll update if successful)
		$this->logData("paypal", $postvars, "output", false);
		
		// Repost all POST vars back to paypal
		$postvars['cmd'] = "_notify-validate";

		// Hold the response.
		$response = "";

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->paypal_url);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postvars));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

		// Wait for confirmation "VERIFIED" or "INVALID"
		$response = curl_exec($ch);
		curl_close($ch);

		// Evaluate Paypal's response
		if ($response == "VERIFIED") {
			$pending = false;
			$coupon = false;
			
			if (isset($postvars['custom'])) {
				$temp = explode("|", $postvars['custom']);	
				$pending = $temp[0];
				$coupon = $temp[1];
				
				if (strlen($coupon) > 0)
					$time = $temp[2];
			}
			
			// Confirm that the payment_status is Completed, since IPNs are
			// also sent for other results such as Pending or Failed
			if ($postvars['payment_status'] != "Completed")
				return false;

			// Validate that the receiver_email is an email address registered
			// in your PayPal account, to prevent the payment from being sent to a fraudster's account
			if ($postvars['business'] != $this->hiddenFields['business'])
				return false;
			
			// Set log status to success
			$this->updateLogStatus("output", true);

			// If VERIFIED and all fields look good, return the following array
			$trans['amount'] = $postvars['mc_gross'];
			$trans['currency'] = $postvars['mc_currency'];
			$trans['uid'] = $getvars['uid'];
			$trans['iid'] = $postvars['invoice'];
			$trans['type'] = "paypal"; // Must be defined in /inc/config.php under "PAYTYPES"
			$trans['transid'] = $postvars['txn_id'];
			$trans['message'] = $postvars['payment_status'];
			$trans['status'] = "approved";
			$trans['coupon'] = $coupon;

			// Send Pending Serivce IDs to mark as being paid
			if ($pending)
				$trans['pending'] = unserialize(base64_decode($pending));

			return $trans;
		}
		else
			return false;
	}
}
?>