<?php
/*******************************************************************************
 @name Worldpay
 @type Non-Merchant Gateway
 @author Sojish at Bobcares.com 
 @date May 6, 2009
 @version 1.1
 @notes For use with Blesta version 2.2.0+
	Configure WorldPay and set Payment Response URL as
	https://www.yoursite.com/your_blesta_install/callback.php?gw=worldpay
	
	More help available at 
	http://www.worldpay.com/support/kb/mergedProjects/paymentresponse/pr5101.html

	You must enable Payment Response Password:

	1. Log in to the Merchant Administration Interface
	2. Select Installations from the left hand navigation
	3. Choose an installation and select the Integration Setup button for either the TEST or PRODUCTION environment
	4. Enter a password into the Payment Response Password input field
	5. Select the Save Changes button
	6. The password is then included in the HTTP(s) Payment Message using the parameter name, callbackPW.


	You must also set up a MD5 secret word.  To do this you must supply Worldpay with
	this secret word.  This secret must be sent to Worldpay's technical support team and 
	you will need to enter it in the MD5 secret for transactions field in the 
	Integration Setup for your installation - via the MAI.

	This gateway updated by Cody Phillips.

*******************************************************************************/
 
 
class worldpay extends Gateway {

	private $worldpay_url = "https://select.worldpay.com/wcc/purchase";
	private $gatewayinfo;
	private $orderinfo;
	private $button = array("url" => "", "alt" => "Pay with WorldPay");
	private $hiddenFields = array("instId" => "", "cartId" => "", "amount" => "", "MC_uid" => "", "MC_iid" => "",
         	"desc" => "", "currency" => "", "signatureFields" => "", "signature" => "", "testMode" => "");
	
	private static $settingFields = array("username" => "Installation Id", "key" => "Cart ID", "opt1" => "Secret Word", "opt2" => "Payment Response Password", "testmode" => "Test Mode");
	private static $supportedCurrencyCodes = array("ARS", "AUD", "BRL", "CAD", "CHF",
			"CLP", "CNY", "COP", "CZK", "DKK", "EUR", "GBP", "HKD",
			"HUF", "IDR", "ISK", "JPY", "KES", "KRW", "MXP", "MYR", "NOK",
			"NZD", "PHP", "PLN", "PTE", "SEK", "SGD", "SKK", "THB", "TWD",
			"USD", "VND", "ZAR");

	function __construct($gatewayinfo, $orderinfo) {
		$this->gatewayinfo = $gatewayinfo;
		$this->orderinfo = $orderinfo;

		$this->button['url'] = "http" . ($_SERVER['HTTPS'] ? "s" : "") . "://" . $_SERVER['SERVER_NAME'] . WEBDIR . "templates/" . THEME . "/images/worldpay.gif";
		$this->hiddenFields['testMode'] = "0";

		if ($gatewayinfo['testmode']['value'] == "True") {
			$this->hiddenFields['testMode'] = "100";
			$this->hiddenFields['name'] =  "AUTHORISED";
		}

		$this->hiddenFields['instId'] = $gatewayinfo['username']['value'];
		$this->hiddenFields['cartId'] = $gatewayinfo['key']['value'];
		$this->hiddenFields['amount'] =  $orderinfo['amount'];
		$this->hiddenFields['MC_uid'] = $orderinfo['uid'];
		$this->hiddenFields['MC_iid'] = $orderinfo['iid'];
		$this->hiddenFields['desc'] = $orderinfo['description'];
		//$this->hiddenFields['currency'] = (worldpay::$supportedCurrencyCodes[$gatewayinfo['currency']] == 1 ? $gatewayinfo['currency'] : "");
		$this->hiddenFields['currency'] = (in_array($gatewayinfo['currency'], worldpay::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "");
		$this->hiddenFields['signatureFields'] = "MC_uid:amount:currency:cartId";
		$this->hiddenFields['signature'] = md5($gatewayinfo['opt1']['value'] . ":" . $orderinfo['uid'] . ":" . $orderinfo['amount'] . ":" . $this->hiddenFields['currency'] . ":" . $gatewayinfo['key']['value']);
		$this->hiddenFields['MC_signature'] = $this->hiddenFields['signature'];
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
	 	}

		$fields['postto'] = $this->worldpay_url;
		$fields['button'] = $this->button;

		return $fields;
	}
	
	
	public function getSettingFields() {
		return worldpay::$settingFields;
	}
 
	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway (default to failure -- we'll update if successful)
		$this->logData("worldpay", $postvars, "output", false);
		
		// Validate that the funds were actually sent to the right account
		if ($postvars['instId'] != $this->gatewayinfo['username']['value'])
			return false;

		// Ensure the response values are authentic
		if ($postvars['MC_signature'] != md5($this->gatewayinfo['opt1']['value'] . ":" . $postvars['MC_uid'] . ":" . $postvars['amount'] . ":" . $postvars['currency'] . ":" . $this->gatewayinfo['key']['value']))
			return false;

		// Ensure the response came from Worldpay
		if ($postvars['callbackPW'] != $this->gatewayinfo['opt2']['value'])
			return false;
		
		$this->updateLogStatus("output", true);
		
		$trans['amount'] = $postvars['amount'];
		$trans['currency'] = $postvars['currency'];
		$trans['uid'] = $postvars['MC_uid'];
		$trans['iid'] = $postvars['MC_iid'];
		$trans['type'] = "worldpay"; // Must be defined in /inc/config.php under "PAYTYPES"   
		$trans['transid'] = $postvars['transId'];
		$trans['message'] = $postvars['rawAuthMessage'];

 	 	if ($postvars['transStatus']== "Y")
			$trans['status'] = "approved";
		elseif ($postvars['transStatus']== "C") 
			$trans['status'] = "declined";
		else
			$trans['status'] = "error";

		return $trans;
	}
}
?>