<?php

include_once(ROOTWEBDIR . "inc/pdf/tcpdf/tcpdf.php");

class InvoiceTCPDF extends TCPDF implements InvoiceInterface {
	private $company_info;
	private $background;

	public function __construct($company_info) {
		parent::__construct("P", "px", "A4", true, 'UTF-8', false);
		
		$this->company_info = $company_info;
		$this->background = $this->company_info['background'];
		
		// Set tag to use on page numbering replacement
		$this->AliasNbPages("{P}");

		// Set image scale factor
		$this->setImageScale(2); 
		
		// Default monospaced font
		$this->SetDefaultMonospacedFont("courier");
		
		// Set margins
		$this->SetMargins(25, 330, 25);
		$this->SetFooterMargin(160);
		
		// Set auto page breaks y-px from the bottom of the page
		$this->SetAutoPageBreak(true, 190);
		
		$lang = array();
		$lang['a_meta_charset'] = 'UTF-8';
		$lang['a_meta_dir'] = 'ltr';
		
		// Set language settings
		$this->setLanguageArray($lang);
		
		// Set font
		$this->SetFont('dejavusanscondensed', '', 12);
	}
	
	public function Header() {
        // Set background image by fetching current margin break,
		// then disable page break, set the image, and re-enable page break with
		// the current margin
        $bMargin = $this->getBreakMargin();
        $auto_page_break = $this->AutoPageBreak;
        $this->SetAutoPageBreak(false, 0);
		if (file_exists($this->background))
			$this->Image($this->background, 0, 0, 0, 0, '', '', '', false, 300, '', false, false, 0);
        // restore auto-page-break status
        $this->SetAutoPageBreak($auto_page_break, $bMargin);
		
		// Set logo
		if ($this->company_info['use'] == "logo") {
			if (file_exists($this->company_info['logo']))
				$this->Image($this->company_info['logo'], 28, 24);
		}
		// Place the company name on the page
		elseif ($this->company_info['use'] == "name") {
			$data = array(
				array($this->company_info['name']),
				array($this->company_info['address'])
			);
			$options = array(
				'font_size'=>12,
				'y_pos'=>34,
				'x_pos'=>38,
				'align'=>'L'
			);
			// Draw Invoice Date and Page #
			$this->drawTable($data, $options);
		}
		
		// Add Invoice Date and Page #
		$tax_id = "";
		if (strlen($this->company_info['tax_id']) > 0 && $this->inv['tax1_total'] > 0)
			$tax_id = $this->company_info['tax_id'];
		$data = array(
			array(_INVOICE_PDF_INVDATE . comfyDate($this->inv['date_billed'])),
			array($tax_id),
			array(str_replace(array("[x]", "[y]"), array($this->getGroupPageNo(), $this->getPageGroupAlias()), _INVOICE_PDF_PAGE))
		);
		$options = array(
			'font_size'=>12,
			'y_pos'=>70,
			'align'=>'R'
		);
		// Draw Invoice Date and Page #
		$this->drawTable($data, $options);
		
		// Set Address
		$address = $this->inv['billing']['fname'] . " " . $this->inv['billing']['lname'] . "\n";
		if (strlen($this->inv['billing']['company']) > 0)
			$address .= $this->inv['billing']['company'] . "\n";
		$address .= $this->inv['billing']['address'] . "\n";
		if (strlen($this->inv['billing']['address2']) > 0)
			$address .= $this->inv['billing']['address2'] . "\n";
		$address .= $this->inv['billing']['city'] . ", " . $this->inv['billing']['state'] . " " . $this->inv['billing']['zip'] . " " . $this->inv['billing']['country'];
		
		$data = array(
			array($address)
		);
		$options = array(
			'font_size'=>12,
			'x_pos'=>52,
			'y_pos'=>155,
			'col'=> array(
				array('width'=>210)
			)
		);
		// Draw Address
		$this->drawTable($data, $options);
		
		// Set Invoice number/Customer ID
		$data = array(
			array(_INVOICE_PDF_INVNUMBER . $this->inv['id'] . "\n" . _INVOICE_PDF_CUSTOMERID . $this->inv['client_id'])
		);
		$options = array(
			'font_size'=>12,
			'x_pos'=>395,
			'y_pos'=>170
		);
		// Draw Invoice number/Customer ID
		$this->drawTable($data, $options);
		
		// Set Change tallies
		$data = array(
			array(
				'previous_balance'=>$this->format_currency($this->inv['previous_balance'], $this->inv['currency']),
				'credits'=>$this->format_currency($this->inv['amount_applied'], $this->inv['currency']),
				'current_charges'=>$this->format_currency($this->inv['total'], $this->inv['currency']),
				'total_due'=>$this->format_currency($this->inv['amount_due'], $this->inv['currency']),
				'due_date'=>comfyDate($this->inv['date_due']),
			)
		);
		$options = array(
			'font_size'=>10,
			'x_pos'=>22,
			'y_pos'=>268,
			'align'=>"C",
			'col'=> array(
				'previous_balance' => array('width'=>110),
				'credits' => array('width'=>110),
				'current_charges' => array('width'=>113),
				'total_due' => array('width'=>106),
				'due_date' => array('width'=>112),
			)
		);
		$this->drawTable($data, $options);
	}
	
	public function Footer() {
		// Draw notes
		$data = array(
			array($this->company_info['terms'])
		);
		$options = array(
			'font_size'=>10,
			'border'=>0,
			'x_pos'=>50,
			'y_pos'=>-135
		);
		$this->drawTable($data, $options);
	}
	
	/**
	 * Draws a complete invoice
	 */
	private function drawInvoice() {
		$options = array(
			'font_size'=>10,
			'x_pos'=>35,
			'y_pos'=>$this->GetY(),
			'type' => 'multicell',
			'text_color'=> array(0, 0, 0),
			'border'=>0,
			'col'=> array(
				'name'=> array(
					'width'=>420
				),
				'price'=> array(
					'width'=>100,
					'align'=>'R'
				),
			)
		);
		
		$lines = array();
		for ($i=0; $i<count($this->inv['line_items']); $i++) {
			$lines[$i]['name'] = $this->inv['line_items'][$i]['name'];
			
			// Format the line item price for this currency and tax type
			// Sales is formatted without tax applied to line items
			if ($this->inv['line_items'][$i]['tax1']['type'] == "sales")
				$lines[$i]['price'] = $this->format_currency($this->inv['line_items'][$i]['price'], $this->inv['currency']);
			// VAT and GST include tax in line items ('total' is line item 'price' with tax included)
			else
				$lines[$i]['price'] = $this->format_currency($this->inv['line_items'][$i]['total'], $this->inv['currency']);
		}
		
		$this->drawTable($lines, $options);
		
		// Add tax lines, if needed
		if ($this->inv['tax1_total'] > 0) {
			$tax_lines[0]['name'] = $this->inv['tax1_name'] . " " . $this->inv['tax1_rate'] . _PERCENTAGE;
			$tax_lines[0]['price'] = $this->format_currency($this->inv['tax1_total'], $this->inv['currency']);
			
			if ($this->inv['tax2_total'] > 0) {
				$tax_lines[1]['name'] = $this->inv['tax2_name'] . " " . $this->inv['tax2_rate'] . _PERCENTAGE;
				$tax_lines[1]['price'] = $this->format_currency($this->inv['tax2_total'], $this->inv['currency']);
			}
			
			$options['y_pos'] = $this->GetY()+20;
			$options['col']['name']['align'] = "R";
			$this->drawTable($tax_lines, $options);
		}
		
		// Draw public notes
		if (strlen($this->inv['public_notes']) > 0)
			$this->drawTable(array(array($this->inv['public_notes'])), array('font_size'=>10,'x_pos'=>35,'y_pos'=>$this->getY()+20));
	}
	
	public function makeInvoice($invoice_data) {
		$total_invs = count($invoice_data);
		// Loop through all of the given invoices
		for ($i=0; $i<$total_invs; $i++) {
			// Set the invoice data for this invoice
			$this->inv = $invoice_data[$i];
			
			$this->startPageGroup();
			$this->AddPage();

			// Draw all line items for this invoice
			$this->drawInvoice();
		}
	}
	
	/**
	 * Returns the current PDF document
	 * 
	 * @return string The PDF document
	 */
	public function fetch() {
		return $this->Output($this->inv['id'] . ".pdf", 'S');
	}
	
	/**
	 * Streams the current PDF document
	 */
	public function stream() {
		$this->Output($this->inv['id'] . ".pdf", 'I');
	}
	
	/**
	 * Shortcut for formatting currencies
	 *
	 * @param float $value The value to be formated
	 * @param array $currency The currency to format $value into
	 * @return string The formatted currency value
	 */
	private function format_currency($value, $currency) {
		return format_currency($value, true, false, false, $currency);
	}	
	
	/**
	 * Draws a table using the given data
	 * @param array An array of 'column' => values
	 * @param array An array of options affecting the table and it's columns
	 */
	private function drawTable($data=array(), $options=null) {
		$opt = array(
			'type' => 'multicell', // Accepted types: multicell, cell
			'x_pos' => $this->GetX(),
			'y_pos' => $this->GetY(),
			'border' => 0,
			'align' => 'L',
			'text_color' => array(0, 0, 0),
			'font_size' => $this->getFontSize(),
		);
		
		// Overwrite default options
		if (is_array($options)) {
			foreach ($options as $key => $value) {
				$opt[$key] = $value;
			}
		}
		
		// Fetch the original font-size (to be restored)
		$orig_font_size = $this->getFontSize();
		
		// Set the location of this table
		$this->SetXY($opt['x_pos'], $opt['y_pos']);
		
		// Insert rows
		for ($i=0; $i<count($data); $i++) {
			
			// Set X position of this row
			$this->SetX($opt['x_pos']);
			
			if (is_array($data[$i])) {
				$j = count($data[$i]);
				$end_row = false;
				
				$max_y = 0; // Max Y pos of the tallest cell in this row
				$page_start = $this->getPage();
				$y_start = $this->GetY();
				$y_end = array();
				$page_end = array();
				
				foreach ($data[$i] as $col => $text) {
					if (--$j == 0)
						$end_row = true;
					
					// Determine line style for this cell
					$line_style = (isset($opt['col'][$col]['line_style']) ? $opt['col'][$col]['line_style'] : (isset($opt['line_style']) ? $opt['line_style'] : null));
					if (is_array($line_style))
						$this->SetLineStyle($opt['line_style']);
					
					// Determine fill color for this cell
					$fill_color = (isset($opt['col'][$col]['fill_color']) ? $opt['col'][$col]['fill_color'] : (isset($opt['fill_color']) ? $opt['fill_color'] : null));
					$fill = 0; // transparent by default (0)
					// Set fill color, if available
					if (is_array($fill_color)) {
						$this->SetFillColorArray($fill_color);
						$fill = 1; // color given, so fill it (1 non-transparent)
					}
					
					// Determin text color for this cell
					$text_color = (isset($opt['col'][$col]['text_color']) ? $opt['col'][$col]['text_color'] : (isset($opt['text_color']) ? $opt['text_color'] : null));
					// Set text color, if available
					if (is_array($text_color))
						$this->SetTextColorArray($text_color);
					
					// Set font size specified
					$font_size = (isset($opt['col'][$col]['font_size']) ? $opt['col'][$col]['font_size'] : (isset($opt['font_size']) ? $opt['font_size'] : null));
					if (is_numeric($font_size))
						$this->SetFontSize($font_size);
					
					$w = (isset($opt['col'][$col]['width']) ? $opt['col'][$col]['width'] : (isset($opt['width']) ? $opt['width'] : 0));
					$h = (isset($opt['col'][$col]['height']) ? $opt['col'][$col]['height'] : (isset($opt['height']) ? $opt['height'] : 0));
					$border = (isset($opt['col'][$col]['border']) ? $opt['col'][$col]['border'] : $opt['border']);
					$text_align = (isset($opt['col'][$col]['align']) ? $opt['col'][$col]['align'] : $opt['align']);
					
					// Set page to begin drawing cell on
					$this->setPage($page_start);
					
					if ($opt['type'] == 'cell')
						$this->Cell($w, $h, $text, $border, ($end_row ? 1 : 0), $text_align, $fill);
					else {
						if ($end_row)
							$this->MultiCell($w, $h, $text, $border, $text_align, $fill, 1, $this->GetX(), $y_start);
						else
							$this->MultiCell($w, $h, $text, $border, $text_align, $fill, 2, $this->GetX(), $y_start);
					}
					
					// Record page and y position cell finished drawing on
					$page_end[] = $this->getPage();
					$y_end[] = $this->GetY();
				}
				
				$y_new = $y_end[count($y_end)-1];
				
				// set the new row position by case
				if (max($page_end) == $page_start)
					$y_new = max($y_end);
				elseif ($page_end[0] == $page_end[count($page_end)-1])
					$y_new = max($y_end);
				elseif ($page_end[0] > $page_end[count($page_end)-1])
					$y_new = $y_end[0];
				
				$this->setPage(max($page_end));

				$this->SetY($y_new);
			}
		}

		$this->SetFontSize($orig_font_size);
	}
}
?>