<?php
/*******************************************************************************
 @name CentovaCast Module
 @author Paul Phillips
 @date Dec 7, 2009
 @version 1.0
 @notes For use with Blesta version 2.1.0+
*******************************************************************************/

class centovacast implements ModuleInterface {
	private static $module;

	public function __construct() {
		// Adds a new centovacast that can be added, edited, or deleted
		self::$module = new Module("centovacast", true, true, true);
		
		self::$module->addDBField("varchar(255)", "hostname", "Hostname", "yourhostname.com");
		self::$module->addDBField("varchar(255)", "username", "Username", "username");
		self::$module->addDBField("varchar(255)", "password", "Password", "password");
		
		self::$module->addModuleHTMLField("text", "Hostname", "hostname");
		self::$module->addModuleHTMLField("text", "Username", "username");
		self::$module->addModuleHTMLField("text", "Password", "password");
		
		self::$module->setModuleRowSelect("Hostname", "hostname");
		
		// Module Options while creating a package with this module
		// NOTE: Only one field is accepted of this type and must be named "mstring"
		self::$module->addPackageHTMLField("text", "Template Name", "mstring", "^.+", "Template Name must not be left empty.");
	
		// This adds a note to be displayed along with the above package field
		self::$module->addPackageNote("mstring", "Enter the Template Name as configured in CentovaCast, i.e. \"Template-1\".");
		
		// These are shown as column names for each service of this type.
		self::$module->addServiceColumn("user1", "Username");
		self::$module->addServiceColumn("pass", "Password");
		self::$module->addServiceColumn("user2", "IP Address");
		self::$module->addServiceColumn("opt1", "Port");

		// These are the fields that may be edited when performing a local edit on this service
		self::$module->addServiceEditField("user1", "Username");
		self::$module->addServiceEditField("user2", "IP Address");
		self::$module->addServiceEditField("pass", "Password");
		self::$module->addServiceEditField("opt1", "Port");
		self::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
		self::$module->addServiceEditField("dated", "Date Deleted");
		self::$module->addServiceEditField("datep", "Date Suspended");
		self::$module->addServiceEditField("dater", "Date Renews");
		self::$module->addServiceEditField("notes", "Notes");
		
		// These displayed as tags and are usable when creating/editing a packages welcome e-mail
	  	self::$module->addServiceTag("user1", "username");
		self::$module->addServiceTag("pass", "password");
		self::$module->addServiceTag("user2", "ipaddress");
		self::$module->addServiceTag("opt1", "port");
		self::$module->addServiceTag("hostname", "hostname");
		
		// Fields that are shown to the admins when adding a service that uses this module from within the control panel
		//self::$module->addServiceHTMLField("text", "Hostname", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The hostname you entered does not appear to be valid.");
		self::$module->addServiceHTMLField("text", "Username", "user1", "^[a-z0-9]{2,12}$", "The username you entered is not valid, please choose something between 2 and 12 characters.");
		self::$module->addServiceHTMLField("text", "Password", "pass", "^.{5,14}$", "Passwords must be between 5 and 14 characters.");

		// Fields that are shown to the user when adding a service that uses this module from within the control panel
		//self::$module->addUserServiceHTMLField("text", "Hostname", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The hostname you entered does not appear to be valid.");
		self::$module->addUserServiceHTMLField("text", "Username", "user1", "^[a-z0-9]{2,12}$", "The username you entered is not valid, please choose something between 2 and 12 characters.");
		self::$module->addUserServiceHTMLField("text", "Password", "pass", "^.{5,14}$", "Passwords must be between 5 and 14 characters.");
		
		
		// Specify which field from the module table to display in the drop down and give it a friendly name
		// By default items will be shown by ID and have the friendly name of "Module ID".
		// This appears when adding or editing a service for a user.
		self::$module->setModuleRowSelect("Server", "hostname");
		
		// Creates the module, adds the module to the database if the module table does not
		// already exist. WARNING: if a module has already been created and you attempt to
		// add additional fields they WILL NOT BE ADDED, and any refrence to those fields
		// WILL RESULT IN AN ERROR.
		self::$module->createModule();
	}

	// Returns the module object, this function MUST be present
	public function getModule() {
		return self::$module;
	}
	
	/*******************************************************************************
	********************************************************************************
	* This is the MAGIC function. This function automatically creates a service    *
	* for a user.                                                                  *
	*                                                                              *
	* IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
	* NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
	*                                                                              *
	* IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
	* THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
	********************************************************************************
	*******************************************************************************/

	public function promodule($action, $serviceInfo, $packageInfo) {
	// packageInfo comes from packagesetinfos() function
	$template = $packageInfo[4][2];

	$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
	$data = centovacast::$module->getRows($conditions);

	// We are only expecting 1 row returned, so take the first element of this
	// 2-dimensional array
	$this->hostname = $data[0]['hostname'];
	$this->username = $data[0]['username'];
	$this->password = $data[0]['password'];

		switch ($action) {
			case "add":
				
				$userinfo = $this->parse_data($this->checkuser($this->hostname, $this->username, $this->password, $serviceInfo));
				
				if ($userinfo[0]['username'] == $serviceInfo['user1']) {
					// ERROR USERNAME TAKEN
					return false;
				} else {
					if ($this->add($this->hostname, $this->username, $this->password, $serviceInfo, $template)) {
						
						// build extra service info details - ip/port etc and return
						// Fetch account info
						$acctinfo = $this->parse_data($this->getaccount($this->hostname, $this->username, $this->password, $serviceInfo));
						$serviceInfo['user1'] = $acctinfo[0]['username'];
						$serviceInfo['pass'] = $acctinfo[0]['adminpassword'];
						$serviceInfo['user2'] = $acctinfo[0]['ipaddress'];
						$serviceInfo['opt1'] = $acctinfo[0]['port'];
						
						return $serviceInfo;
					
					} else {
						return false;
					}
				}

				//die;
			
			break;
			case "remove":
				return $this->remove($this->hostname, $this->username, $this->password, $serviceInfo);
			break;
			case "suspend":
				return $this->suspend($this->hostname, $this->username, $this->password, $serviceInfo);
			break;
			case "unsuspend":
				return $this->unsuspend($this->hostname, $this->username, $this->password, $serviceInfo);
			break;
		}

		return false;
	}


	/* ------------------------------------------------------------------------------------------------
	The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	/*
	 * grabs the IP/port info from an existing user
	 * @return array containing the IP/port
	 */
	private function getaccount($hostname, $username, $password, $serviceInfo) {
		//$password = ($username!='admin') ? $username.'|'.$password : $password;
		$password = $username.'|'.$password;
	$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="server" method="getaccount">
			<password>$password</password>
			<username>{$serviceInfo['user1']}</username>
		</request>
	</centovacast>
EOT;
		return $this->request($this->hostname, $xml);
	}
	 
	 
	/*
	 * Checks if a user already exists on this server
	 * @return true on available, false otherwise
	 */
	 private function checkuser($hostname, $username, $password, $serviceInfo) {
		$password = ($username!='admin') ? $username.'|'.$password : $password;
	$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="system" method="info">
			<password>$password</password>
			<username>{$serviceInfo['user1']}</username>
		</request>
	</centovacast>
EOT;

		return $this->request($this->hostname, $xml);
	 }
	 
	/*
	* Adds a user for the given domain and plan.
	* @return true on success, false otherwise
	*/
	private function add($hostname, $username, $password, $serviceInfo, $template) {
		$password = ($username!='admin') ? $username.'|'.$password : $password;
	$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="system" method="provision">
			<password>$password</password>
			<hostname>$hostname</hostname>
			<username>{$serviceInfo['user1']}</username>
			<adminpassword>{$serviceInfo['pass']}</adminpassword>
			<sourcepassword>{$serviceInfo['pass']}</sourcepassword>
			<ipaddress>auto</ipaddress>
			<port>auto</port>
			<template>$template</template>
			<autorebuildlist>1</autorebuildlist>
			<autostart>1</autostart>
			<email>no-reply@domain.com</email>
		</request>
	</centovacast>
EOT;

		$result = $this->request($this->hostname, $xml);
		if (preg_match("/success/i", $result))
				return true;
		return false;
	}

	/*
	* Removes a user from the server.
	* @return true on success, false otherwise
	*/
	private function remove($hostname, $username, $password, $serviceInfo) {
		$password = ($username!='admin') ? $username.'|'.$password : $password;
	$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="system" method="terminate">
			<password>$password</password>
			<username>{$serviceInfo['user1']}</username>
		</request>
	</centovacast>
EOT;

		$response = $this->request($this->hostname, $xml);
		
		return preg_match("/<response type=\"success\">/i", $response) ? true : false;
	}

	/*
	* Suspends a user on the server.
	* @return true on success, false otherwise
	*/
	private function suspend($hostname, $username, $password, $serviceInfo) {
		$password = ($username!='admin') ? $username.'|'.$password : $password;
	$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="system" method="setstatus">
			<password>$password</password>
			<username>{$serviceInfo['user1']}</username>
			<status>disabled</status>
		</request>
	</centovacast>
EOT;

		$response = $this->request($this->hostname, $xml);
		
		return preg_match("/<response type=\"success\">/i", $response) ? true : false;
	}

	/*
	* Unsuspends a user on the server.
	* @return true on success, false otherwise
	*/
	private function unsuspend($hostname, $username, $password, $serviceInfo) {
		$password = ($username!='admin') ? $username.'|'.$password : $password;
	$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="system" method="setstatus">
			<password>$password</password>
			<username>{$serviceInfo['user1']}</username>
			<status>enabled</status>
		</request>
	</centovacast>
EOT;

		$response = $this->request($this->hostname, $xml);

		// Also try to start it up after an unsuspend
		$xml = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
	<centovacast>
		<request class="server" method="start">
			<password>$password</password>
			<username>{$serviceInfo['user1']}</username>
		</request>
	</centovacast>
EOT;

		@$this->request($this->hostname, $xml);

		
		return preg_match("/<response type=\"success\">/i", $response) ? true : false;
	}

	/*
	 * Parse XML response
	 * @return array fields
	 */
	private function parse_data($data) {
		if (!preg_match('/<data[^\>]*?>([\s\S]+)<\/data>/i',$data,$matches)) return false;
		list(,$rowxml) = $matches;
		
		$rows = array();
		if (!preg_match_all('/<row[^\>]*?>([\s\S]*?)<\/row>/i',$rowxml,$rowmatches,PREG_SET_ORDER)) return $rows;

		foreach ($rowmatches as $k=>$rowmatch) {
			$fields = array();
			list(,$fieldxml) = $rowmatch;
			
			if (preg_match_all('/<field(?:\s+name\s*=\s*"([^"]*?)")?[^\>]*?>([\s\S]*?)<\/field>/i',$fieldxml,$fieldmatches,PREG_SET_ORDER)) {
				foreach ($fieldmatches as $k=>$fieldmatch) {
					list(,$fieldname,$fieldvalue) = $fieldmatch;
					if (strlen($fieldname)) {
						$fields[ $fieldname ] = $fieldvalue;
					} else {
						$fields[] = $fieldvalue;
					}
				}
			}
			
			$rows[] = $fields;
			
		}
		return $rows;
	}


	/*
	* Prepares CURL to perform Centova Cast API request
	* @return response
	*/
	private function request($hostname, $xml) {
		// Log data sent to CentovaCast
		centovacast::$module->logData($xml, "input", true);

		$headers = array("Content-Type: text/xml");

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, "http://" . $hostname . "/api.php");
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

		$response = curl_exec($ch);

		//debug
		//echo $response;
		//print_r($this->parse_data($response));
		//die;

		// Log data received by CentovaCast
		centovacast::$module->logData($response, "output", (preg_match("/<response type=\"success\">/i", $response) ? true : false));
		curl_close($ch);

		return $response;
	}
}

?>