<?php
/*******************************************************************************
 @name cPanel Module
 @author Cody Phillips
 @date July 1, 2007
 @version 1.0
 @notes For use with Blesta version 0.9.0+
*******************************************************************************/

class cpanel implements ModuleInterface {
     private static $module;

     public function __construct() {
          // Adds a new cpanel that can be added, edited, or deleted
          cpanel::$module = new Module("cpanel", true, true, true);

          // Adds the given fields to the database for this module if it has not already
          // been created. WARNING: if the module has already been created with the name
          // given in the Module constructor you must delete the old module before the
          // new one will be created. This may have an adverse effect on services.
          cpanel::$module->addDBField("varchar(255)", "hostn", "Server");
          cpanel::$module->addDBField("varchar(255)", "user", "User");
          cpanel::$module->addDBField("text", "remotekey", "Key");
          cpanel::$module->addDBField("varchar(5)", "usessl", "Use SSL", "true");
          cpanel::$module->addDBField("int(11)", "max", "Limit", "0");
          cpanel::$module->addDBField("int(11)", "cur", "Current", "0");

          // Options for creating a new module row
          // Example: cPanel module may store multiple different servers. The HTML Fields below
          // allow you to create/edit a server for the cPanel module
          // NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
          cpanel::$module->addModuleHTMLField("text", "Server Name", "hostn");
          cpanel::$module->addModuleHTMLField("text", "User Name", "user");
          cpanel::$module->addModuleHTMLField("textarea", "Remote Key", "remotekey");
          cpanel::$module->addModuleHTMLField("select", "Use SSL", "usessl");
               cpanel::$module->addModuleHTMLSelectOption("usessl", "Yes", "true");
               cpanel::$module->addModuleHTMLSelectOption("usessl", "No", "false");
          cpanel::$module->addModuleHTMLField("text", "Account Limit", "max", "^[0-9]+", "Account Limit must be an integer.");
          cpanel::$module->addModuleHTMLField("text", "Current Accounts", "cur", "^[0-9]+", "Current Accounts must be an integer.");

          // Module Options while creating a package with this module
          // NOTE: Only one field is accepted of this type and must be named "mstring"
          cpanel::$module->addPackageHTMLField("text", "Plan Name", "mstring", "^.+", "Plan Name must not be left empty.");

          // This adds a note to be displayed along with the above package field
          cpanel::$module->addPackageNote("mstring", "Enter the Plan Name as configured in WHM, i.e. \"Bronze\".");

          // Specify which field from the module table to display in the drop down and give it a friendly name
          // By default items will be shown by ID and have the friendly name of "Module ID".
          // This appears when adding or editing a service for a user.
          cpanel::$module->setModuleRowSelect("Server", "hostn");

          // These are shown as column names for each service of this type.
          cpanel::$module->addServiceColumn("user2", "User Name");
          cpanel::$module->addServiceColumn("pass", "Password");
          cpanel::$module->addServiceColumn("hostn", "Server");

          // These are displayed as tags and are usable when creating/editing a packages welcome e-mail
          cpanel::$module->addServiceTag("user1", "domain");
          cpanel::$module->addServiceTag("user2", "username");
          cpanel::$module->addServiceTag("pass", "pass");
          cpanel::$module->addServiceTag("term", "term");
          cpanel::$module->addServiceTag("hostn", "server");

          // These are the fields that may be edited when performing a local edit on this service
          cpanel::$module->addServiceEditField("user1", "Domain");
          cpanel::$module->addServiceEditField("user2", "User Name");
          cpanel::$module->addServiceEditField("pass", "Password");             
          cpanel::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
          cpanel::$module->addServiceEditField("hostn", "Server");
          cpanel::$module->addServiceEditField("dated", "Date Deleted");
          cpanel::$module->addServiceEditField("datep", "Date Suspended");
          cpanel::$module->addServiceEditField("dater", "Date Renews");
          cpanel::$module->addServiceEditField("notes", "Notes");


          // Fields that are shown to the admins when adding a service that uses this module.
          cpanel::$module->addServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
          cpanel::$module->addServiceHTMLField("text", "Password", "pass", "^.{4,16}$", "Passwords must be between 4 and 16 characters.");

          // Fields that are shown to the user when adding a service that uses this module.
          cpanel::$module->addUserServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
          cpanel::$module->addUserServiceHTMLField("text", "Password", "pass", "^.{4,16}$", "Passwords must be between 4 and 16 characters.");


          // Creates the module, adds the module to the database if the module table does not
          // already exist. WARNING: if a module has already been created and you attempt to
          // add additional fields they WILL NOT BE ADDED, and any refrence to those fields
          // WILL RESULT IN AN ERROR.
          cpanel::$module->createModule();
     }

     // Returns the module object, this function MUST be present
     public function getModule() {
          return cpanel::$module;
     }

     /*******************************************************************************
     ********************************************************************************
     * This is the MAGIC function. This function automatically creates a service    *
     * for a user.                                                                  *
     *                                                                              *
     * IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
     * NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
     *                                                                              *
     * IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
     * THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
     ********************************************************************************
     *******************************************************************************/

     public function promodule($action, $serviceInfo, $packageInfo) {
	  // packageInfo comes from packagesetinfos() function
	  $plan = $packageInfo[4][2];

	  $conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
	  $data = cpanel::$module->getRows($conditions);

	  // We are only expecting 1 row returned, so take the first element of this
	  // 2-dimensional array
	  $host = $data[0]['hostn'];
	  $key = $data[0]['remotekey'];
	  $reselleruser = $data[0]['user'];
	  $usessl = $data[0]['usessl'];

	  switch ($action) {
	       case "add":
		    // Ensure we do not exceed the maximum number of accounts for this server.
		    if ($data[0]['cur'] >= $data[0]['max'])
			 return false;

		    // All user names should be up to the first 8 characters of the domain name
		    $serviceInfo['user2'] = cpanel::makeUserName($serviceInfo['user1']);

		    if (DEMO)
			 $result = "wwwacct creation finished";
		    else
			 $result = cpanel::createacct($host, $reselleruser, $key, $usessl, $serviceInfo['user1'], $serviceInfo['user2'], $serviceInfo['pass'], $plan);
		    //if (eregi("wwwacct creation finished", $result)) {
			if (preg_match("/wwwacct creation finished/i", $result)) {
			 // Update the current number of accounts on this server
			 cpanel::$module->updateDBFields("cur", $data[0]['cur']+1, $serviceInfo['mid']);

                          // Log data received from cPanel
                          cpanel::$module->logData($result, "output", true);                          
                          
			 return $serviceInfo;  // return the info that we want to be inserted into the services table
		    }                   
		    break;
	       case "remove":
		    if (DEMO)
			 $result = "Done";
		    else
			 $result = cpanel::killacct($host, $reselleruser, $key, $usessl, $serviceInfo['user2']);
		    //if (eregi("Done", $result)) {
			if (preg_match("/Done/i", $result)) {
			 // Update the current number of accounts on this server
			 cpanel::$module->updateDBFields("cur", $data[0]['cur']-1, $serviceInfo['mid']);                         

                          // Log data received from cPanel
                          cpanel::$module->logData($result, "output", true);                          
                          
			 return true;
		    }
		    break;
	       case "suspend":
		    if (DEMO)
			 $result = "account has been suspended";
		    else
			 $result = cpanel::suspendacct($host, $reselleruser, $key, $usessl, $serviceInfo['user2']);
		    //if (eregi("account has been suspended", $result)) {
			if (preg_match("/account has been suspended/i", $result)) {

                          // Log data received from cPanel
                          cpanel::$module->logData($result, "output", true);
                                               
			 return true;
                     }
		    break;
	       case "unsuspend":
		    if (DEMO)
			 $result = "account is now active";
		    else
			 $result = cpanel::unsuspendacct($host, $reselleruser, $key, $usessl, $serviceInfo['user2']);
		    //if (eregi("account is now active", $result)) {
			if (preg_match("/account is now active/i", $result)) {

                          // Log data received from cPanel
                          cpanel::$module->logData($result, "output", true);
                                               
			 return true;
                     }
		    break;
	  }

                     
           // Log failed response from cPanel
           cpanel::$module->logData($result, "output", false);                                 
	  return false;
     }


     /* ------------------------------------------------------------------------------------------------
	The following are support functions for this module
     --------------------------------------------------------------------------------------------------*/
     private static function suspendacct($host,$user,$accesshash,$usessl,$suspenduser) {
	  $result = cpanel::whmreq("/scripts/remote_suspend?user=${suspenduser}",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }
	  return $result;
     }
     private static function unsuspendacct($host,$user,$accesshash,$usessl,$suspenduser) {
	  $result = cpanel::whmreq("/scripts/remote_unsuspend?user=${suspenduser}",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }
	  return $result;
     }
     private static function killacct($host,$user,$accesshash,$usessl,$killuser) {
	  $result = cpanel::whmreq("/scripts/killacct?user=${killuser}&nohtml=1",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }
	  return $result;
     }
     private static function showversion ($host,$user,$accesshash,$usessl) {
	  $result = cpanel::whmreq("/scripts2/showversion",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }
	  return $result;
     }
     private static function createacct ($host,$user,$accesshash,$usessl,$acctdomain,$acctuser,$acctpass,$acctplan) {
	  $result = cpanel::whmreq("/scripts/wwwacct?remote=1&nohtml=1&username=${acctuser}&password=${acctpass}&domain=${acctdomain}&plan=${acctplan}",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }
	  return $result;
     }

     private static function listaccts ($host,$user,$accesshash,$usessl) {
	  $result = whmreq("/scripts2/listaccts?nohtml=1&viewall=1",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }

	  //$page = split("\n",$result);
	  $page = preg_split("/\n/",$result);
	   foreach ($page as $line) {
	       //list($acct,$contents) = split("=", $line);
		   list($acct,$contents) = preg_split("/=/", $line);
	       if ($acct != "") {
		    //$allc = split(",", $contents);
			$allc = preg_split("/,/", $contents);
		    $accts[$acct] = $allc;
	       }
	  }
	 return($accts);
     }
     private static function listpkgs ($host,$user,$accesshash,$usessl) {
	  $result = whmreq("/scripts/remote_listpkg",$host,$user,$accesshash,$usessl);
	  if ($cpanelaccterr != "") { return; }

	  //$page = split("\n",$result);
	  $page = preg_split("/\n/",$result);
	  foreach ($page as $line) {
	       //list($pkg,$contents) = split("=", $line);
		   list($pkg,$contents) = preg_split("/=/", $line);
	       if ($pkg != "") {
		    //$allc = split(",", $contents);
			$allc = preg_split("/,/", $contents);
		    $pkgs[$pkg] = $allc;
	       }
	  }
	 return($pkgs);
     }
     private static function whmreq ($request,$host,$user,$accesshash,$usessl) {
	  $cleanaccesshash = preg_replace("'(\r|\n)'","",$accesshash);
	  $authstr = $user . ":" . $cleanaccesshash;
	  $cpanelaccterr = ""; 
	  
	  if (function_exists("curl_init")) {
	       $ch = curl_init();
	       if ($usessl) {
		    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,0);
		    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,0);
		    curl_setopt($ch, CURLOPT_URL, "https://${host}:2087" . $request);
	       }
	       else {
		    curl_setopt($ch, CURLOPT_URL, "http://${host}:2086" . $request);
	       }
	       curl_setopt($ch, CURLOPT_HEADER, 0);
	       curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
	       
	       $curlheaders[0] = "Authorization: WHM $authstr";
	       curl_setopt($ch,CURLOPT_HTTPHEADER,$curlheaders);
	       
                // Log data sent out to cPanel
	       cpanel::$module->logData($request, "input", true);
                
	       $data=curl_exec ($ch);
                              
	       curl_close ($ch);
	  }
	  elseif (function_exists("socket_create")) {
	       if ($usessl) {
		    $cpanelaccterr = "SSL Support requires curl";
		    return;
	       }
	       $service_port = 2086;
	       $address = gethostbyname($host);
	       $socket = socket_create (AF_INET, SOCK_STREAM, 0);
	       if ($socket < 0) {
		    $cpanelaccterr = "socket_create() failed";
		    return;
	       }
	       $result = socket_connect ($socket, $address, $service_port);
	       if ($result < 0) {
		    $cpanelaccterr = "socket_connect() failed";
		    return;
	       }
	       $in = "GET $request HTTP/1.0\n";
	       socket_write($socket,$in,strlen($in));
	       
	       $in = "Connection: close\n";
	       socket_write($socket,$in,strlen($in));
	       
	       $in = "Authorization: WHM $authstr\n\n\n";
	       socket_write($socket,$in,strlen($in));
	       
	       $inheader = 1;
	       while(($buf = socket_read($socket, 512)) != false) {
		    if (!$inheader) {
			 $data .= $buf;
		    }
		    if(preg_match("'\r\n\r\n$'s", $buf)) {
			 $inheader = 0;
		    }
		    if(preg_match("'\n\n$'s", $buf)) {
			 $inheader = 0;
		    }
		    if(preg_match("'\r\n$'s", $buf)) {
			 $inheader = 0;
		    }
	       }  
	  }
	  else {
	       $cpanelaccterr = "php not compiled with --enable-sockets OR curl";
	       return;
	  }
	  
	  return $data;
     }

     // Creates a username that is between 5 and 8 characters long
     // Usernames that are generated randomly are 7 characters in length
     private static function makeUserName($domain) {
	  // Remove everything except letters from the domain
	  $temp = preg_replace('/[^a-zA-Z]/', '', $domain);
	  
	  // If what we have left to build the username is less than 5 characters
	  // add in some random characters to create a unique username.
	  if (strlen($temp) < 5) {
	       // Add 3-random characters followed by the rest of temp.
	       $chars = array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t',
	       'u','v','w','x','y','z');
	       $count = count($chars) - 1;
	   
	       for ($i=0; $i<3; $i++)
		    $temp = $chars[rand(0, $count)] . $temp;
	  }
	  
	  return substr($temp, 0, 8);
     }
}
?>