<?php
/*******************************************************************************
 @name DirectAdmin Module
 @author Cody Phillips
 @date July 21, 2009
 @version 1.3
 @notes For use with Blesta version 1.0.0+
*******************************************************************************/

class directadmin implements ModuleInterface {
	private static $module;

	public function __construct() {
		// Adds a new directadmin that can be added, edited, or deleted
		directadmin::$module = new Module("directadmin", true, true, true);

		// Adds the given fields to the database for this module if it has not already
		// been created. WARNING: if the module has already been created with the name
		// given in the Module constructor you must delete the old module before the
		// new one will be created. This may have an adverse effect on services.
		directadmin::$module->addDBField("varchar(255)", "serverip", "Server IP");
		directadmin::$module->addDBField("varchar(255)", "user", "User Name");
		directadmin::$module->addDBField("text", "password", "Password");
		directadmin::$module->addDBField("varchar(5)", "usessl", "Use SSL", "false");
		directadmin::$module->addDBField("int(11)", "max", "Limit", "0");
		directadmin::$module->addDBField("int(11)", "cur", "Current", "0");

		// Options for creating a new module row
		// Example: DirectAdmin module may store multiple different servers. The HTML Fields below
		// allow you to create/edit a server for the DirectAdmin module
		// NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
		directadmin::$module->addModuleHTMLField("text", "Server IP", "serverip");
		directadmin::$module->addModuleHTMLField("text", "User Name", "user");
		directadmin::$module->addModuleHTMLField("text", "Password", "password");
		directadmin::$module->addModuleHTMLField("select", "Use SSL", "usessl");
			directadmin::$module->addModuleHTMLSelectOption("usessl", "Yes", "true");
			directadmin::$module->addModuleHTMLSelectOption("usessl", "No", "false");
		directadmin::$module->addModuleHTMLField("text", "Account Limit", "max", "^[0-9]+", "Account Limit must be an integer.");
		directadmin::$module->addModuleHTMLField("text", "Current Accounts", "cur", "^[0-9]+", "Current Accounts must be an integer.");

		// Module Options while creating a package with this module
		// NOTE: Only one field is accepted of this type and must be named "mstring"
		directadmin::$module->addPackageHTMLField("text", "Package", "mstring", "^.+", "Package must not be left empty.");

		// This adds a note to be displayed along with the above package field
		directadmin::$module->addPackageNote("mstring", "Enter the Package as configured in DirectAdmin, i.e. \"Bronze\".");

		// Specify which field from the module table to display in the drop down and give it a friendly name
		// By default items will be shown by ID and have the friendly name of "Module ID".
		// This appears when adding or editing a service for a user.
		directadmin::$module->setModuleRowSelect("Server IP", "serverip");

		// These are shown as column names for each service of this type.
		directadmin::$module->addServiceColumn("user2", "User Name");
		directadmin::$module->addServiceColumn("pass", "Password");
		directadmin::$module->addServiceColumn("serverip", "Server IP");

		// These are displayed as tags and are usable when creating/editing a package's welcome e-mail
		directadmin::$module->addServiceTag("user1", "domain");
		directadmin::$module->addServiceTag("user2", "username");
		directadmin::$module->addServiceTag("pass", "pass");
		directadmin::$module->addServiceTag("term", "term");
		directadmin::$module->addServiceTag("serverip", "serverip");

		// These are the fields that may be edited when performing a local edit on this service
		directadmin::$module->addServiceEditField("user1", "Domain");
		directadmin::$module->addServiceEditField("user2", "User Name");
		directadmin::$module->addServiceEditField("pass", "Password");
		directadmin::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
		directadmin::$module->addServiceEditField("serverip", "Server");
		directadmin::$module->addServiceEditField("dated", "Date Deleted");
		directadmin::$module->addServiceEditField("datep", "Date Suspended");
		directadmin::$module->addServiceEditField("dater", "Date Renews");
		directadmin::$module->addServiceEditField("notes", "Notes");


		// Fields that are shown to the admins when adding a service that uses this module.
		directadmin::$module->addServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
		directadmin::$module->addServiceHTMLField("text", "Password", "pass", "^.{5,16}$", "Passwords must be between 5 and 16 characters.");
		directadmin::$module->addServiceHTMLField("text", "E-mail", "opt1", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");


		// Fields that are shown to the user when adding a service that uses this module.
		directadmin::$module->addUserServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
		directadmin::$module->addUserServiceHTMLField("text", "Password", "pass", "^.{5,16}$", "Passwords must be between 5 and 16 characters.");
		directadmin::$module->addUserServiceHTMLField("text", "E-mail", "opt1", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");


		// Creates the module, adds the module to the database if the module table does not
		// already exist. WARNING: if a module has already been created and you attempt to
		// add additional fields they WILL NOT BE ADDED, and any refrence to those fields
		// WILL RESULT IN AN ERROR.
		directadmin::$module->createModule();
	}

	// Returns the module object, this function MUST be present
	public function getModule() {
		return directadmin::$module;
	}

	/*******************************************************************************
	********************************************************************************
	* This is the MAGIC function. This function automatically creates a service    *
	* for a user.                                                                  *
	*                                                                              *
	* IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
	* NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
	*                                                                              *
	* IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
	* THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
	********************************************************************************
	*******************************************************************************/

	public function promodule($action, $serviceInfo, $packageInfo) {
		// packageInfo comes from packagesetinfos() function
		$plan = $packageInfo[4][2];
	
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = directadmin::$module->getRows($conditions);
	
		// We are only expecting 1 row returned, so take the first element of this
		// 2-dimensional array
		$host = $data[0]['serverip'];
		$key = $data[0]['password'];
		//$reselleruser = "admin|" . $data[0]['user']; // Sometimes this "admin|" is required for authentication
		$reselleruser = $data[0]['user'];
		$usessl = $data[0]['usessl'];
	
	
		switch ($action) {
			case "add":
				// Ensure we do not exceed the maximum number of accounts for this server.
				if ($data[0]['cur'] >= $data[0]['max'])
					return false;
				
				// All user names should be up to the first 8 characters of the domain name
				$serviceInfo['user2'] = directadmin::makeUserName($serviceInfo['user1']);
				
				$acct['user'] = $serviceInfo['user2'];
				$acct['email'] = $serviceInfo['opt1'];
				$acct['pass'] = $serviceInfo['pass'];
				$acct['domain'] = $serviceInfo['user1'];
				$acct['plan'] = $plan;
				
				if (DEMO) {
					$result = array();
					$result['error'] = "0";
				}
				else
					$result = directadmin::createacct($host, $reselleruser, $key, $usessl, $acct);
				
				if ($result['error'] == "0") {
					// Update the current number of accounts on this server
					directadmin::$module->updateDBFields("cur", $data[0]['cur']+1, $serviceInfo['mid']);
					
					directadmin::$module->logData($result, "output", true);
					return $serviceInfo;  // return the info that we want to be inserted into the services table
				}
				break;
			case "remove":
				$acct['user'] = $serviceInfo['user2'];
			
				if (DEMO) {
					$result = array();
					$result['error'] = "0";
				}
				else
					$result = directadmin::killacct($host, $reselleruser, $key, $usessl, $acct);
				
				if ($result['error'] == "0") {
					// Update the current number of accounts on this server
					directadmin::$module->updateDBFields("cur", $data[0]['cur']-1, $serviceInfo['mid']);
					
					directadmin::$module->logData($result, "output", true);
					return $serviceInfo;  // return the info that we want to be inserted into the services table
				}
				break;
			case "suspend":
				$acct['user'] = $serviceInfo['user2'];
				
				if (DEMO) {
					$result = array();
					$result['error'] = "0";
				}
				else
					$result = directadmin::suspendacct($host, $reselleruser, $key, $usessl, $acct);
				
				if ($result['error'] == "0") {
					directadmin::$module->logData($result, "output", true);
					return true;
				}
				break;
			case "unsuspend":
				$acct['user'] = $serviceInfo['user2'];
				
				if (DEMO) {
					$result = array();
					$result['error'] = "0";
				}
				else
					$result = directadmin::unsuspendacct($host, $reselleruser, $key, $usessl, $acct);
				
				if ($result['error'] == "0") {
					directadmin::$module->logData($result, "output", true);
					return true;
				}
				break;
		}

		directadmin::$module->logData($result, "output", false);
		return false;
	}


	/* ------------------------------------------------------------------------------------------------
	The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	private static function suspendacct($host, $user, $pass, $usessl, $acct) {
		$fields = array();
		$fields['location'] = "CMD_API_SELECT_USERS";
		$fields['dosuspend'] = "Suspend";
		$fields['select0'] = $acct['user'];
		
		// Get the status of the user
		$status = directadmin::getuserstatus($host, $user, $pass, $usessl, $acct);
		
		// If the user doesn't exist, or some other error occured, return
		if ($status['error'] == "1")
			return $status;
		
		$return_result = array();
		// Only attempt to suspend if the account is not already suspended.
		if ($status['suspended'] == "no") {
			$result = directadmin::req("CMD_API_SELECT_USERS", $fields, $host, $user, $pass, $usessl);
			parse_str($result, $result);
			return $result;
		}
		else
			return array('error'=>"0");
	}

	private static function unsuspendacct($host, $user, $pass, $usessl, $acct) {
		$fields = array();
		$fields['location'] = "CMD_API_SELECT_USERS";
		$fields['dounsuspend'] = "Unsuspend";
		$fields['select0'] = $acct['user'];
	
		// Get the status of the user
		$status = directadmin::getuserstatus($host, $user, $pass, $usessl, $acct);

		// If the user doesn't exist, or some other error occured, return
		if ($status['error'] == "1")
			return $status;

		$return_result = array();
		// Only attempt to unsuspend if the account is already suspended.
		if ($status['suspended'] == "yes") {
			$result = directadmin::req("CMD_API_SELECT_USERS", $fields, $host, $user, $pass, $usessl);
			parse_str($result, $result);
			return $result;
		}
		else
			return array('error'=>"0");
	}

	private static function killacct($host, $user, $pass, $usessl, $acct) {
		$fields = array();
		$fields['location'] = "CMD_API_SELECT_USERS";
		$fields['delete'] = "yes";
		$fields['confirmed'] = "Confirm";
		$fields['select0'] = $acct['user'];
	
		$return_result = array();
		$result = directadmin::req("CMD_API_SELECT_USERS", $fields, $host, $user, $pass, $usessl);
		parse_str($result, $result);
		return $result;
	}

	private static function createacct($host, $user, $pass, $usessl, $acct) {
		$fields = array();
		$fields['action'] = "create";
		$fields['add'] = "Submit";
		$fields['username'] = $acct['user'];
		$fields['email'] = $acct['email'];
		$fields['passwd'] = $acct['pass'];
		$fields['passwd2'] = $acct['pass'];
		$fields['domain'] = $acct['domain'];
		$fields['package'] = $acct['plan'];
		$fields['ip'] = $host;
		$fields['notify'] = "no";
	
		// If this package exists as a user package, set it up as a user package,
		// otherwise attempt to add it as a reseller package.
		if (directadmin::isauserpackage($host, $user, $pass, $usessl, $fields['package']))
			$result = directadmin::req("CMD_API_ACCOUNT_USER", $fields, $host, $user, $pass, $usessl);
		else {
			// If shared, domain will use the server's main ip. assign will
			// use one of the reseller's ips. sharedreseller will set the
			// resellers ip to shared and assign the reseller to it.
			$fields['ip'] = "sharedreseller"; // shared, sharedreseller or assign.
			$result = directadmin::req("CMD_API_ACCOUNT_RESELLER", $fields, $host, $user, $pass, $usessl);
		}
		parse_str($result, $result);
		return $result;
	}

	private static function getuserstatus($host, $user, $pass, $usessl, $acct) {
		$fields = array();
		$fields['action'] = "create";
		$fields['add'] = "Submit";
		$fields['user'] = $acct['user'];
	
		$result = directadmin::req("CMD_API_SHOW_USER_CONFIG", $fields, $host, $user, $pass, $usessl);
		parse_str($result, $result);
		return $result;
	}

	private static function isauserpackage($host, $user, $pass, $usessl, $package) {
		$fields = array();
		$fields['package'] = $package;
	
		$result = directadmin::req("CMD_API_PACKAGES_USER", $fields, $host, $user, $pass, $usessl);
	
		if (strlen($result) < 2)
			return false;
		return true;
	}

	private static function req($command, $fields, $host, $user, $pass, $usessl) {
		// Conver the fields to a query string, if necessary
		$fields = (is_array($fields) ? http_build_query($fields) : $fields);
	
		$authstr = $user . ":" . $pass;
		$directadminaccterr = "";
	
		directadmin::$module->logData($command . "?" . $fields, "input", true);
	
		if (function_exists("curl_init")) {
			$ch = curl_init();
			if ($usessl == "true") {
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,0);
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,0);
				curl_setopt($ch, CURLOPT_URL, "https://" . $host . ":2222/" . $command . "?" . $fields);
			}
			else {
				curl_setopt($ch, CURLOPT_URL, "http://" . $host . ":2222/" . $command . "?" . $fields);
			}
   
			curl_setopt($ch, CURLOPT_HEADER, 0);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
   
			$curlheaders[0] = "Authorization: Basic " . base64_encode($authstr);
			curl_setopt($ch,CURLOPT_HTTPHEADER,$curlheaders);
			
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $fields);
			
			$data=curl_exec($ch);
			
			if ($command == "CMD_API_SELECT_USERS")
				directadmin::$module->logData($data, "output", true);
			
			curl_close($ch);
		}
		elseif (function_exists("socket_create")) {
			if ($usessl) {
				$directadminaccterr = "SSL Support requires curl";
				return;
			}
			$service_port = 2222;
			$address = gethostbyname($host);
			$socket = socket_create(AF_INET, SOCK_STREAM, 0);
			if ($socket < 0) {
				$directadminaccterr = "socket_create() failed";
				return;
			}
			$result = socket_connect($socket, $address, $service_port);
			if ($result < 0) {
				$directadminaccterr = "socket_connect() failed";
				return;
			}
			$in = "GET " . $request . " HTTP/1.0\n";
			socket_write($socket,$in,strlen($in));
   
			$in = "Connection: close\n";
			socket_write($socket,$in,strlen($in));
   
			$in = "Authorization: Basic " . base64_encode($authstr) . "\n\n\n";
			socket_write($socket,$in,strlen($in));
   
			$inheader = 1;
			while(($buf = socket_read($socket, 512)) != false) {
				if (!$inheader) {
					$data .= $buf;
				}
				if(preg_match("'\r\n\r\n$'s", $buf)) {
					$inheader = 0;
				}
				if(preg_match("'\n\n$'s", $buf)) {
					$inheader = 0;
				}
				if(preg_match("'\r\n$'s", $buf)) {
					$inheader = 0;
				}
			}
		}
		else {
			$directadminaccterr = "php not compiled with --enable-sockets OR curl";
			return;
		}
	
		return $data;
	}

	// Creates a username that is between 5 and 8 characters long
	// Usernames that are generated randomly are 7 characters in length
	private static function makeUserName($domain) {
		// Remove everything except letters from the domain
		$temp = preg_replace('/[^a-zA-Z]/', '', $domain);
	
		// If what we have left to build the username is less than 5 characters
		// add in some random characters to create a unique username.
		if (strlen($temp) < 5) {
			// Add 3-random characters followed by the rest of temp.
			$chars = array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t',
			'u','v','w','x','y','z');
			$count = count($chars) - 1;
   
			for ($i=0; $i<3; $i++)
				$temp = $chars[rand(0, $count)] . $temp;
		}
	
		return substr($temp, 0, 8);
	}
}
?>