<?php
/*******************************************************************************
 @name HyperVM Module
 @author Cody Phillips
 @date November 20, 2009
 @version 1.2
 @notes For use with Blesta version 1.0.0+
*******************************************************************************/

class hypervm implements ModuleInterface {
     private static $module;

     public function __construct() {
          // Adds a new hypervm that can be added, edited, or deleted
          hypervm::$module = new Module("hypervm", true, true, true);

          // Adds the given fields to the database for this module if it has not already
          // been created. WARNING: if the module has already been created with the name
          // given in the Module constructor you must delete the old module before the
          // new one will be created. This may have an adverse effect on services.
          hypervm::$module->addDBField("varchar(255)", "server", "Server");
          hypervm::$module->addDBField("varchar(255)", "user", "User Name");
          hypervm::$module->addDBField("varchar(255)", "password", "Password");
          hypervm::$module->addDBField("varchar(5)", "usessl", "Use SSL", "false");
          hypervm::$module->addDBField("int(11)", "max", "Limit", "0");
          hypervm::$module->addDBField("int(11)", "cur", "Current", "0");

          // Options for creating a new module row
          // Example: hypervm module may store multiple different servers. The HTML Fields below
          // allow you to create/edit a server for the hypervm module
          // NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
          hypervm::$module->addModuleHTMLField("text", "Server", "server");
          hypervm::$module->addModuleHTMLField("text", "User Name", "user");
          hypervm::$module->addModuleHTMLField("text", "Password", "password");
          hypervm::$module->addModuleHTMLField("select", "Use SSL", "usessl");
               hypervm::$module->addModuleHTMLSelectOption("usessl", "Yes", "true");
               hypervm::$module->addModuleHTMLSelectOption("usessl", "No", "false");
          hypervm::$module->addModuleHTMLField("text", "Account Limit", "max", "^[0-9]+", "Account Limit must be an integer.");
          hypervm::$module->addModuleHTMLField("text", "Current Accounts", "cur", "^[0-9]+", "Current Accounts must be an integer.");

          // Module Options while creating a package with this module
          // NOTE: Only one field is accepted of this type and must be named "mstring"
          hypervm::$module->addPackageHTMLField("text", "Configuration", "mstring", "^(.)+\|(.)+\|(.)+\|(.)+\|[0-9]+", "Configuration must contain 5 delimited fields.");

          // This adds a note to be displayed along with the above package field
          hypervm::$module->addPackageNote("mstring", "Enter: Server Type|Resource Plan|OS Template|Server|# of IPs<br /> (ex. openvz|my plan|centos-5-i386-afull|localhost|5)");

          // Specify which field from the module table to display in the drop down and give it a friendly name
          // By default items will be shown by ID and have the friendly name of "Module ID".
          // This appears when adding or editing a service for a user.
          hypervm::$module->setModuleRowSelect("Server", "server");

          // These are shown as column names for each service of this type.
          hypervm::$module->addServiceColumn("user1", "Virtual Machine");
          hypervm::$module->addServiceColumn("server", "Server");
          hypervm::$module->addServiceColumn("pass", "Password");
          hypervm::$module->addServiceColumn("opt1", "E-mail");

          // These are displayed as tags and are usable when creating/editing a package's welcome e-mail
          hypervm::$module->addServiceTag("user1", "vm"); // virtual machine created using a package of this module
          hypervm::$module->addServiceTag("user2", "physicalserver"); // the physical server this vm resides on
          hypervm::$module->addServiceTag("pass", "pass");
          hypervm::$module->addServiceTag("opt1", "email");
          hypervm::$module->addServiceTag("term", "term");
          hypervm::$module->addServiceTag("server", "server"); // the server used to create the vm

          // These are the fields that may be edited when performing a local edit on this service
          hypervm::$module->addServiceEditField("user1", "Virtual Machine");
          hypervm::$module->addServiceEditField("user2", "Physical Server"); // the physical server this vm resides one
          hypervm::$module->addServiceEditField("pass", "Password");
          hypervm::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
          hypervm::$module->addServiceEditField("server", "Server"); // the server used to create the vm
          hypervm::$module->addServiceEditField("dated", "Date Deleted");
          hypervm::$module->addServiceEditField("datep", "Date Suspended");
          hypervm::$module->addServiceEditField("dater", "Date Renews");
          hypervm::$module->addServiceEditField("notes", "Notes");


          // Fields that are shown to the admins when adding a service that uses this module.
          hypervm::$module->addServiceHTMLField("text", "Virtual Machine", "user1", "^[a-z0-9.]+$", "Virtual Machine names may only contain a-z, 0-9 and periods.");
          hypervm::$module->addServiceHTMLField("text", "Password", "pass", "^.{5,16}$", "Passwords must be between 5 and 16 characters.");
          hypervm::$module->addServiceHTMLField("text", "E-mail", "opt1", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");


          // Fields that are shown to the user when adding a service that uses this module.
          hypervm::$module->addUserServiceHTMLField("text", "Virtual Machine", "user1", "^[a-z0-9.]+$", "Virtual Machine names may only contain a-z, 0-9 and periods.");
          hypervm::$module->addUserServiceHTMLField("text", "Password", "pass", "^.{5,16}$", "Passwords must be between 5 and 16 characters.");
          hypervm::$module->addUserServiceHTMLField("text", "E-mail", "opt1", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");


          // Creates the module, adds the module to the database if the module table does not
          // already exist. WARNING: if a module has already been created and you attempt to
          // add additional fields they WILL NOT BE ADDED, and any refrence to those fields
          // WILL RESULT IN AN ERROR.
          hypervm::$module->createModule();
     }

     // Returns the module object, this function MUST be present
     public function getModule() {
          return hypervm::$module;
     }

     /*******************************************************************************
     ********************************************************************************
     * This is the MAGIC function. This function automatically creates a service    *
     * for a user.                                                                  *
     *                                                                              *
     * IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
     * NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
     *                                                                              *
     * IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
     * THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
     ********************************************************************************
     *******************************************************************************/

     public function promodule($action, $serviceInfo, $packageInfo) {
	  // packageInfo comes from packagesetinfos() function
	  $config = explode("|", $packageInfo[4][2]);

	  $conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
	  $data = hypervm::$module->getRows($conditions);

           $serviceInfo['user2'] = $config[3]; // Physical server (usually localhost)

	  switch ($action) {
	       case "add":
		    // Ensure we do not exceed the maximum number of accounts for this server.
		    if ($data[0]['cur'] >= $data[0]['max'])
			 return false;

		    if (DEMO)
			 return $serviceInfo;
		    elseif ($this->addVPS($serviceInfo['user1'], $serviceInfo['pass'], $serviceInfo['opt1'], $data, $config)) {
			 // Update the current number of accounts on this server
			 hypervm::$module->updateDBFields("cur", $data[0]['cur']+1, $serviceInfo['mid']);

			 return $serviceInfo;  // return the info that we want to be inserted into the services table
		    }
		    break;
	       case "remove":
                     if (DEMO)
                     	 return true;
                     elseif ($this->removeVPS($serviceInfo['user1'], $data, $config)) {
			 // Update the current number of accounts on this server
			 hypervm::$module->updateDBFields("cur", $data[0]['cur']-1, $serviceInfo['mid']);

                          return true;
                     }
		    break;
	       case "suspend":
                     if (DEMO)
                     	return true;
                     return $this->suspendVPS($serviceInfo['user1'], $data);
		    break;
	       case "unsuspend":
                     if (DEMO)
                     	return true;
                     return $this->unsuspendVPS($serviceInfo['user1'], $data);
		    break;
	  }

	  return false;
     }


	/* ------------------------------------------------------------------------------------------------
	The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	private function addVPS(&$vmName, &$vmPass, &$vmEmail, &$data, &$config) {
		$params = array();
		$params['login-class'] = "client";
		$params['login-name'] = $data[0]['user'];
		$params['login-password'] = $data[0]['password'];
		$params['output-type'] = "json";
		
		$params['class'] = "vps";
		$params['name'] = $vmName;
		$params['action'] = "add";
		$params['v-password'] = $vmPass;
		$params['v-type'] = $config[0];
		$params['v-num_ipaddress_f'] = $config[4];
		$params['v-contactemail'] = $vmEmail;
		$params['v-send_welcome_f'] = "off";
		$params['v-ostemplate'] = $config[2];
		$params['v-syncserver'] = $config[3];
		$params['v-plan_name'] = $config[1];
		
		return $this->request($data[0]['server'], $data[0]['usessl'], $params);
	}
	
	private function removeVPS(&$vmName, &$data) {
		$params = array();
		$params['login-class'] = "client";
		$params['login-name'] = $data[0]['user'];
		$params['login-password'] = $data[0]['password'];
		$params['output-type'] = "json";
		
		$params['class'] = "vps";
		$params['name'] = $vmName . ".vm";
		$params['action'] = "delete";
		
		return $this->request($data[0]['server'], $data[0]['usessl'], $params);
	}
	
	private function suspendVPS(&$vmName, &$data) {
		$params = array();
		$params['login-class'] = "client";
		$params['login-name'] = $data[0]['user'];
		$params['login-password'] = $data[0]['password'];
		$params['output-type'] = "json";
		
		$params['class'] = "vps";
		$params['name'] = $vmName . ".vm";
		$params['action'] = "update";
		$params['subaction'] = "disable";
		
		return $this->request($data[0]['server'], $data[0]['usessl'], $params);
	}
	
	private function unsuspendVPS(&$vmName, &$data) {
		$params = array();
		$params['login-class'] = "client";
		$params['login-name'] = $data[0]['user'];
		$params['login-password'] = $data[0]['password'];
		$params['output-type'] = "json";
		
		$params['class'] = "vps";
		$params['name'] = $vmName . ".vm";
		$params['action'] = "update";
		$params['subaction'] = "enable";
		
		return $this->request($data[0]['server'], $data[0]['usessl'], $params);
	}
	
	/*
	* Prepares and executes a CURL request that performs HyperVM API request
	* @return response
	*/
	private function request($host, &$usessl, &$params) {
		self::$module->logData($params, "input", true);
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, "http" . ($usessl == "true" ? "s" : "") . "://" . $host . "/webcommand.php");
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		
		$response = (array)json_decode(curl_exec($ch));
		curl_close($ch);
		
		self::$module->logData($response, "output", ($response['return'] == "success" ? true : false));
		
		if ($response['return'] == "success")
			return true;
		return false;
	}
}
?>