<?php
/*******************************************************************************
 @name OpenSRS Module
 @author Tyson Phillips
 @date November 12, 2010
 @version 1.0
 @notes For use with Blesta version 2.3.0+
*******************************************************************************/

class opensrs implements ModuleInterface {
	private static $module;
     
	public function __construct() {
		global $mysqldb;
		// Get values for countries and states
		$countries = countriesinfo($mysqldb);
		$states = statesinfo($mysqldb);
		
		$num_countries = count($countries);
		$num_states = count($states);
		
		// Adds a new opensrs that can be added, edited, or deleted
		self::$module = new Module("opensrs", false, true, true);
		
		// Adds the given fields to the database for this module if it has not already
		// been created. WARNING: if the module has already been created with the name
		// given in the Module constructor you must delete the old module before the
		// new one will be created. This may have an adverse effect on services.
		self::$module->addDBField("varchar(255)", "username", "Username");
		self::$module->addDBField("varchar(255)", "key", "Key");
		self::$module->addDBField("varchar(5)", "testmode", "Test Mode", "true");
		self::$module->addDBField("varchar(5)", "lock", "Lock", "true");
		self::$module->addDBField("varchar(255)", "ns1", "NS1");
		self::$module->addDBField("varchar(255)", "ns2", "NS2");
		self::$module->addDBField("varchar(255)", "ns3", "NS3");
		
		
		// Options for creating a new module row
		// Example: eNom module may store multiple different servers. The HTML Fields below
		// allow you to create/edit a server for the opensrs module
		// NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
		self::$module->addModuleHTMLField("text", "Username", "username");
		self::$module->addModuleHTMLField("text", "Key", "key");
		self::$module->addModuleHTMLField("select", "Test Mode", "testmode");
			self::$module->addModuleHTMLSelectOption("testmode", "Yes", "true");
			self::$module->addModuleHTMLSelectOption("testmode", "No", "false");
		self::$module->addModuleHTMLField("select", "Lock Domains", "lock");
			self::$module->addModuleHTMLSelectOption("lock", "Yes", "true");
			self::$module->addModuleHTMLSelectOption("lock", "No", "false");
		self::$module->addModuleHTMLField("text", "NS1", "ns1");
		self::$module->addModuleHTMLField("text", "NS2", "ns2");
		self::$module->addModuleHTMLField("text", "NS3", "ns3");
		
		
		// Specify which field from the module table to display in the drop down and give it a friendly name
		// By default items will be shown by ID and have the friendly name of "Module ID".
		// This appears when adding or editing a service for a user.
		self::$module->setModuleRowSelect("OpenSRS Account", "username");
		
		// If the 3rd parameter is anything other than user1, user2, pass, opt1, opt2, it must be formatted as an element of the detail array.
		// ex. detail[emailaddress].  Note, any field other than user1, user2, pass, opt1, or opt2  will *NOT* be stored in the database,
		// and may only be used in this module file.
		// Fields that are shown to the admins when adding a service that uses this module from within the control panel
		self::$module->addServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The Domain Name you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
		self::$module->addServiceHTMLField("text", "Password", "pass", "^.{4,16}$", "Passwords must be between 4 and 16 characters.");
		self::$module->addServiceHTMLField("text", "E-mail Address", "detail[email]", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");
		self::$module->addServiceHTMLField("text", "First Name", "detail[fname]", "^.{2,}", "You must provide a valid First Name.");
		self::$module->addServiceHTMLField("text", "Last Name", "detail[lname]", "^.{2,}", "You must provide a valid Last Name.");
		self::$module->addServiceHTMLField("text", "Address 1", "detail[address1]", "^.{2,}", "You must provide a valid Address.");
		self::$module->addServiceHTMLField("text", "Address 2", "detail[address2]");
		self::$module->addServiceHTMLField("text", "City", "detail[city]", "^.{2,}", "You must provide a valid City.");
		self::$module->addServiceHTMLField("text", "Postal Code", "detail[postcode]", "^.{2,}", "You must provide a Postal Code.");		
		self::$module->addServiceHTMLField("select", "State/Province", "detail[state]");
		if (is_array($states)) {
			self::$module->addServiceHtmlSelectOption("detail[state]", "-- None --", "");
			for ($i=0; $i<$num_states; $i++) {
				self::$module->addServiceHtmlSelectOption("detail[state]", $states[$i][1], $states[$i][2]);
			}
		}
		self::$module->addServiceHTMLField("select", "Country", "detail[country]");
		if (is_array($countries)) {
			for ($i=0; $i<$num_countries; $i++) {
				self::$module->addServiceHtmlSelectOption("detail[country]", $countries[$i][1], $countries[$i][2]);
			}
		}
		self::$module->addServiceHTMLField("text", "Phone", "detail[phone]", "^\+(([0-9]{1,}\.[0-9]{7,10})|([0-9]{1,}\.[0-9]{7,10}x[0-9]{1,}))$", "You must provide a valid Phone Number of the format: +1.4165550123x1902");
		self::$module->addServiceHTMLField("text", "Fax", "detail[fax]");
		self::$module->addServiceHTMLField("text", "Company/Org.", "detail[org]", "^.{1,}", "You must provide a Company/Organization.");
		self::$module->addServiceHTMLField("text", "Job Title", "detail[title]");
		
		// If the 3rd parameter is anything other than user1, user2, pass, opt1, opt2, then the field created will *NOT* be stored
		// in the database, and may only be used in this module file.
		// Fields that are shown to the user when adding a service that uses this module from within the control panel
		self::$module->addUserServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
		self::$module->addUserServiceHTMLField("text", "Password", "pass", "^.{4,16}$", "Passwords must be between 4 and 16 characters.");
		self::$module->addUserServiceHTMLField("text", "E-mail Address", "detail[email]", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");
		self::$module->addUserServiceHTMLField("text", "First Name", "detail[fname]", "^.{2,}", "You must provide a valid First Name.");
		self::$module->addUserServiceHTMLField("text", "Last Name", "detail[lname]", "^.{2,}", "You must provide a valid Last Name.");
		self::$module->addUserServiceHTMLField("text", "Address 1", "detail[address1]", "^.{2,}", "You must provide a valid Address.");         
		self::$module->addUserServiceHTMLField("text", "Address 2", "detail[address2]");
		self::$module->addUserServiceHTMLField("text", "City", "detail[city]", "^.{2,}", "You must provide a valid City.");
		self::$module->addUserServiceHTMLField("text", "Postal Code", "detail[postcode]", "^.{2,}", "You must provide a Postal Code.");
		self::$module->addUserServiceHTMLField("select", "State/Province", "detail[state]");
		if (is_array($states)) {
			self::$module->addUserServiceHtmlSelectOption("detail[state]", "-- None --", "");
			for ($i=0; $i<$num_states; $i++) {
				self::$module->addUserServiceHtmlSelectOption("detail[state]", $states[$i][1], $states[$i][2]);
			}
		}
		self::$module->addUserServiceHTMLField("select", "Country", "detail[country]");
		if (is_array($countries)) {
			for ($i=0; $i<$num_countries; $i++) {
				self::$module->addUserServiceHtmlSelectOption("detail[country]", $countries[$i][1], $countries[$i][2]);
			}
		}
		self::$module->addUserServiceHTMLField("text", "Phone", "detail[phone]", "^\+(([0-9]{1,}\.[0-9]{7,10})|([0-9]{1,}\.[0-9]{7,10}x[0-9]{1,}))$", "You must provide a valid Phone Number of the format: +1.4165550123x1902");
		self::$module->addUserServiceHTMLField("text", "Fax", "detail[fax]");
		self::$module->addUserServiceHTMLField("text", "Company/Org.", "detail[org]", "^.{1,}", "You must provide a Company/Organization.");
		self::$module->addUserServiceHTMLField("text", "Job Title", "detail[title]");
		
		// These displayed as tags and are usable when creating/editing a packages welcome e-mail
		self::$module->addServiceTag("user1", "domain");
		self::$module->addServiceTag("user2", "username");
        self::$module->addServiceTag("pass", "pass");
		
		// These are the fields that may be edited when performing a local edit on this service
		self::$module->addServiceEditField("user1", "Domain");
		self::$module->addServiceEditField("user2", "User Name");
		self::$module->addServiceEditField("pass", "Password");
		self::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
		self::$module->addServiceEditField("dated", "Date Deleted");
		self::$module->addServiceEditField("datep", "Date Suspended");
		self::$module->addServiceEditField("dater", "Date Renews");
		self::$module->addServiceEditField("notes", "Notes");
		
		// Creates the module, adds the module to the database if the module table does not
		// already exist. WARNING: if a module has already been created and you attempt to
		// add additional fields they WILL NOT BE ADDED, and any refrence to those fields
		// WILL RESULT IN AN ERROR.
		self::$module->createModule();
	}
     
	// Returns the module object, this function MUST be present
	public function getModule() {
		return self::$module;
	}
     
     
	/*******************************************************************************
	********************************************************************************
	* This is the MAGIC function. This function automatically creates a service    *
	* for a user.                                                                  *
	*                                                                              *
	* IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
	* NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
	*                                                                              *
	* IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
	* THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
	********************************************************************************
	*******************************************************************************/
	
	public function promodule($action, $serviceInfo, $packageInfo) {
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = self::$module->getRows($conditions);
	  
		$vars = array();
		// We are only expecting 1 row returned, so take the first element of this
		// 2-dimensional array
		$testmode = $data[0]['testmode'];
		$lock = ($data[0]['lock'] == "true") ? 1 : 0;	  
	  
		// Extract the SLD and TLD
		preg_match('/^([a-z0-9]+[a-z0-9\-]*[a-z0-9]+)\.([a-z]+[a-z\.]*[a-z]+)$/i', $serviceInfo['user1'], $type);
		$sld = $type[1];
		$tld = $type[2];
	
		switch ($action) {
			case "add":
				for ($i=1; $i<=3; $i++) {
					if (strlen($data[0]['ns' . $i]) > 0) {
						$vars['ns' . $i] = $data[0]['ns' . $i];
						$num_nameservers = $i;
					}
				}
				
				$custom_nameservers = 1;
				// Use the default name servers as set in OpenSRS
				if (!isset($vars['ns1'])) {
					$custom_nameservers = 0;
				}
				
				// Make username
				$serviceInfo['user2'] = self::makeUserName($serviceInfo['user1']);
				
				// Years to register for, must be at least 1
				$period = ($serviceInfo['term'] > 12 ? floor($serviceInfo['term']/12) : 1);
				
				// TLD specifics
				$tld_specific = "";
				switch ($tld) {
					case "au":
						$tld_specific .= "<item key=\"eligibility_name\">" .
							$serviceInfo['detail']['fname'] . " " .
							$serviceInfo['detail']['lname'] . "</item>";
						$tld_specific .= "<item key=\"eligibility_type\">Citizen/Resident</item>" .
							"<item key=\"registrant_name\">" .
							$serviceInfo['detail']['fname'] . " " .
							$serviceInfo['detail']['lname'] . "</item>";
						break;
					case "ca":
						//CCT corresponds to "Canadian Citizen"
						$tld_specific .= "<item key=\"legal_type\">CCT</item>";
						break;
					case "us":
						//C11 is a citizen of the US or territories
						//P3 is for personal use
						$tld_specific .= "<item key=\"tld_data\">" .
								"<dt_assoc>" .
									"<item key=\"nexus\">" .
										"<dt_assoc>" .
											"<item key=\"category\">C11</item>" .
											"<item key=\"app_purpose\">P3</item>" .
										"</dt_assoc>" .
									"</item>" .
								"</dt_assoc>" .
							"</item>";
						break;
				}
				
				$contact =<<<XML
					<item key="first_name">{$serviceInfo['detail']['fname']}</item>
					<item key="last_name">{$serviceInfo['detail']['lname']}</item>
					<item key="phone">{$serviceInfo['detail']['phone']}</item>
					<item key="fax">{$serviceInfo['detail']['fax']}</item>
					<item key="email">{$serviceInfo['detail']['email']}</item>
					<item key="org_name">{$serviceInfo['detail']['org']}</item>
					<item key="address1">{$serviceInfo['detail']['address1']}</item>
					<item key="address2">{$serviceInfo['detail']['address2']}</item>
					<item key="address3"/>
					<item key="city">{$serviceInfo['detail']['city']}</item>
					<item key="state">{$serviceInfo['detail']['state']}</item>
					<item key="country">{$serviceInfo['detail']['country']}</item>
					<item key="postal_code">{$serviceInfo['detail']['postcode']}</item>
					<item key="lang">en</item>
					<item key="lang_pref">EN</item>
					<item key="url"/>
XML;
					
				$xml =<<<XML
					<item key="auto_renew">1</item>
					<item key="link_domains">0</item>
					<item key="reg_domain"/>
					<item key="f_lock_domain">{$lock}</item>
					<item key="f_whois_privacy">1</item>
					<item key="f_parkp">Y</item>
					<item key="domain">{$serviceInfo['user1']}</item>
					<item key="affiliate_id"/>
					<item key="period">{$period}</item>
					<item key="reg_type">new</item>
					<item key="reg_username">{$serviceInfo['user2']}</item>
					<item key="reg_password">{$serviceInfo['pass']}</item>						
					<item key="custom_tech_contact">0</item>
					<item key="encoding_type"/>
					<item key="custom_nameservers">{$custom_nameservers}</item>						
					<item key="contact_set">
						<dt_assoc>
							<item key="owner">
								<dt_assoc>
									{$contact}
								</dt_assoc>
							</item>
							<item key="admin">
								<dt_assoc>
									{$contact}
								</dt_assoc>
							</item>
							<item key="billing">
								<dt_assoc>
									{$contact}
								</dt_assoc>
							</item>
							<item key="tech">
								<dt_assoc>
									{$contact}
								</dt_assoc>
							</item>
						</dt_assoc>
					</item>
XML;

				// Add nameserver list 
				if ($custom_nameservers == 1) {
					$servers = "";
					for ($i=1; $i<=$num_nameservers; $i++) {
						$servers .= "<item key=\"" . ($i-1) . "\">" .
								"<dt_assoc>" .
									"<item key=\"sortorder\">" . $i . "</item>" .
									"<item key=\"name\">" . $vars['ns' . $i] . "</item>" .
								"</dt_assoc>" .
							"</item>";
					}
					
					$xml .=<<<XML
						<item key="nameserver_list">
							<dt_array>
								{$servers}
							</dt_array>
						</item>
XML;
				}
				
				// Add TLD specific items
				$xml .= $tld_specific;
				
				$vars['action'] = "SW_REGISTER";
				$vars['object'] = "DOMAIN";
				$vars['xml'] = $xml;				
					
				if (!DEMO && self::call($vars, $testmode))
					return $serviceInfo;
				break;
			case "remove":
				// Sets the domain to not auto-renew
				
				// First, get a cookie for use in modifying the domain
				$fields = array(
					'username' => $serviceInfo['user2'],
					'password' => $serviceInfo['pass'],
					'domain' => $serviceInfo['user1']
				);
				$cookie = self::getCookie($fields);
				
				$xml =<<<XML
					<item key="data">expire_action</item>
					<item key="affect_domains">0</item>
					<item key="auto_renew">0</item>
					<item key="let_expire">0</item>
XML;
				
				$vars['action'] = "MODIFY";
				$vars['object'] = "DOMAIN";
				$vars['cookie'] = $cookie;
				$vars['xml'] = $xml;
				
				return self::call($vars, $testmode);
				break;
			case "suspend":
				// No action to perform.
				return true;
				break;
			case "unsuspend":
				// No action to perform.
				return true;
				break;
		}
	
		return false;
	}
	
	// Gets a cookie for the domain with username and password
	public function getCookie($fields) {
		$data = self::$module->getRows();
		$testmode = $data[0]['testmode'];
		
		$vars = array();
		$vars['action'] = "SET";
		$vars['object'] = "COOKIE";
		$vars['xml'] =<<<XML
			<item key="reg_username">{$fields['username']}</item>
			<item key="reg_password">{$fields['password']}</item>
			<item key="domain">{$fields['domain']}</item>
XML;
		
		$result = self::call($vars, $testmode, true);
		
		if (!empty($result)) {
			$matches = array();
			if (preg_match("/<item key=\"cookie\">(.*?)<\/item>/", $result, $matches)) {
				if (isset($matches[0]))
					return $matches[0];
			}
		}
		return "";
	}

	// Checks if the given domain is available through opensrs
	public function checkDomain($domain) {
		$data = self::$module->getRows();
	  
		$testmode = $data[0]['testmode'];
		
		$vars = array();
		$vars['action'] = "LOOKUP";
		$vars['object'] = "DOMAIN";
		$vars['xml'] =<<<XML
			<item key="domain">{$domain}</item>
			<item key="no_cache">1</item>
XML;
		
		if (self::call($vars, $testmode))
			return "true";
		return "false";
	}
     
     
	/* ------------------------------------------------------------------------------------------------
   The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	// Perform the call action
	// @param boolean $return True to return the response (array), false to return boolean
	private static function call($vars, $testmode, $return=false) {
		if ($testmode == "true")
			$apiurl = "horizon.opensrs.net";
		else
			$apiurl = "rr-n1-tor.opensrs.net";
		
		if (!isset($vars['cookie']))
			$vars['cookie'] = "";
			
		//action: LOOKUP, SW_REGISTER, MODIFY, SET
		//object: COOKIE, DOMAIN
		$xml =<<<XML
<?xml version="1.0" encoding='UTF-8' standalone='no' ?>
<!DOCTYPE OPS_envelope SYSTEM 'ops.dtd'>
<OPS_envelope>
	<header>
		<version>0.9</version>
	</header>
	<body>
		<data_block>
			<dt_assoc>
				<item key="protocol">XCP</item>
				<item key="action">{$vars['action']}</item>
				<item key="object">{$vars['object']}</item>
				{$vars['cookie']}
				<item key="attributes">
					<dt_assoc>
						{$vars['xml']}
					</dt_assoc>
				</item>
			</dt_assoc>
		</data_block>
	</body>
</OPS_envelope>
XML;

		// Log data sent to OpenSRS
		self::$module->logData($xml, "input", true);
		
		$data = self::$module->getRows();
		$key = $data[0]['key'];
		
		$signature = md5(md5($xml . $key) . $key);
		
		$headers = array();
		$headers[] = "Content-Type: text/xml";
		$headers[] = "X-Username: " . $data[0]['username'];
		$headers[] = "X-Signature: " . $signature;
		$headers[] = "Content-Length: " . strlen($xml);
		
		$ch = curl_init();	  
		curl_setopt($ch, CURLOPT_URL, "https://" . $apiurl . ":55443/");
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,0);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
	  
		$response = curl_exec($ch);
		curl_close ($ch);
	
		$success = false;
		// Response codes for Domain Available (210), or Command Successful (200)
		if (preg_match("/<item key=\"response_code\">210<\/item>/", $response) ||
			preg_match("/<item key=\"response_code\">200<\/item>/", $response))
			$success = true;
		
		// Log data received from OpenSRS
		self::$module->logData($response, "output", $success);
	
		if ($return)
			return $response;
		return $success;
	}
	
	// Creates a username that is up to 8 characters long
	private static function makeUserName($domain) {
	   $username = preg_replace("/^[^a-z]+/i", "", $domain);
	   $username = preg_replace("/[^a-z0-9]*/i", "", $username);
	 
	   return substr($username, 0, 8);
	}	
}
?>