<?php
/*******************************************************************************
 @name Plesk Module
 @author Cody Phillips
 @date Jun 25, 2008
 @version 1.2
 @notes For use with Blesta version 1.4.0+
*******************************************************************************/

class plesk implements ModuleInterface {
     private static $module;
     private $host;
     private $user;
     private $password;
     private $usessl;

     public function __construct() {
          // Adds a new Plesk module that can be added, edited, or deleted
          plesk::$module = new Module("plesk", true, true, true);

          // Adds the given fields to the database for this module if it has not already
          // been created. WARNING: if the module has already been created with the name
          // given in the Module constructor you must delete the old module before the
          // new one will be created. This may have an adverse effect on services.
          plesk::$module->addDBField("varchar(255)", "hostip", "Server IP");
          plesk::$module->addDBField("varchar(255)", "user", "User");
          plesk::$module->addDBField("varchar(255)", "password", "Password");
          plesk::$module->addDBField("varchar(5)", "usessl", "Use SSL", "true");
          plesk::$module->addDBField("int(11)", "max", "Limit", "0");
          plesk::$module->addDBField("int(11)", "cur", "Current", "0");

          // Options for creating a new module row
          // Example: plesk module may store multiple different servers. The HTML Fields below
          // allow you to create/edit a server for the plesk module
          // NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
          plesk::$module->addModuleHTMLField("text", "Server IP", "hostip");
          plesk::$module->addModuleHTMLField("text", "Login", "user");
          plesk::$module->addModuleHTMLField("text", "Password", "password");
          plesk::$module->addModuleHTMLField("select", "Use SSL", "usessl");
               plesk::$module->addModuleHTMLSelectOption("usessl", "Yes", "true");
               plesk::$module->addModuleHTMLSelectOption("usessl", "No", "false");
          plesk::$module->addModuleHTMLField("text", "Account Limit", "max", "^[0-9]+", "Account Limit must be an integer.");
          plesk::$module->addModuleHTMLField("text", "Current Accounts", "cur", "^[0-9]+", "Current Accounts must be an integer.");

          // Module Options while creating a package with this module
          // NOTE: Only one field is accepted of this type and must be named "mstring"
          plesk::$module->addPackageHTMLField("text", "Plan Name", "mstring", "^.+", "Plan Name must not be left empty.");

          // This adds a note to be displayed along with the above package field
          plesk::$module->addPackageNote("mstring", "Enter the Domain Template Name as configured in Plesk, i.e. \"Bronze\".");

          // Specify which field from the module table to display in the drop down and give it a friendly name
          // By default items will be shown by ID and have the friendly name of "Module ID".
          // This appears when adding or editing a service for a user.
          plesk::$module->setModuleRowSelect("Server", "hostip");

          // These are shown as column names for each service of this type.
          plesk::$module->addServiceColumn("user2", "User Name");
          plesk::$module->addServiceColumn("pass", "Password");
          plesk::$module->addServiceColumn("hostip", "Server");

          // These are displayed as tags and are usable when creating/editing a packages welcome e-mail
          plesk::$module->addServiceTag("user1", "domain");
          plesk::$module->addServiceTag("user2", "username");
          plesk::$module->addServiceTag("pass", "pass");
          plesk::$module->addServiceTag("term", "term");
          plesk::$module->addServiceTag("hostip", "server");

          // These are the fields that may be edited when performing a local edit on this service
          plesk::$module->addServiceEditField("user1", "Domain");
          plesk::$module->addServiceEditField("user2", "User Name");
          plesk::$module->addServiceEditField("pass", "Password");
          plesk::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
          plesk::$module->addServiceEditField("hostip", "Server");
          plesk::$module->addServiceEditField("dated", "Date Deleted");
          plesk::$module->addServiceEditField("datep", "Date Suspended");
          plesk::$module->addServiceEditField("dater", "Date Renews");
          plesk::$module->addServiceEditField("notes", "Notes");


          // Fields that are shown to the admins when adding a service that uses this module.
          plesk::$module->addServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
          plesk::$module->addServiceHTMLField("text", "Password", "pass", "^.{5,14}$", "Passwords must be between 5 and 14 characters.");

          // Fields that are shown to the user when adding a service that uses this module.
          plesk::$module->addUserServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
          plesk::$module->addUserServiceHTMLField("text", "Password", "pass", "^.{5,14}$", "Passwords must be between 5 and 14 characters.");


          // Creates the module, adds the module to the database if the module table does not
          // already exist. WARNING: if a module has already been created and you attempt to
          // add additional fields they WILL NOT BE ADDED, and any refrence to those fields
          // WILL RESULT IN AN ERROR.
          plesk::$module->createModule();
     }

     // Returns the module object, this function MUST be present
     public function getModule() {
          return plesk::$module;
     }

     /*******************************************************************************
     ********************************************************************************
     * This is the MAGIC function. This function automatically creates a service    *
     * for a user.                                                                  *
     *                                                                              *
     * IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
     * NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
     *                                                                              *
     * IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
     * THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
     ********************************************************************************
     *******************************************************************************/

     public function promodule($action, $serviceInfo, $packageInfo) {
	  // packageInfo comes from packagesetinfos() function
	  $plan = $packageInfo[4][2];

	  $conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
	  $data = plesk::$module->getRows($conditions);

	  // We are only expecting 1 row returned, so take the first element of this
	  // 2-dimensional array
	  $this->host = $data[0]['hostip'];
	  $this->password = $data[0]['password'];
	  $this->user = $data[0]['user'];
	  $this->usessl = $data[0]['usessl'];

	  switch ($action) {
	       case "add":
		    // Ensure we do not exceed the maximum number of accounts for this server.
		    if ($data[0]['cur'] >= $data[0]['max'])
			 return false;

		    // All user names should be up to the first 8 characters of the domain name
		    $serviceInfo['user2'] = plesk::makeUserName($serviceInfo['user1']);

		    if (DEMO)
			 return $serviceInfo;
		    elseif ($this->add($serviceInfo['user2'], $serviceInfo['pass'], $serviceInfo['user1'], $plan)) {
			 // Update the current number of accounts on this server
			 plesk::$module->updateDBFields("cur", $data[0]['cur']+1, $serviceInfo['mid']);

			 return $serviceInfo;  // return the info that we want to be inserted into the services table
		    }
		    break;
	       case "remove":
                     if (DEMO)
                     	return true;
                     elseif ($this->remove($serviceInfo['user2'])) {
			 // Update the current number of accounts on this server
			 plesk::$module->updateDBFields("cur", $data[0]['cur']-1, $serviceInfo['mid']);

                          return true;
                     }
		    break;
	       case "suspend":
                     if (DEMO)
                     	return true;
                     return $this->suspend($serviceInfo['user2']);
		    break;
	       case "unsuspend":
                     if (DEMO)
                     	return true;
                     return $this->unsuspend($serviceInfo['user2']);
		    break;
	  }

	  return false;
     }


     /* ------------------------------------------------------------------------------------------------
	The following are support functions for this module
     --------------------------------------------------------------------------------------------------*/
     /*
     * Adds a user for the given domain and plan.
     * @return true on success, false otherwise
     */
     private function add($user, $pass, $domain, $plan) {

           // First check if this client already exists on the plesk server
           $xml = <<<EOT
<?xml version="1.0"?>
                      <packet version="1.4.2.0">
                              <client>
                                      <get>
                                              <filter>
                                                      <login>$user</login>
                                              </filter>
                                              <dataset>
                                                  <gen_info/>
                                              </dataset>
                                      </get>
                              </client>
                      </packet>
EOT;
           
	  $result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);
           
           // Ensure that the account does not already exist
           //if (eregi("<errcode>1013</errcode>", $result)) {
		   if (preg_match("/<errcode>1013<\/errcode>/i", $result)) {
           	// Attempt to create the account and hosting service.
                 $xml = <<<EOT
<?xml version="1.0"?>
                 <packet version="1.4.2.0">
                       <client>
                             <add>
                                   <gen_info>
                                         <cname></cname>
                                         <pname>$user</pname>
                                         <login>$user</login>
                                         <passwd>$pass</passwd>
                                         <status>0</status>
                                         <phone></phone>
                                         <fax></fax>
                                         <email></email>
                                         <address></address>
                                         <city></city>
                                         <state/>
                                         <pcode/>
                                   </gen_info>
                             </add>
                       </client>
                 </packet>
EOT;
                 
	  	$result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);
		preg_match("/<id>([0-9]+)<\/id>/", $result, $matches);
                 $client_id = $matches[1];

                 // Ensure there were no errors when adding the account
                 //if (eregi("<errcode>", $result))
				 if (preg_match("/<errcode>/i", $result)) 
                 	return false;             

		// Add the default IP to the IP pool for this user... why do we have to do this
                 // in a separate API call altogether?
                 // Because Plesk is retarded, and their API is garbage.
                 $xml = <<<EOT
<?xml version="1.0"?>
                 <packet version="1.4.2.0">
                       <client>
                             <ippool_add_ip>
                                 <client_id>$client_id</client_id>
                                 <ip_address>$this->host</ip_address>
                             </ippool_add_ip>
                       </client>
                 </packet>
EOT;
                 
	  	$result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);

                 // Create the domain
                 $xml = <<< EOT
<?xml version="1.0"?>
                 <packet version="1.4.2.0">
                       <domain>
                             <add>
                                   <gen_setup>
                                         <name>$domain</name>
                                         <client_id>$client_id</client_id>
                                         <htype>vrt_hst</htype>
                                         <ip_address>$this->host</ip_address>
                                         <status>0</status>
                                   </gen_setup>
	                          <hosting>
	                           	<vrt_hst>
	                           		<ftp_login>$user</ftp_login>
	                           		<ftp_password>$pass</ftp_password>
	                           		<ip_address>$this->host</ip_address>
	                           	</vrt_hst>
	                           </hosting>
                                   <template-name>$plan</template-name>
                             </add>
                       </domain>
                 </packet>
EOT;

	  	$result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);
                 // Ensure there were no errors when adding the account
                 //if (eregi("<errcode>", $result))
				 if (preg_match("/<errcode>/i", $result))
                 	return false;
		return true;
           }
           else
           	return false;
     }

     /*
     * Removes a user from the server.
     * @return true on success, false otherwise
     */
     private function remove($user) {
	  $xml = <<< EOT
<?xml version="1.0"?>
           <packet version="1.4.2.0">
           <client>
              <del>
                 <filter>
                     <login>$user</login>
                 </filter>
              </del>
           </client>
           </packet>
EOT;

           $result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);

           //if (eregi("<errcode>", $result))
		   if (preg_match("/<errcode>/i", $result))
                  return false;
           return true;
     }

     /*
     * Suspends a user on the server.
     * @return true on success, false otherwise
     */
     private function suspend($user) {
	  $xml = <<< EOT
<?xml version="1.0"?>
           <packet version="1.4.2.0">
                 <client>
           	      <set>
	                    <filter>
	                          <login>$user</login>
	                    </filter>
                             <values>
			    	  <gen_info>
                         	  	<status>16</status>
                             	  </gen_info>
                             </values>
                       </set>
           	</client>
           </packet>
EOT;

           $result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);

           //if (eregi("<errcode>", $result))
		   if (preg_match("/<errcode>/i", $result))
                  return false;
           return true;
     }

     /*
     * Unsuspends a user on the server.
     * @return true on success, false otherwise
     */
     private function unsuspend($user, $pass, $domain) {
	  $xml = <<< EOT
<?xml version="1.0"?>
           <packet version="1.4.2.0">
                 <client>
           	      <set>
	                    <filter>
	                          <login>$user</login>
	                    </filter>
                             <values>
			    	  <gen_info>
                         	  	<status>0</status>
                             	  </gen_info>
                             </values>
                       </set>
           	</client>
           </packet>
EOT;

           $result = $this->request($this->host, $this->user, $this->password, $this->usessl, $xml);

           //if (eregi("<errcode>", $result))
		   if (preg_match("/<errcode>/i", $result))
                  return false;
           return true;
     }


     // Creates a username that is between 5 and 8 characters long
     // Usernames that are generated randomly are 7 characters in length
     private static function makeUserName($domain) {
	  // Remove everything except letters from the domain
	  $temp = preg_replace('/[^a-zA-Z]/', '', $domain);

	  // If what we have left to build the username is less than 5 characters
	  // add in some random characters to create a unique username.
	  if (strlen($temp) < 5) {
	       // Add 3-random characters followed by the rest of temp.
	       $chars = array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t',
	       'u','v','w','x','y','z');
	       $count = count($chars) - 1;

	       for ($i=0; $i<3; $i++)
		    $temp = $chars[rand(0, $count)] . $temp;
	  }

	  return substr($temp, 0, 8);
     }

     /*
     * Prepares CURL to perform Plesk API request
     * @return response
     */
     private function request($host, $login, $password, $usessl, $xml) {
     
           // Log data sent to plesk
           plesk::$module->logData($xml, "input", true);     
     
           $ch = curl_init();
           curl_setopt($ch, CURLOPT_URL, "http" . ($usessl ? "s" : "") . "://" . $host . ":8443/enterprise/control/agent.php");
           curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
           curl_setopt($ch, CURLOPT_POST, true);
	  curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
           curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
           curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
           curl_setopt($ch, CURLOPT_HTTPHEADER,
                  array("HTTP_AUTH_LOGIN: " . $login,
                         "HTTP_AUTH_PASSWD: " . $password,
                         "HTTP_PRETTY_PRINT: TRUE",
                         "Content-Type: text/xml")
           );
	  //curl_setopt($ch, CURLOPT_HEADER, true);

           $response = curl_exec($ch);
           
           // Log data received by plesk
           //plesk::$module->logData($response, "output", (eregi("<errcode>", $response) === false ? true : false));
		   plesk::$module->logData($response, "output", (preg_match("/<errcode>/i", $response) ? true : false));
                            
           curl_close($ch);

           return $response;
     }
}
?>