<?php
/*******************************************************************************
 @name Reseller Club Module
 @author Tyson Phillips
 @date June 6, 2011
 @version 1.0
 @notes For use with Blesta version 2.0.0+
*******************************************************************************/

class reseller_club implements ModuleInterface {
	private static $module;
     
	public function __construct() {
		// Adds a new resellec club that can be added, edited, or deleted
		self::$module = new Module("reseller_club", false, true, false);
		
		// Adds the given fields to the database for this module if it has not already
		// been created. WARNING: if the module has already been created with the name
		// given in the Module constructor you must delete the old module before the
		// new one will be created. This may have an adverse effect on services.
		self::$module->addDBField("varchar(255)", "resellerid", "Reseller ID");
		self::$module->addDBField("varchar(255)", "password", "Password");
		self::$module->addDBField("varchar(5)", "testmode", "Test Mode", "true");
		self::$module->addDBField("varchar(5)", "lock", "Lock", "true");
		self::$module->addDBField("varchar(255)", "ns1", "NS1");
		self::$module->addDBField("varchar(255)", "ns2", "NS2");
		self::$module->addDBField("varchar(255)", "ns3", "NS3");
		
		
		// Options for creating a new module row
		// Example: eNom module may store multiple different servers. The HTML Fields below
		// allow you to create/edit a server for the enom module
		// NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
		self::$module->addModuleHTMLField("text", "Reseller ID", "resellerid");
		self::$module->addModuleHTMLField("text", "Password", "password");
		self::$module->addModuleHTMLField("select", "Test Mode", "testmode");
			self::$module->addModuleHTMLSelectOption("testmode", "Yes", "true");
			self::$module->addModuleHTMLSelectOption("testmode", "No", "false");
		self::$module->addModuleHTMLField("select", "Lock Domains", "lock");
			self::$module->addModuleHTMLSelectOption("lock", "Yes", "true");
			self::$module->addModuleHTMLSelectOption("lock", "No", "false");
		self::$module->addModuleHTMLField("text", "NS1", "ns1");
		self::$module->addModuleHTMLField("text", "NS2", "ns2");
		self::$module->addModuleHTMLField("text", "NS3", "ns3");
		
		
		// Specify which field from the module table to display in the drop down and give it a friendly name
		// By default items will be shown by ID and have the friendly name of "Module ID".
		// This appears when adding or editing a service for a user.
		self::$module->setModuleRowSelect("Reseller Club Account", "resellerid");
		
		// If the 3rd parameter is anything other than user1, user2, pass, opt1, opt2, it must be formatted as an element of the detail array.
		// ex. detail[emailaddress].  Note, any field other than user1, user2, pass, opt1, or opt2  will *NOT* be stored in the database,
		// and may only be used in this module file.
		// Fields that are shown to the admins when adding a service that uses this module from within the control panel
		self::$module->addServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The Domain Name you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
		self::$module->addServiceHTMLField("text", "Password", "pass", "^(?=.*\d)(?=.*[a-zA-Z]).{8,15}$", "The password for this account must contain characters and digits between 8 and 15 characters in length.");
		self::$module->addServiceHTMLField("text", "E-mail Address", "detail[email]", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");
		self::$module->addServiceHTMLField("text", "First Name", "detail[fname]", "^.{2,}", "You must provide a valid First Name.");
		self::$module->addServiceHTMLField("text", "Last Name", "detail[lname]", "^.{2,}", "You must provide a valid Last Name.");
		self::$module->addServiceHTMLField("text", "Address 1", "detail[address1]", "^.{2,}", "You must provide a valid Address.");
		self::$module->addServiceHTMLField("text", "Address 2", "detail[address2]");
		self::$module->addServiceHTMLField("text", "City", "detail[city]", "^.{2,}", "You must provide a valid City.");
		self::$module->addServiceHTMLField("text", "State/Province", "detail[state]", "^.{2,}", "You must provide a valid State/Province.");
		self::$module->addServiceHTMLField("text", "Postal Code", "detail[postcode]", "^[0-9]{2,}$", "You must provide a valid Postal Code.");
		self::$module->addServiceHTMLField("text", "Country Code", "detail[country]", "^[a-zA-Z]{2}$", "You must provide a valid 2-character Country Code.");
		self::$module->addServiceHTMLField("text", "Phone Country Code", "detail[phone_cc]", "^[0-9]{1,3}", "You must provide a valid Phone number Country Code.");
		self::$module->addServiceHTMLField("text", "Phone", "detail[phone]", "^[0-9]{7,}", "You must provide a valid Phone Number containing only digits.");
		self::$module->addServiceHTMLField("text", "Fax", "detail[fax]");
		self::$module->addServiceHTMLField("text", "Organization", "detail[org]", "^.{2,}", "You must provide an organization.");
		
		// If the 3rd parameter is anything other than user1, user2, pass, opt1, opt2, then the field created will *NOT* be stored
		// in the database, and may only be used in this module file.
		// Fields that are shown to the user when adding a service that uses this module from within the control panel
		self::$module->addUserServiceHTMLField("text", "Domain Name", "user1", "^[a-z0-9._%-]+\.[a-z]{2,6}$", "The domain you entered does not appear to be valid. It should be lowercase and in the format: domain.com.");
		self::$module->addUserServiceHTMLField("text", "Password", "pass", "^(?=.*\d)(?=.*[a-zA-Z]).{8,15}$", "The password for this account must contain characters and digits between 8 and 15 characters in length.");
		self::$module->addUserServiceHTMLField("text", "E-mail Address", "detail[email]", "^[a-z0-9._%-]+@[a-z0-9._%-]+\.[a-z]{2,6}$", "The E-mail address you entered does not appear to be valid.");
		self::$module->addUserServiceHTMLField("text", "First Name", "detail[fname]", "^.{2,}", "You must provide a valid First Name.");
		self::$module->addUserServiceHTMLField("text", "Last Name", "detail[lname]", "^.{2,}", "You must provide a valid Last Name.");
		self::$module->addUserServiceHTMLField("text", "Address 1", "detail[address1]", "^.{2,}", "You must provide a valid Address.");         
		self::$module->addUserServiceHTMLField("text", "Address 2", "detail[address2]");
		self::$module->addUserServiceHTMLField("text", "City", "detail[city]", "^.{2,}", "You must provide a valid City.");
		self::$module->addUserServiceHTMLField("text", "State/Province", "detail[state]", "^.{2,}", "You must provide a valid State/Province.");
		self::$module->addUserServiceHTMLField("text", "Postal Code", "detail[postcode]", "^[0-9]{2,}$", "You must provide a valid Postal Code.");
		self::$module->addUserServiceHTMLField("text", "Country Code", "detail[country]", "^[a-zA-z]{2}$", "You must provide a valid 2-character Country Code.");
		self::$module->addUserServiceHTMLField("text", "Phone Country Code", "detail[phone_cc]", "^[0-9]{1,3}", "You must provide a valid Phone number Country Code.");
		self::$module->addUserServiceHTMLField("text", "Phone", "detail[phone]", "^[0-9]{7,}", "You must provide a valid Phone Number containing only digits.");
		self::$module->addUserServiceHTMLField("text", "Fax", "detail[fax]");
		self::$module->addUserServiceHTMLField("text", "Organization", "detail[org]", "^.{2,}", "You must provide an organization.");
		
		// These displayed as tags and are usable when creating/editing a packages welcome e-mail
		self::$module->addServiceTag("user1", "domain");
		
		// These are the fields that may be edited when performing a local edit on this service
		self::$module->addServiceEditField("user1", "Domain");
		self::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
		self::$module->addServiceEditField("dated", "Date Deleted");
		self::$module->addServiceEditField("datep", "Date Suspended");
		self::$module->addServiceEditField("dater", "Date Renews");
		self::$module->addServiceEditField("notes", "Notes");
		
		// Creates the module, adds the module to the database if the module table does not
		// already exist. WARNING: if a module has already been created and you attempt to
		// add additional fields they WILL NOT BE ADDED, and any refrence to those fields
		// WILL RESULT IN AN ERROR.
		self::$module->createModule();
	}
     
	// Returns the module object, this function MUST be present
	public function getModule() {
		return self::$module;
	}
     
     
	/*******************************************************************************
	********************************************************************************
	* This is the MAGIC function. This function automatically creates a service    *
	* for a user.                                                                  *
	*                                                                              *
	* IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
	* NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
	*                                                                              *
	* IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
	* THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
	********************************************************************************
	*******************************************************************************/
	
	public function promodule($action, $serviceInfo, $packageInfo) {
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = self::$module->getRows($conditions);
	  
		$vars = array();
		// We are only expecting 1 row returned, so take the first element of this
		// 2-dimensional array
		$testmode = $data[0]['testmode'];
		$reseller_id = array('key'=>"auth-userid", 'value'=>$data[0]['resellerid']);
		$password = array('key'=>"auth-password", 'value'=>$data[0]['password']);
	  
		// Extract the SLD and TLD
		preg_match('/^([a-z0-9]+[a-z0-9\-]*[a-z0-9]+)\.([a-z]+[a-z\.]*[a-z]+)$/i', $serviceInfo['user1'], $type);
		$sld = $type[1];
		$tld = strtolower($type[2]);
	
		switch ($action) {
			case "add":
				
				// Check if this client already has an account
				// where the username is the email address
				$vars[] = $reseller_id;
				$vars[] = $password;
				if (($customer_id = self::getCustomerIdByUsername($serviceInfo['detail']['email'], $vars, $testmode))) {
					// Use the existing account to register a domain
					
					// Associate extra details with this customer if they are registering
					// a .US, or .CA domain
					$vars = array();
					$vars[] = $reseller_id;
					$vars[] = $password;
					self::setCustomerDetails($customer_id, $vars, $testmode, $tld);
				}
				else {
					// Create a new account
					$vars[] = array('key'=>"username", 'value'=>$serviceInfo['detail']['email']);
					$vars[] = array('key'=>"passwd", 'value'=>$serviceInfo['pass']);
					$vars[] = array('key'=>"name", 'value'=>$serviceInfo['detail']['fname'] . " " . $serviceInfo['detail']['lname']);
					$vars[] = array('key'=>"company", 'value'=>$serviceInfo['detail']['org']);
					$vars[] = array('key'=>"address-line-1", 'value'=>$serviceInfo['detail']['address1']);
					$vars[] = array('key'=>"city", 'value'=>$serviceInfo['detail']['city']);
					$vars[] = array('key'=>"state", 'value'=>$serviceInfo['detail']['state']);
					$vars[] = array('key'=>"country", 'value'=>$serviceInfo['detail']['country']);
					$vars[] = array('key'=>"zipcode", 'value'=>$serviceInfo['detail']['postcode']);
					$vars[] = array('key'=>"phone-cc", 'value'=>$serviceInfo['detail']['phone_cc']);
					$vars[] = array('key'=>"phone", 'value'=>$serviceInfo['detail']['phone']);
					$vars[] = array('key'=>"lang-pref", 'value'=>"en");
					$vars[] = array('key'=>"address2", 'value'=>$serviceInfo['detail']['address2']);
					$vars[] = array('key'=>"fax", 'value'=>$serviceInfo['detail']['fax']);
					
					// Create a new customer account
					$customer_id = self::createCustomer($vars, $testmode);
					
					// Associate extra details with this customer if they are registering
					// a .US, or .CA domain
					$vars = array();
					$vars[] = $reseller_id;
					$vars[] = $password;
					self::setCustomerDetails($customer_id, $vars, $testmode, $tld);
				}
				
				// Get the customer contact ID and register the domain
				if ($customer_id) {
					// Get the customer contact ID
					$vars = array();
					$vars[] = $reseller_id;
					$vars[] = $password;
					
					if (($contact_id = self::getContactId($customer_id, $vars, $testmode))) {
						// Register the domain
						$vars = array();
						$vars[] = $reseller_id;
						$vars[] = $password;
						$vars[] = array('key'=>"domain-name", 'value'=>$serviceInfo['user1']);
						$vars[] = array('key'=>"years", 'value'=>($serviceInfo['term'] > 12 ? floor($serviceInfo['term']/12) : 1));
						
						// Add nameservers for domain
						for ($i=1; $i<=3; $i++) {
							if (strlen($data[0]['ns' . $i]) > 0)
								$vars[] = array('key'=>"ns", 'value'=>$data[0]['ns' . $i]);
						}
						
						// Add contacts
						$vars[] = array('key'=>"customer-id", 'value'=>$customer_id);
						$vars[] = array('key'=>"reg-contact-id", 'value'=>$contact_id);
						
						// Must pass -1 for NZ, EU, or CA tld's
						switch ($tld) {
							case "nz":
							case "eu":
								$vars[] = array('key'=>"admin-contact-id", 'value'=>-1);
								$vars[] = array('key'=>"tech-contact-id", 'value'=>-1);
							case "ca":
								$vars[] = array('key'=>"billing-contact-id", 'value'=>-1);
								break;
							default:
								$vars[] = array('key'=>"admin-contact-id", 'value'=>$contact_id);
								$vars[] = array('key'=>"tech-contact-id", 'value'=>$contact_id);
								$vars[] = array('key'=>"billing-contact-id", 'value'=>$contact_id);
						}
						
						$vars[] = array('key'=>"invoice-option", 'value'=>"NoInvoice");
						$vars[] = array('key'=>"protect-privacy", 'value'=>"true");
						
						// Register the domain
						if (self::registerDomain($vars, $testmode)) {
							// Lock the domain
							if ($data[0]['lock'] == "true") {
								// Get the order ID of this domain to lock
								$vars = array();
								$vars[] = $reseller_id;
								$vars[] = $password;
								$vars[] = array('key'=>"domain-name", 'value'=>$serviceInfo['user1']);
								
								if (($order_id = self::getOrderId($vars, $testmode))) {
									// Lock the domain
									$vars = array();
									$vars[] = $reseller_id;
									$vars[] = $password;
									$vars[] = array('key'=>"order-id", 'value'=>$order_id);
									
									self::lockDomain($vars, $testmode);
								}
							}
							
							if (!DEMO)
								return $serviceInfo;
						}
					}
				}
				
				break;
			case "remove":
				// No action to perform.
				return true;
				break;
			case "suspend":
				// No action to perform.
				return true;
				break;
			case "unsuspend":
				// No action to perform.
				return true;
				break;
		}
	
		return false;
	}
     
     
	/* ------------------------------------------------------------------------------------------------
   The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	/**
	 * Creates a customer account
	 */
	public function createCustomer(array $vars, $testmode) {
		$action_uri = "customers/signup.xml";
		
		$customer = self::call($vars, $testmode, $action_uri, "POST", true);
		
		// Return the customer ID created
		if ($customer) {
			preg_match("/<int>(.*?)<\/int>/i", $customer, $matches);
			return (!empty($matches[1]) ? $matches[1] : false);
		}
		
		return false;
	}
	
	/**
	 * Sets extra details for a customer account for US domains
	 */
	public function setCustomerDetails($customer_id, array $vars, $testmode, $tld) {
		// Associate extra details with this user if they are registering
		// a .US, or .CA domain
		if ($tld == "us") {
			// Get the contact ID of the customer to add extra details for
			if (($contact_id = self::getContactId($customer_id, $vars, $testmode))) {
				$vars[] = array('key'=>"contact-id", 'value'=>$contact_id);
				
				// TLD specific data
				// Set customer as US citizen (C11) using this for personal use (P3)
				$vars[] = array('key'=>"attr-name1", 'value'=>"purpose");
				$vars[] = array('key'=>"attr-value1", 'value'=>"P3");
				$vars[] = array('key'=>"attr-name2", 'value'=>"category");
				$vars[] = array('key'=>"attr-value2", 'value'=>"C11");
				$vars[] = array('key'=>"product-key", 'value'=>"domus");
				
				// Add the extra settings
				$action_uri = "contacts/set-details.xml";
				
				self::call($vars, $testmode, $action_uri, "POST");
			}
		}
	}
	
	/**
	 * Retrieves customer account ID by username
	 */
	public function getCustomerIdByUsername($username, array $vars, $testmode) {
		$vars[] = array('key'=>"username", 'value'=>$username);
		$action_uri = "customers/details.xml";
		
		$customer = self::call($vars, $testmode, $action_uri, "GET", true);
		
		if ($customer) {
			$customer = str_replace(array("\n", "\r", "\r\n", "\t", " "), "", $customer);
			preg_match("/<string>customerid<\/string><string>(.*?)<\/string>/i", $customer, $matches);
			return (!empty($matches[1]) ? $matches[1] : false);
		}
		
		return false;
	}
	
	/**
	 * Retrieves customer contact ID
	 */
	public function getContactId($customer_id, array $vars, $testmode) {
		$vars[] = array('key'=>"customer-id", 'value'=>$customer_id);
		$vars[] = array('key'=>"type", 'value'=>"Contact");
		$action_uri = "contacts/default.xml";
		
		$contacts = self::call($vars, $testmode, $action_uri, "POST", true);
		
		if ($contacts) {
			$contacts = str_replace(array("\n", "\r", "\r\n", "\t", " "), "", $contacts);
			preg_match("/<string>registrant<\/string><string>(.*?)<\/string>/i", $contacts, $matches);
			return (!empty($matches[1]) ? $matches[1] : false);
		}
		
		return false;
	}
	
	/**
	 * Retrieves the order ID of a domain
	 */
	public function getOrderId(array $vars, $testmode) {
		$action_uri = "domains/orderid.xml";
		
		$order = self::call($vars, $testmode, $action_uri, "GET", true);
		
		if ($order) {
			preg_match("/<int>(.*?)<\/int>/i", $order, $matches);
			return (!empty($matches[1]) ? $matches[1] : false);
		}
		
		return false;
	}
	
	/**
	 * Registers a domain on behalf of a customer
	 */
	public function registerDomain(array $vars, $testmode) {
		$action_uri = "domains/register.xml";
		
		return self::call($vars, $testmode, $action_uri, "POST");
	}
	
	/**
	 * Locks a domain
	 */
	public function lockDomain(array $vars, $testmode) {
		$action_uri = "domains/enable-theft-protection.xml";
		
		return self::call($vars, $testmode, $action_uri, "POST");
	}

	// Perform the call action
	private static function call($vars, $testmode, $action_uri, $type="POST", $return=false) {
		
		// Leg data sent to reseller club
		self::$module->logData($vars, "input", true);
		
		$apiurl = "httpapi.com/api/";
		
		if ($testmode == "true")
			$apiurl = "test.httpapi.com/api/";
		
		$apiurl .= $action_uri . "?";
		
		$num_params = count($vars);
		for ($i=0; $i<$num_params; $i++) {
			$apiurl .= ($i>0 ? "&" : "") . $vars[$i]['key'] . "=" . rawurlencode($vars[$i]['value']);
		}
		
		$ch = curl_init();
	  
		curl_setopt($ch, CURLOPT_URL, "https://" . $apiurl);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,0);
		
		if ($type == "POST") {
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $vars);
		}
	  
		$response = curl_exec($ch);
		curl_close ($ch);
	
		// Success if we have retrieved a success message or data we requested
		$result = str_replace(array("\n", "\r", "\r\n", "\t", " "), "", $response);
		if (((preg_match("/<status>Error<\/status>/i", $result) === 0) && (preg_match("/<string>Error<\/string>/i", $result, $matches) === 0) && (preg_match("/\"ERROR\"/i", $result, $matches) === 0)))
			$success = true; // Success
		else
			$success = false;
		
		// Log data received from enom
		self::$module->logData($response, "output", $success);
		
		// Return the response on a successful action when getting info
		if ($return && $success)
			return $response;
		return $success;
	}
}
?>