<?php
/*******************************************************************************
 @name VPS.NET
 @author Cody Phillips
 @date Mar 11, 2009
 @version 1.2
 @notes For use with Blesta version 2.3.0+
*******************************************************************************/

class vpsnet implements ModuleInterface {
	private static $module;
	private static $version = "api10json";
	private static $url = "https://api.vps.net/";

	private $username;
	private $key;
	private $templates;
	private $cookie_file;

	public function __construct() {
		
		// Adds a new vpsnet that can be added, edited, or deleted
		self::$module = new Module("vpsnet", true, true, true);

		// Adds the given fields to the database for this module if it has not already
		// been created. WARNING: if the module has already been created with the name
		// given in the Module constructor you must delete the old module before the
		// new one will be created. This may have an adverse effect on services.
		//self::$module->addDBField("int(11)", "template", "Template");
		self::$module->addDBField("varchar(255)", "username", "Email", "you@domain.com");
		self::$module->addDBField("varchar(255)", "key", "API Key");
		self::$module->addDBField("longtext", "templates", "Templates");
		self::$module->addDBField("datetime", "templates_updated", "Templates Updated");

		// Creates the module, adds the module to the database if the module table does not
		// already exist. WARNING: if a module has already been created and you attempt to
		// add additional fields they WILL NOT BE ADDED, and any refrence to those fields
		// WILL RESULT IN AN ERROR.
		self::$module->createModule();
		
		$this->templates = $this->getTemplates();

		// Options for creating a new module row
		// Example: vpsnet module may store multiple different servers. The HTML Fields below
		// allow you to create/edit a server for the vpsnet module
		// NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
		self::$module->addModuleHTMLField("text", "Email", "username");
		self::$module->addModuleHTMLField("text", "API Key", "key");

		// Module Options while creating a package with this module
		// NOTE: Only one field is accepted of this type and must be named "mstring"
		self::$module->addPackageHTMLField("text", "Nodes", "mstring", "^[1-9]+[0-9]*", "The number of nodes to use must be at least 1.");

		// This adds a note to be displayed along with the above package field
		self::$module->addPackageNote("mstring", "The number of nodes a VPS created with this package will use");

		// Specify which field from the module table to display in the drop down and give it a friendly name
		// By default items will be shown by ID and have the friendly name of "Module ID".
		// This appears when adding or editing a service for a user.
		self::$module->setModuleRowSelect("VPS Account", "username");

		// These are shown as column names for each service of this type.
		self::$module->addServiceColumn("opt1", "Label");
		self::$module->addServiceColumn("opt2", "IP Address");
		self::$module->addServiceColumn("pass", "Password");

		// These are displayed as tags and are usable when creating/editing a package's welcome e-mail
		self::$module->addServiceTag("user1", "hostname"); // The hostname of the VPS
		self::$module->addServiceTag("user2", "vm_id"); // The virtual machine ID
		self::$module->addServiceTag("opt2", "ip"); // The virtual machine IP address
		self::$module->addServiceTag("pass", "pass"); // The root password for the VPS
		self::$module->addServiceTag("opt1", "label"); // The label given for the VPS
		self::$module->addServiceTag("term", "term");

		// These are the fields that may be edited when performing a local edit on this service
		self::$module->addServiceEditField("user1", "Hostname");
		self::$module->addServiceEditField("opt1", "Label");
		self::$module->addServiceEditField("pass", "Password");
		self::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
		self::$module->addServiceEditField("dated", "Date Deleted");
		self::$module->addServiceEditField("datep", "Date Suspended");
		self::$module->addServiceEditField("dater", "Date Renews");
		self::$module->addServiceEditField("notes", "Notes");


		// Fields that are shown to the admins when adding a service that uses this module.
		self::$module->addServiceHTMLField("text", "Hostname", "user1", "^[a-z0-9_%-]+\.[a-z0-9._%-]+\.[a-z]{2,6}$", "The hostname you entered does not appear to be valid. It should be lowercase and in the format: sub.domain.com.");
		self::$module->addServiceHTMLField("text", "Label", "opt1", "^[ -~]+$", "The label must not be left empty.");
		self::$module->addServiceHTMLField("select", "Template", "detail[template]", "^[ -~]+$", "The template must not be left empty.");
			if (is_array($this->templates)) {
				foreach ($this->templates as $cloud_id => $temp) {
					self::$module->addServiceHTMLSelectGroup("detail[template]", $temp['cloud']);
					for ($i=0; $i<count($temp['templates']); $i++)
						self::$module->addServiceHTMLSelectOption("detail[template]", $temp['templates'][$i]['name'], $temp['templates'][$i]['template']);
				}
			}

		// Fields that are shown to the user when adding a service that uses this module.
		self::$module->addUserServiceHTMLField("text", "Hostname", "user1", "^[a-z0-9_%-]+\.[a-z0-9._%-]+\.[a-z]{2,6}$", "The hostname you entered does not appear to be valid. It should be lowercase and in the format: sub.domain.com.");
		self::$module->addUserServiceHTMLField("text", "Label", "opt1", "^[ -~]+$", "The label must not be left empty.");
		self::$module->addUserServiceHTMLField("select", "Template", "detail[template]", "^[ -~]+$", "The template must not be left empty.");
			if (is_array($this->templates)) {
				foreach ($this->templates as $cloud_id => $temp) {
					self::$module->addUserServiceHTMLSelectGroup("detail[template]", $temp['cloud']);
					for ($i=0; $i<count($temp['templates']); $i++)
						self::$module->addUserServiceHTMLSelectOption("detail[template]", $temp['templates'][$i]['name'], $temp['templates'][$i]['template']);
				}
			}

		// Add management tabs for Clients/Admins to manage services of this module type
		self::$module->addAction("Overview", array($this, "getOverview"));
		/*self::$module->addAction("Console", array($this, "getConsole"));*/
		self::$module->addAction("CPU Charts", array($this, "getCPUCharts"));
		self::$module->addAction("Bandwidth Charts", array($this, "getBandwidthCharts"));
	}

	/**
	 * Initialize an API request based on the given service information
	 *
	 * @param array $serviceInfo The service being examined
	 */
	private function initAction($serviceInfo) {
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = self::$module->getRows($conditions);
		$data = $data[0];
		
		$this->username = $data['username'];
		$this->key = $data['key'];
	}

	/**
	 * Create the Overview tab of the manage serive section
	 *
	 * @param array $serviceInfo The service being examined
	 * @return string HTML to send to output for this tab
	 */
	public function getOverview($serviceInfo) {
		$this->initAction($serviceInfo);
		
		$t = new Template(dirname(__FILE__) . "/templates/get_overview.pdt");
		
		if (isset($_GET['do']) && $serviceInfo['datep'] == "0000-00-00" && $serviceInfo['dated'] == "0000-00-00") {
			$message = "The action was sent";
			
			switch ($_GET['do']) {
				case "poweron":
					$this->powerOnVPS($serviceInfo['user2']);
					break;
				case "poweroff":
					$this->powerOffVPS($serviceInfo['user2']);
					break;
				case "reboot":
					$this->rebootVPS($serviceInfo['user2']);
					break;
				case "shutdown":
					$this->shutdownVPS($serviceInfo['user2']);
					break;
				case "edit":
					if (!empty($_POST)) {
						$vps = $this->getVPS($serviceInfo['user2']);

						$fields = self::$module->getUserServiceHTMLFields();
						foreach ($_POST as $key => $value) {
							for ($i=0; $i<count($fields); $i++) {
								if ($fields[$i]['dbFieldName'] == $key) {
									// test regex for validity
									if (!preg_match("/" . $fields[$i]['regex'] . "/", $value)) {
										$error = $fields[$i]['errorMessage'];
										break 2;
									}
									
									break; // on to the next field
								}
							}
						}
						
						if (!isset($error)) {
							$vars = array();
							$vars['label'] = $_POST['opt1'];
							$vars['slices_required'] = $vps['slices_count'];
							
							if ($this->updateVPS($serviceInfo['user2'], $vars)) {
								self::$module->updateService($serviceInfo['id'], $_POST);
								
								$message = "The information was successfully updated";
							}
							else
								$error = "The information could not be updated, please try again";
							
							// Update visually, all service fields
							foreach ($_POST as $key => $value) {
								if (isset($serviceInfo[$key]))
									$serviceInfo[$key] = $value;
							}
						}
					}
					break;
			}
			
			if ($error)
				$t->set("error", $error);
			elseif ($message)
				$t->set("message", $message);
		}
		
		$postto = preg_replace("/ajax=([^&]*)&?/i", "", $_SERVER['REQUEST_URI']);

		// Fetch the VPS if we haven't already
		if (!isset($vps))
			$vps = $this->getVPS($serviceInfo['user2']);
			
		$t->set("vps", $vps);
		
		$t->set("service", $serviceInfo);
		$t->set("postto", $postto);
		$t->set("template_dir", "/" . str_replace(ROOTWEBDIR, "", dirname(__FILE__) . "/templates"));
		
		return $t->fetch();
	}
	
	/**
	 * Create Console tab of the manage serive section
	 * 
	 * @param array $serviceInfo The service being examined
	 * @return string HTML to send to output for this tab
	 */
	public function getConsole($serviceInfo) {

		if (isset($_GET['framed'])) {
			/*
			require_once(ROOTWEBDIR . "/inc/modules/vpsnet/VPSNET.php");
			
			VPSDOTNET::getInstance("sales@blesta.com", "vGzMq6lYqmfInA==");

			$vm = new VirtualMachine();
			$vm->id = $serviceInfo['user2'];
			$vm->loadFully();
			ob_start();
			$vm->showConsole();
			$data = ob_get_clean();
			$data = preg_replace("/(src|href)=\"(.*\.(css|js))\"/i", "$1=\"$2?" . $_SERVER['QUERY_STRING'] . "\"", $data);
			echo $data;
			exit();
			*/
			
			
			$url_path = substr($_SERVER['PATH_INFO'], 1);

			$this->initAction($serviceInfo);
			if ($_SERVER['REQUEST_METHOD'] == 'POST') {
				$requestdata = http_build_query($_POST);
				
				$response = $this->call("POST", "virtual_machines/" . $serviceInfo['user2'] . "/console_proxy/" . $url_path, $requestdata, false);
				
				header('Content-type: ' . $response['info']['content-type']);
				
				echo $response['body'];
			}
			else {
				$response = $this->call("GET", "virtual_machines/" . $serviceInfo['user2'] . "/console_proxy/" . $url_path, false, false);
				
				if (strpos($url_path, '.css'))
					header('Content-type: text/css');
				else
					header("Content-type: " . $response['info']['content-type']);
					
				$response['body'] = preg_replace("/(src|href)=\"(.*\.(css|js))\"/i", "$1=\"$2?" . $_SERVER['QUERY_STRING'] . "\"", $response['body']);
				
				echo $response['body'];
			}
		
			exit();
		}

		$t = new Template(dirname(__FILE__) . "/templates/get_console.pdt");
		$t->set("service", $serviceInfo);
		return $t->fetch();
	}
	
	/**
	 * Create CPU Charts tab of the manage serive section
	 * 
	 * @param array $serviceInfo The service being examined
	 * @return string HTML to send to output for this tab
	 */
	public function getCPUCharts($serviceInfo) {
		$this->initAction($serviceInfo);

		if (isset($_GET['period']))
			$this->getChart($serviceInfo['user2'], "cpu", $_GET['period']);
		
		$t = new Template(dirname(__FILE__) . "/templates/get_cpu.pdt");
		$t->set("service", $serviceInfo);
		return $t->fetch();
	}
	
	/**
	 * Create Bandwidth Charts tab of the manage serive section
	 * 
	 * @param array $serviceInfo The service being examined
	 * @return string HTML to send to output for this tab
	 */
	public function getBandwidthCharts($serviceInfo) {
		$this->initAction($serviceInfo);

		if (isset($_GET['period']))
			$this->getChart($serviceInfo['user2'], "network", $_GET['period']);
		
		$t = new Template(dirname(__FILE__) . "/templates/get_bandwidth.pdt");
		$t->set("service", $serviceInfo);
		return $t->fetch();
	}
	
	/**
	 * Queries the API for the given graph type and time period and echos the
	 * image to the browser directly
	 *
	 * @param int $id Virtual Machine ID
	 * @param string $type The type of graph to fetch
	 * @param string $period The period to fetch for
	 */
	private function getChart($id, $type, $period) {
		$response = $this->call("GET", "virtual_machines/" . $id . "/" . $type . "_graph/?period=" . $period, false, false);
		
		$stauts = false;
		if (!empty($response['body']))
			$status = true;
		
		self::$module->logData(null, "output", $status);
		
		header('Content-type: ' . $response['info']['content-type']);
		header('Expires: ' . gmdate('D, d M Y H:i:s', time() + (2*3600) ) . ' GMT');
		header('Cache-Control: max-age=3600, public');

		echo $response['body'];
		exit();
	}


	/**
	 * Returns the module object, this function MUST be present
	 *
	 * @return Module the module this class initalizes and modifies
	 */
	public function getModule() {
		return self::$module;
	}

	/*******************************************************************************
	********************************************************************************
	* This is the MAGIC function. This function automatically creates a service    *
	* for a user.                                                                  *
	*                                                                              *
	* IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
	* NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
	*                                                                              *
	* IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
	* THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
	********************************************************************************
	*******************************************************************************/

	public function promodule($action, $serviceInfo, $packageInfo) {
		global $mysqldb;
		$this->cookie_file = settval($mysqldb, "tmpdir") . ".vpsnet.cookie";
		
		// packageInfo comes from packagesetinfos() function
		$nodes = (int)$packageInfo[4][2];
	
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = self::$module->getRows($conditions);
		$data = $data[0];
		
		$this->username = $data['username'];
		$this->key = $data['key'];
	
		switch ($action) {
			case "add":
				if (DEMO)
					return $serviceInfo;
				
				$cloud_id = 0;
				
				$temp = explode("|", $serviceInfo['detail']['template']);
				$cloud_id = $temp[0];
				$template_id = $temp[1];
				
				/*
				// Find the cloud ID to use
				if (is_array($this->templates)) {
					foreach ($this->templates as $cloud => $temp) {
						for ($i=0; $i<count($temp['templates']); $i++) {
							if ($temp['templates'][$i]['template'] == $serviceInfo['detail']['template']) {
								$cloud_id = $cloud;
								break;
							}
						}
					}
				}
				*/
				
				$vars = array();
				$vars['hostname'] = $serviceInfo['user1'];
				$vars['label'] = $serviceInfo['opt1'];
				$vars['slices'] = $nodes;
				$vars['template_id'] = $template_id;
				$vars['cloud_id'] = $cloud_id;
				$vars['user_id'] = $serviceInfo['uid'];

				$result = $this->addVPS($vars);
				
				if ($result) {
					$serviceInfo['user2'] = $result['virtual_machine']['id'];
					$serviceInfo['pass'] = $result['virtual_machine']['password'];
					$serviceInfo['opt2'] = $result['virtual_machine']['primary_ip_address'];
					return $serviceInfo;
				}
				return false;
				break;
			case "remove":
				if (DEMO)
					return true;
				return $this->removeVPS($serviceInfo['user2']);
					return true;
				break;
			case "suspend":
				if (DEMO)
					return true;
				return $this->shutdownVPS($serviceInfo['user2']);
				break;
			case "unsuspend":
				if (DEMO)
					return true;
				return $this->powerOnVPS($serviceInfo['user2']);
				break;
		}
	
		return false;
	}


	/* ------------------------------------------------------------------------------------------------
   The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	
	private function getTemplates() {
		static $templates = null;
		
		if ($templates == null) {
			// Fetch templates
			$data = self::$module->getRows();
			$data = $data[0];
			
			$templates = array();
			// If templates are stale, update them
			if ($data['templates_updated'] == null || time() > strtotime($data['templates_updated'] . " +1 day")) {
				$this->username = $data['username'];
				$this->key = $data['key'];
	
				$templates = $this->getCloudsAndTemplates($data['id']);
				
				// Parse templates for easy use later
				// Loop through each cloud
				$t = array();
				for ($i=0; $i<count($templates); $i++) {
					// Loop through each template
					if (isset($templates[$i]['cloud'])) {
						for ($j=0; $j<count($templates[$i]['cloud']['system_templates']); $j++) {
							$t[$templates[$i]['cloud']['id']]['cloud'] = $templates[$i]['cloud']['label'];
							$t[$templates[$i]['cloud']['id']]['templates'][$j]['template'] = $templates[$i]['cloud']['id'] . "|" . $templates[$i]['cloud']['system_templates'][$j]['id'];
							$t[$templates[$i]['cloud']['id']]['templates'][$j]['name'] = $templates[$i]['cloud']['system_templates'][$j]['label'];
						}
					}
				}
				
				$templates = $t;
				
				if (!empty($templates)) {
					self::$module->updateDBFields("templates", base64_encode(serialize($templates)), $data['id']);
					self::$module->updateDBFields("templates_updated", date("Y-m-d H:i:s"), $data['id']);
				}
				else
					$templates = unserialize(base64_decode($data['templates']));
			}
			else
				$templates = unserialize(base64_decode($data['templates']));
		}
		return $templates;
	}
	
	/**
	 * Used to unsuspend a VPS. Performs the power_on command.
	 *
	 * @param int $id The ID of the VPS to power on
	 */
	private function powerOnVPS($id) {
		$response = $this->call("POST", "virtual_machines/" . $id . "/power_on", false, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}
	
	private function powerOffVPS($id) {
		$response = $this->call("POST", "virtual_machines/" . $id . "/power_off", false, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}

	/**
	 * Used to suspend a VPS. Performs the shutdown command.
	 *
	 * @param int $id The ID of the VPS to shut down
	 */	
	private function shutdownVPS($id) {
		$response = $this->call("POST", "virtual_machines/" . $id . "/shutdown", false, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}
	
	/**
	 * Used to reboot a VPS.
	 *
	 * @param int $id The ID of the VPS to shut down
	 */
	private function rebootVPS($id) {
		$response = $this->call("POST", "virtual_machines/" . $id . "/reboot", false, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}
	
	/**
	 * Fetches the given VPS
	 *
	 * @param int $id The ID of the VPS to fetch
	 */
	private function getVPS($id) {
		$response = $this->call("GET", "virtual_machines/" . $id, false, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);

		return $response['body']['virtual_machine'];
	}

	/**
	 * Used to remove/cancel a VPS
	 *
	 * @param int $id The ID of the VPS to remove
	 */
	private function removeVPS($id) {
		$response = $this->call("DELETE", "virtual_machines/" . $id, false, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}

	
	private function addVPS($vars) {
	
		$free_nodes = $this->getFreeNodes();
		
		if ($free_nodes < $vars['slices']) {
			// Attempt to add enough nodes to add this VPS
			if (!$this->addNodes(array("quantity"=>$vars['slices']-$free_nodes)))
				return false; // add failed, can not add VPS at this time
		}
		
		$params = array('virtual_machine'=>array());
		$params['virtual_machine']['label'] = $vars['label'];
		$params['virtual_machine']['fqdn'] = $vars['hostname'];
		$params['virtual_machine']['slices_required'] = $vars['slices'];
		$params['virtual_machine']['backups_enabled'] = 0;
		$params['virtual_machine']['system_template_id'] = $vars['template_id'];
		$params['virtual_machine']['cloud_id'] = $vars['cloud_id'];
		//$params['virtual_machine']['consumer_id'] = $vars['user_id'];
		
		$response = $this->call("POST", "virtual_machines." . self::$version, $params);
		
		$status = true;
		if (empty($response['body']) || isset($response['body']['errors']))
			$status = false;
		
		self::$module->logData($response, "output", $status);
		
		if ($status)
			return $response['body'];
		return false;
	}
	
	private function updateVPS($id, $vars) {
		
		$params = array();
		$params['virtual_machine'] = $vars;
		
		$response = $this->call("PUT", "virtual_machines/" . $id, $params);
		
		$status = false;
		if ($response['info']['http_code'] == "200")
			$status = true;
		
		self::$module->logData($response, "output", $status);
		
		if ($status)
			return $response['body'];
		return false;
	}
	
	private function getFreeNodes() {
		
		$response = $this->call("GET", "nodes." . self::$version);
		
		self::$module->logData($response, "output", (!empty($response['body']) ? true : false));
		
		$count = 0;
		for ($i=0; $i<count($response['body']); $i++) {
			// If this slice (aka 'node') has no machine id assigned, it is free
			if ($response['body'][$i]['slice']['virtual_machine_id'] == "")
				$count++;
		}
		
		return $count;
	}
	
	private function addNodes($params) {
		
		$response = $this->call("POST", "nodes." . self::$version, $params, true);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}
	
	private function getCloudsAndTemplates() {
		$response = $this->call("GET", "available_clouds", false, false);
		
		$status = ($response['info']['http_code'] == "200" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $response['body'];
	}
	
	private function call($method="GET", $action, $params=false, $headerResponse=false) {
		self::$module->logData($method . " " . self::$url . $action . "\n" . ($params ? print_r($params, true) : ""), "input", true);
		
		$headers = array('Content-Type: application/json', 'Accept: application/json');
		
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, self::$url . $action);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($ch, CURLOPT_USERPWD, $this->username . ":" . $this->key);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($ch, CURLOPT_USERAGENT, "VPSNET_API_10_JSON/PHP");
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_COOKIEFILE, $this->cookie_file);
	    curl_setopt($ch, CURLOPT_COOKIEJAR, $this->cookie_file);
		
		if ($params) {
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
			curl_setopt($ch, CURLOPT_POST, true);
		}
		else
			curl_setopt($ch, CURLOPT_HTTPGET, true);
		
		$response = array();
		$response['body'] = curl_exec($ch);
		$response['info'] = curl_getinfo($ch);

		curl_close($ch);
		
		if ($response['info']['content_type'] == "application/json; charset=utf-8")
			$response['body'] = json_decode($response['body'], true);
		
		return $response;
	}
}
?>