<?php
/*******************************************************************************
 @author Cody Phillips
 @date Jun 18, 2008
 @description Domain Order Form Template Plug-in.
 
 DO NOT EDIT THIS FILE. THIS FILE IS NOT INTENDED TO BE MODIFIED.
*******************************************************************************/
include_once(ROOTWEBDIR . "order_forms_inc/whois.class.php");

class domain extends AbstractOrder {
	private static $templateName = "Domain Only";
	private static $whois;
	private $packages = array();
	private $package = array();
	private $packageFields = array();
	private $settings = array();
	private $user = array();
	private $customSession = array();
	private $tlds = array();
	private $miscfields = array();
	private $misc_settings = array();

	public function __construct($misc_settings=false) {
		self::$whois = new whois();
		$this->misc_settings = $misc_settings;
		
		// Get TLDs
		$tld_list = ROOTWEBDIR . "order_forms_inc/tlds.txt";
		if (file_exists($tld_list)) {
		   $tlds = explode("\n", file_get_contents($tld_list));
		
			for ($i=0, $j=0; $i<count($tlds); $i++) {
				if (strlen(trim($tlds[$i])) > 2)
					$this->tlds[$j++] = trim($tlds[$i]);
			}
		}
		
		// Build all misc setting HTML fields (all fields must be array elements named misc[])
		$tld_html = $this->getTLDHTML($misc_settings['tlds']);
		
		$this->miscfields = array(array("name" => _ORDERFORM_MISC_TLDS, "html" => $tld_html));
	}

	public function __destruct() {
	}
         
	// Required method.  Fetches HTML for custom field settings
	public function getMiscFields() {
		return $this->miscfields;
	}

	// Required method. Fetches the name of this template.        
	public function getTemplateName() {
		return self::$templateName;
	}

	// Required method. Used to perform an action when a package is added to the cart.         
	public function addedToCart($package) {

	}

	// Required method. Sets fields to be sent to the order form template file.
	public function getVars($step, $postdata) {
		$this->packages = $this->getPackages();
		$this->settings = $this->getSettings();
		$this->package = $this->getPackage();
		$this->packageFields = $this->getPackageFields();
		$this->user = $this->getUser();
		$this->customSession = $this->getCustomSession();

		$vars = array();
		switch ($step) {
			case 1:
				$postdata['domain'] = strtok($postdata['domain'], ".");
				
				$vars['showDomainChecker'] = true;
				if (isset($postdata['registerdomain']) && $postdata['registerdomain'] == "false") {
					$vars['showDomainChecker'] = false;
					$this->setSession("domain", $postdata['domain']);
				}
				
				if (isset($this->customSession['lastorder'])) {
					if ($this->customSession['lastorder'] == $this->settings['domainid'])
						$vars['showDomainChecker'] = false;
					$this->unsetSession("lastorder");
				}
				$vars['domainresults']['domain'] = $postdata['domain'] . $postdata['tld'];
				$vars['domainresults']['available'] = false;
				
				if (isset($postdata['check'])) {
					if (self::$whois->lookup($postdata['domain'], $postdata['tld']))
						$vars['domainresults']['available'] = true;
					
					
					$vars['showDomainResults'] = true;                                                 
					$vars['postvars'] = $postdata;
				}
				
				for ($i=0; $i<count($this->packages); $i++) {
					if ($i==0) {
						$vars['domainPackage'] = $this->packages[$i];
						array_splice($this->packages, $i, 1); // we only want to return non domain registration packages
						break;
					}
				}
				$vars['packages'] = $this->packages;
				$vars['tlds'] = $this->misc_settings['tlds'];                                
				break;
			case 2:
				for ($i=0; $i<count($this->packageFields); $i++) {
					if ($this->packageFields[$i]['dbFieldName'] == "user1") {
						$this->packageFields[$i]['type'] = "hidden";
						
						if (isset($postdata['domain'])) {
							$this->packageFields[$i]['value'] = $postdata['domain'];
							$this->setSession("domain", $postdata['domain']);
						}
						
						$vars['domain'] = $this->packageFields[$i]['value'];
					}
					elseif (isset($this->customSession['domain']) && $this->packageFields[$i]['dbFieldName'] == "user1") {
						$this->packageFields[$i]['value'] = $this->customSession['domain'];
						$this->unsetSession("domain");
					}                        
					
					// Set user fields where available                 
					if (isset($this->user)) {
						if ($this->packageFields[$i]['dbFieldName'] == "detail[email]")
							$this->packageFields[$i]['value'] = $this->user['email'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[fname]")
							$this->packageFields[$i]['value'] = $this->user['fname'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[lname]")
							$this->packageFields[$i]['value'] = $this->user['lname'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[address1]")
							$this->packageFields[$i]['value'] = $this->user['address'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[address2]")
							$this->packageFields[$i]['value'] = $this->user['address2'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[city]")
							$this->packageFields[$i]['value'] = $this->user['city'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[state]")
							$this->packageFields[$i]['value'] = $this->user['state'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[country]")
							$this->packageFields[$i]['value'] = $this->user['country'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[phone]")
							$this->packageFields[$i]['value'] = $this->user['phone'];
						elseif ($this->packageFields[$i]['dbFieldName'] == "detail[fax]")
							$this->packageFields[$i]['value'] = $this->user['fax'];
					}
				}
				$vars['package'] = $this->package;
				$vars['packageFields'] = $this->packageFields;
				break;
			case 3:
				break;
			case 4:
				break;
			case 5:
				break;
			case 6:
				break;
		}
		return $vars;
	}

	private function getTLDHTML($selectedtlds) {
		$tld_html = "";
		for ($i=0; $i<count($this->tlds); $i++) {
			$selected = false;
			for ($j=0; $j<count($selectedtlds); $j++) {
				if ($this->tlds[$i] == $selectedtlds[$j]) {
					$selected = true;
					break;
				}
			}
			$tld_html .= "<input type=\"checkbox\" name=\"misc[tlds][]\" value=\"" . $this->tlds[$i] . "\"" . ($selected ? " checked" : "") . " /> " . $this->tlds[$i] . " ";	
		}
		
		return $tld_html;
	}
}
?>